package demo.barcharts.category ;

import java.awt.Adjustable ;
import java.awt.BorderLayout ;
import java.awt.Color ;
import java.awt.GradientPaint ;

import javax.swing.BorderFactory ;
import javax.swing.JPanel ;
import javax.swing.JScrollBar ;
import javax.swing.event.ChangeEvent ;
import javax.swing.event.ChangeListener ;

import com.ijchart.xychart.axis.category.CategoryAxis ;
import com.ijchart.xychart.axis.value.NumberValueAxis ;
import com.ijchart.xychart.client.ChartFactory ;
import com.ijchart.xychart.client.ChartPanel ;
import com.ijchart.xychart.client.IJChart ;
import com.ijchart.xychart.constant.ChartOrientation ;
import com.ijchart.xychart.constant.DatasetSlidingType ;
import com.ijchart.xychart.constant.GradientPaintTransformType ;
import com.ijchart.xychart.data.NumberChartData ;
import com.ijchart.xychart.dataset.DefaultMatrixDataset ;
import com.ijchart.xychart.dataset.IMatrixDataset ;
import com.ijchart.xychart.dataset.SlidingDataset ;
import com.ijchart.xychart.plot.CategoryPlot ;
import com.ijchart.xychart.render.category.Bar2DRender ;
import com.ijchart.xychart.tickunit.LongTickUnit ;

/**
 * <p>Title: IJChart</p>
 *
 * <p>Description: a chart library for the Java(tm) platform.</p>
 *
 * <p>Copyright: Copyright (c) 2013</p>
 *
 * <p>Company: </p>
 *
 * @author zhang feng min,shanghai china.
 * <a href="mailto:ijchartsh@gmail.com">Contract me by email.</a>
 * @version 1.0
 */
public class SlidingCategoryDatasetDemo1 extends JPanel implements ChangeListener {
  private ChartOrientation orientation = ChartOrientation.HORIZONTAL ;

  private SlidingDataset slidingDataset ;

  private double[] datas ;

  private JScrollBar scroller ;

  public SlidingCategoryDatasetDemo1() {
    super(new BorderLayout()) ;
    initDatas() ;
    addPanel() ;

    scroller = new JScrollBar(Adjustable.VERTICAL, 0, 20, 0, 50) ;
    scroller.getModel().addChangeListener(this) ;
    JPanel jpanel = new JPanel(new BorderLayout()) ;
    jpanel.add(scroller) ;
    jpanel.setBorder(BorderFactory.createEmptyBorder(30, 2, 2, 2)) ;
    jpanel.setBackground(Color.white) ;
    add(jpanel, BorderLayout.EAST) ;
  }

  private void addPanel() {
    final IMatrixDataset dataset = createDataset() ;
    slidingDataset = new SlidingDataset(dataset, DatasetSlidingType.ROW, 0, 20) ;
    final IJChart chart = ChartFactory.createCategoryBar2DChart(
      "SlidingCategoryDatasetDemo1", "Series", "Value", slidingDataset, orientation, false, true) ;

    CategoryPlot plot = (CategoryPlot) chart.getPlot() ;
    CategoryAxis categoryaxis = plot.getCategoryAxis() ;
    categoryaxis.setMaximumCategoryLabelWidthRatio(0.8F) ;
    categoryaxis.setLowerMargin(0.02D) ;
    categoryaxis.setUpperMargin(0.02D) ;
    NumberValueAxis numberaxis = (NumberValueAxis) plot.getValueAxis() ;
    numberaxis.setTickUnit(new LongTickUnit()) ;
    numberaxis.setUserDefineDatasetRange(0, 100) ;

    Bar2DRender render = (Bar2DRender) plot.getRender() ;
    GradientPaint gradientpaint = new GradientPaint(0.0F, 0.0F, Color.blue, 0.0F, 0.0F,
                                                    new Color(0, 0, 64)) ;
    render.setColumnItemFillPaint(0, gradientpaint) ;
    render.setAutoSelectFillPaint(false) ;
    render.setBaseItemFillPaintTransformer(GradientPaintTransformType.VERTICAL) ;

    ChartPanel chartpanel = new ChartPanel(chart) ;
    add(chartpanel) ;
  }

  public void stateChanged(ChangeEvent changeevent) {
    slidingDataset.setStartIndex(scroller.getValue()) ;
  }

  private void initDatas() {
    this.datas = new double[50] ;

    for (int i = 0 ; i < 50 ; i++) {
      datas[i] = Math.random() * 100D ;
    }
  }

  private static IMatrixDataset createDataset() {
    DefaultMatrixDataset dataset = new DefaultMatrixDataset() ;
    dataset.addColKey("S1") ;

    for (int i = 0 ; i < 50 ; i++) {
      dataset.addRowKey("Series " + i) ;
      dataset.addRowData() ;
      dataset.addData(i, new NumberChartData(Math.random() * 100D)) ;
    }

    return dataset ;
  }

}
