package demo.barcharts.category ;

import java.awt.GridLayout ;
import java.text.DecimalFormat ;
import java.text.NumberFormat ;

import javax.swing.JPanel ;

import com.ijchart.xychart.axis.value.WaterfallNumberValueAxis ;
import com.ijchart.xychart.client.ChartFactory ;
import com.ijchart.xychart.client.ChartPanel ;
import com.ijchart.xychart.client.IJChart ;
import com.ijchart.xychart.constant.ChartOrientation ;
import com.ijchart.xychart.data.NumberChartData ;
import com.ijchart.xychart.dataset.DefaultMatrixDataset ;
import com.ijchart.xychart.dataset.IMatrixDataset ;
import com.ijchart.xychart.label.DefaultLabelGenerator ;
import com.ijchart.xychart.plot.CategoryPlot ;
import com.ijchart.xychart.render.category.WaterfallBarRender ;
import com.ijchart.xychart.tickunit.LongTickUnit ;
import com.ijchart.xychart.tooltip.DefaultTooltipGenerator ;

/**
 * <p>Title: IJChart</p>
 *
 * <p>Description: a chart library for the Java(tm) platform.</p>
 *
 * <p>Copyright: Copyright (c) 2013</p>
 *
 * <p>Company: </p>
 *
 * @author zhang feng min,shanghai china.
 * <a href="mailto:ijchartsh@gmail.com">Contract me by email.</a>
 * @version 1.0
 */
public class WaterfallChartDemo1 extends JPanel {

  private ChartOrientation vertical = ChartOrientation.VERTICAL ;

  private DecimalFormat labelFormat = (DecimalFormat) NumberFormat.getNumberInstance() ;

  private DecimalFormat tooltipFormat = (DecimalFormat) NumberFormat.getNumberInstance() ;

  public WaterfallChartDemo1() {
    labelFormat.applyPattern("$##,###.00") ;
    tooltipFormat.applyPattern("$##,###.00") ;

    setLayout(new GridLayout(1, 1)) ;
    addPanel1() ;
  }

  private void addPanel1() {
    IMatrixDataset dataset = createDataset() ;
    IJChart chart = ChartFactory.createWaterfallBarChart(
      "Product Cost Breakdown", "Expense Category", "Cost Per Unit", dataset, vertical, true, true) ;

    CategoryPlot categoryplot = (CategoryPlot) chart.getPlot() ;
    categoryplot.setDomainGridlineVisible(false) ;
    WaterfallNumberValueAxis valueaxis = (WaterfallNumberValueAxis) categoryplot.getValueAxis() ;
    valueaxis.setTickUnit(new LongTickUnit()) ;

    WaterfallBarRender render = (WaterfallBarRender) categoryplot.getRender() ;
    render.setBaseValue(5D) ;
    render.setBaseItemLabelVisible(true) ;
    render.setLabelGenerator(new DefaultLabelGenerator() {
      public String generateLabel(IMatrixDataset dataset, int row, int column) {
        NumberChartData data = (NumberChartData) dataset.getData(row, column) ;
        return labelFormat.format(data.getValue()) ;
      }
    }) ;

    render.setToolTipGenerator(new DefaultTooltipGenerator() {
      public String generateToolTip(IMatrixDataset dataset, int row, int column) {
        Object rowKey = dataset.getRowKey(row) ;
        Object colKey = dataset.getColKey(column) ;
        NumberChartData data = (NumberChartData) dataset.getData(row, column) ;
        return rowKey.toString() + ", " + colKey + ") = " + tooltipFormat.format(data.getValue()) ;
      }
    }) ;

    ChartPanel chartpanel = new ChartPanel(chart) ;
    add(chartpanel) ;
  }

  private static IMatrixDataset createDataset() {
    DefaultMatrixDataset dataset = new DefaultMatrixDataset() ;

    dataset.addRowKey("Product 1") ;
    dataset.addColKey("Labour") ;
    dataset.addColKey("Administration") ;
    dataset.addColKey("Marketing") ;
    dataset.addColKey("Distribution") ;
    dataset.addColKey("Total Expense") ;

    dataset.addRowData() ;
    dataset.addData(0, new NumberChartData(15.76D)) ;
    dataset.addData(0, new NumberChartData(8.6600000000000001D)) ;
    dataset.addData(0, new NumberChartData(4.71D)) ;
    dataset.addData(0, new NumberChartData(3.5099999999999998D)) ;
    dataset.addData(0, new NumberChartData(32.640000000000001D)) ;

    return dataset ;
  }
}
