package demo.combinecharts;

import java.awt.BorderLayout ;

import javax.swing.JPanel ;

import com.ijchart.xychart.axis.value.IntervalXNumberYBaseValueAxis ;
import com.ijchart.xychart.axis.value.XYNumberValueAxis ;
import com.ijchart.xychart.client.ChartPanel ;
import com.ijchart.xychart.client.IJChart ;
import com.ijchart.xychart.constant.AxisLabelType ;
import com.ijchart.xychart.constant.ChartOrientation ;
import com.ijchart.xychart.constant.DateTickUnitType ;
import com.ijchart.xychart.constant.IAngle ;
import com.ijchart.xychart.constant.XYValueAxisType ;
import com.ijchart.xychart.data.IntervalXNumberYBaseChartData ;
import com.ijchart.xychart.data.XYNumberChartData ;
import com.ijchart.xychart.dataset.DefaultMatrixDataset ;
import com.ijchart.xychart.dataset.IMatrixDataset ;
import com.ijchart.xychart.plot.XYPlot ;
import com.ijchart.xychart.plot.combine.CombineValueAxisXYPlot ;
import com.ijchart.xychart.render.xy.IntervalXNumberYBaseBar2DRender ;
import com.ijchart.xychart.render.xy.XYAreaRender ;
import com.ijchart.xychart.render.xy.XYLineAndShapeRender ;
import com.ijchart.xychart.tickunit.DateTickUnit ;
import com.ijchart.xychart.util.DateUtil ;

/**
 * <p>Title: IJChart</p>
 *
 * <p>Description: a chart library for the Java(tm) platform.</p>
 *
 * <p>Copyright: Copyright (c) 2013</p>
 *
 * <p>Company: </p>
 *
 * @author zhang feng min,shanghai china.
 * <a href="mailto:ijchartsh@gmail.com">Contract me by email.</a>
 * @version 1.0
 */
public class CombinedTimeSeriesDemo1 extends JPanel {

  public CombinedTimeSeriesDemo1() {
    setLayout(new BorderLayout()) ;
    addPanel1() ;
  }

  private void addPanel1() {
    ChartOrientation orientation = ChartOrientation.VERTICAL ;

    IMatrixDataset dataset1 = createDataset1() ;
    IMatrixDataset dataset2 = createDataset2() ;
    IMatrixDataset dataset3 = createDataset3() ;

    IntervalXNumberYBaseValueAxis xAxis1 =
      new IntervalXNumberYBaseValueAxis("Date", dataset1, XYValueAxisType.XY_VALUE_AXIS_X, true) ;
    xAxis1.setAxisLabelType(AxisLabelType.COLUMN) ;
    xAxis1.setTickUnit(new DateTickUnit(DateTickUnitType.DATE_UNIT_YEAR)) ;

    IntervalXNumberYBaseValueAxis yAxis1 =
      new IntervalXNumberYBaseValueAxis("Value", dataset1, XYValueAxisType.XY_VALUE_AXIS_Y, false) ;
    yAxis1.setValueLineVisible(true) ;

    IntervalXNumberYBaseBar2DRender render1 = new IntervalXNumberYBaseBar2DRender(dataset1) ;
    if (orientation.equals(ChartOrientation.HORIZONTAL)) {
      xAxis1.setAxisLabelAngle(IAngle.ANGLE_UP_90) ;
    } else if (orientation.equals(ChartOrientation.VERTICAL)) {
      yAxis1.setAxisLabelAngle(IAngle.ANGLE_UP_90) ;
    }

    XYPlot plot1 = new XYPlot(xAxis1, yAxis1, render1) ;
    plot1.setOrientation(orientation) ;

    XYNumberValueAxis xAxis2 = new XYNumberValueAxis("Date2", dataset2,
                                                     XYValueAxisType.XY_VALUE_AXIS_X, true) ;
    xAxis2.setAxisLabelType(AxisLabelType.COLUMN) ;
    xAxis2.setAxisRangeIncludeZero(false) ;
//    xAxis2.setNumberTickUnit(new DateTickUnit(DateTickUnitType.DATE_UNIT_MONTH)) ;

    XYNumberValueAxis yAxis2 = new XYNumberValueAxis(null, dataset2,
                                                     XYValueAxisType.XY_VALUE_AXIS_Y, false) ;
    yAxis2.setValueLineVisible(true) ;

    if (orientation.equals(ChartOrientation.HORIZONTAL)) {
      xAxis2.setAxisLabelAngle(IAngle.ANGLE_UP_90) ;
    } else if (orientation.equals(ChartOrientation.VERTICAL)) {
      yAxis2.setAxisLabelAngle(IAngle.ANGLE_UP_90) ;
    }
    XYAreaRender render = new XYAreaRender(dataset2) ;
    XYPlot plot2 = new XYPlot(xAxis2, yAxis2, render) ;
    plot2.setOrientation(orientation) ;
    plot2.setForegroundAlpha(0.5f) ;

    XYNumberValueAxis yAxis3 = new XYNumberValueAxis(null, dataset3,
                                                     XYValueAxisType.XY_VALUE_AXIS_Y, false) ;
    yAxis2.setValueLineVisible(true) ;

    XYLineAndShapeRender render3 = new XYLineAndShapeRender(dataset3) ;
    if (orientation.equals(ChartOrientation.HORIZONTAL)) {
      xAxis2.setAxisLabelAngle(IAngle.ANGLE_UP_90) ;
    } else if (orientation.equals(ChartOrientation.VERTICAL)) {
      yAxis2.setAxisLabelAngle(IAngle.ANGLE_UP_90) ;
    }
    plot2.setValueAxis(1, yAxis3) ;
    plot2.setRender(1, render3) ;

    CombineValueAxisXYPlot combinePlot = new CombineValueAxisXYPlot() ;
    combinePlot.setPlotGap(15D) ;
    combinePlot.addSubPlot(plot1, 1) ;
    combinePlot.addSubPlot(plot2, 3) ;

    IJChart chart = new IJChart("Sample Combined Plot", IJChart.DEFAULT_TITLE_FONT,
                                combinePlot, true) ;

    ChartPanel chartpanel = new ChartPanel(chart) ;
    add(chartpanel) ;
  }

  private DefaultMatrixDataset createDataset1() {
    DefaultMatrixDataset dataset = new DefaultMatrixDataset() ;
    dataset.addRowKey("Annual") ;
    dataset.addRowData() ;

    dataset.addData(0, createYearData(1998, 80D)) ;
    dataset.addData(0, createYearData(1999, 85D)) ;
    dataset.addData(0, createYearData(2000, 87.599999999999994D)) ;

    return dataset ;
  }

  private DefaultMatrixDataset createDataset2() {
    DefaultMatrixDataset dataset = new DefaultMatrixDataset() ;
    dataset.addRowKey("Monthly A") ;
    dataset.addRowKey("Monthly B") ;
    dataset.addRowData(2) ;

    dataset.addData(0, createMonthData(7, 2000, 85.799999999999997D)) ;
    dataset.addData(0, createMonthData(8, 2000, 85.799999999999997D)) ;
    dataset.addData(0, createMonthData(9, 2000, 85.799999999999997D)) ;
    dataset.addData(0, createMonthData(10, 2000, 86.5D)) ;
    dataset.addData(0, createMonthData(11, 2000, 86.5D)) ;
    dataset.addData(0, createMonthData(12, 2000, 86.5D)) ;
    dataset.addData(0, createMonthData(1, 2001, 87.700000000000003D)) ;
    dataset.addData(0, createMonthData(2, 2001, 87.700000000000003D)) ;
    dataset.addData(0, createMonthData(3, 2001, 87.700000000000003D)) ;
    dataset.addData(0, createMonthData(4, 2001, 88.5D)) ;
    dataset.addData(0, createMonthData(5, 2001, 88.5D)) ;
    dataset.addData(0, createMonthData(6, 2001, 88.5D)) ;
    dataset.addData(0, createMonthData(7, 2001, 90D)) ;
    dataset.addData(0, createMonthData(8, 2001, 90D)) ;
    dataset.addData(0, createMonthData(9, 2001, 90D)) ;
    dataset.addData(0, createMonthData(10, 2001, 90D)) ;
    dataset.addData(0, createMonthData(11, 2001, 90D)) ;
    dataset.addData(0, createMonthData(12, 2001, 90D)) ;
    dataset.addData(0, createMonthData(1, 2002, 90D)) ;
    dataset.addData(0, createMonthData(2, 2002, 90D)) ;
    dataset.addData(0, createMonthData(3, 2002, 90D)) ;
    dataset.addData(0, createMonthData(4, 2002, 90D)) ;
    dataset.addData(0, createMonthData(5, 2002, 90D)) ;
    dataset.addData(0, createMonthData(6, 2002, 90D)) ;

    dataset.addData(1, createMonthData(7, 2000, 83.799999999999997D)) ;
    dataset.addData(1, createMonthData(8, 2000, 83.799999999999997D)) ;
    dataset.addData(1, createMonthData(9, 2000, 83.799999999999997D)) ;
    dataset.addData(1, createMonthData(10, 2000, 73.799999999999997D)) ;
    dataset.addData(1, createMonthData(11, 2000, 84.5D)) ;
    dataset.addData(1, createMonthData(12, 2000, 84.5D)) ;
    dataset.addData(1, createMonthData(1, 2001, 85.700000000000003D)) ;
    dataset.addData(1, createMonthData(2, 2001, 85.700000000000003D)) ;
    dataset.addData(1, createMonthData(3, 2001, 85.700000000000003D)) ;
    dataset.addData(1, createMonthData(4, 2001, 86.5D)) ;
    dataset.addData(1, createMonthData(5, 2001, 86.5D)) ;
    dataset.addData(1, createMonthData(6, 2001, 86.5D)) ;
    dataset.addData(1, createMonthData(7, 2001, 88D)) ;
    dataset.addData(1, createMonthData(8, 2001, 88D)) ;
    dataset.addData(1, createMonthData(9, 2001, 88D)) ;
    dataset.addData(1, createMonthData(10, 2001, 88D)) ;
    dataset.addData(1, createMonthData(11, 2001, 88D)) ;
    dataset.addData(1, createMonthData(12, 2001, 88D)) ;
    dataset.addData(1, createMonthData(1, 2002, 88D)) ;
    dataset.addData(1, createMonthData(2, 2002, 88D)) ;
    dataset.addData(1, createMonthData(3, 2002, 88D)) ;
    dataset.addData(1, createMonthData(4, 2002, 88D)) ;
    dataset.addData(1, createMonthData(5, 2002, 88D)) ;
    dataset.addData(1, createMonthData(6, 2002, 88D)) ;

    return dataset ;
  }

  private DefaultMatrixDataset createDataset3() {
    DefaultMatrixDataset dataset = new DefaultMatrixDataset() ;
    dataset.addRowKey("XXX") ;
    dataset.addRowData() ;

    dataset.addData(0, createMonthData(7, 2000, 81.5D)) ;
    dataset.addData(0, createMonthData(8, 2000, 86D)) ;
    dataset.addData(0, createMonthData(9, 2000, 82D)) ;
    dataset.addData(0, createMonthData(10, 2000, 89.5D)) ;
    dataset.addData(0, createMonthData(11, 2000, 88D)) ;
    dataset.addData(0, createMonthData(12, 2000, 88D)) ;
    dataset.addData(0, createMonthData(1, 2001, 90D)) ;
    dataset.addData(0, createMonthData(2, 2001, 89.5D)) ;
    dataset.addData(0, createMonthData(3, 2001, 90.200000000000003D)) ;
    dataset.addData(0, createMonthData(4, 2001, 90.599999999999994D)) ;
    dataset.addData(0, createMonthData(5, 2001, 87.5D)) ;
    dataset.addData(0, createMonthData(6, 2001, 91D)) ;
    dataset.addData(0, createMonthData(7, 2001, 89.700000000000003D)) ;
    dataset.addData(0, createMonthData(8, 2001, 87D)) ;
    dataset.addData(0, createMonthData(9, 2001, 91.200000000000003D)) ;
    dataset.addData(0, createMonthData(10, 2001, 84D)) ;
    dataset.addData(0, createMonthData(11, 2001, 90D)) ;
    dataset.addData(0, createMonthData(12, 2001, 92D)) ;

    return dataset ;
  }

  private IntervalXNumberYBaseChartData createYearData(int year, double value) {
    long mills = DateUtil.dateToTimeInMillis(year - 1, 10) ;
    long halfYear = DateUtil.dateToTimeInMillis(year, 3) ;
    return new IntervalXNumberYBaseChartData(mills, halfYear, value) ;
  }

  private XYNumberChartData createMonthData(int month, int year, double value) {
    long mills = DateUtil.dateToTimeInMillis(year, month) ;
    return new XYNumberChartData(mills, value) ;
  }

}
