package demo.granttcharts ;

import java.awt.BorderLayout ;
import java.util.Calendar ;

import javax.swing.BorderFactory ;
import javax.swing.JPanel ;
import javax.swing.JScrollBar ;
import javax.swing.event.ChangeEvent ;
import javax.swing.event.ChangeListener ;

import com.ijchart.xychart.client.ChartFactory ;
import com.ijchart.xychart.client.ChartPanel ;
import com.ijchart.xychart.client.IJChart ;
import com.ijchart.xychart.constant.ChartOrientation ;
import com.ijchart.xychart.constant.DatasetSlidingType ;
import com.ijchart.xychart.data.GranttChartData ;
import com.ijchart.xychart.dataset.DefaultMatrixDataset ;
import com.ijchart.xychart.dataset.IMatrixDataset ;
import com.ijchart.xychart.dataset.SlidingDataset ;
import com.ijchart.xychart.plot.CategoryPlot ;

/**
 * <p>Title: IJChart</p>
 *
 * <p>Description: a chart library for the Java(tm) platform.</p>
 *
 * <p>Copyright: Copyright (c) 2013</p>
 *
 * <p>Company: </p>
 *
 * @author zhang feng min,shanghai china.
 * <a href="mailto:ijchartsh@gmail.com">Contract me by email.</a>
 * @version 1.0
 */
public class SlidingGanttDatasetDemo1 extends JPanel implements ChangeListener {

  private ChartOrientation orientation = ChartOrientation.HORIZONTAL ;

  private JScrollBar scroller ;

  private SlidingDataset sliding ;

  public SlidingGanttDatasetDemo1() {
    setLayout(new BorderLayout()) ;
    addPanel() ;

    scroller = new JScrollBar(1, 0, 15, 0, 50) ;
    scroller.getModel().addChangeListener(this) ;
    JPanel jpanel = new JPanel(new BorderLayout()) ;
    jpanel.add(scroller) ;
    jpanel.setBorder(BorderFactory.createEmptyBorder(30, 2, 2, 2)) ;
    add(jpanel, BorderLayout.EAST) ;

  }

  private void addPanel() {
    IMatrixDataset dataset = createDataset() ;
    sliding = new SlidingDataset(dataset, DatasetSlidingType.ROW, 0, 15) ;
    IJChart chart = ChartFactory.createGranttChart("Sliding Gantt Chart Demo", "Task", "Date",
                                                   sliding, orientation, true, true) ;

    CategoryPlot plot = (CategoryPlot) chart.getPlot() ;
    plot.setDomainGridlineVisible(false) ;
    plot.getCategoryAxis().setMaximumCategoryLabelWidthRatio(10F) ;

    ChartPanel chartpanel = new ChartPanel(chart) ;
    add(chartpanel) ;
  }

  public void stateChanged(ChangeEvent changeevent) {
    sliding.setStartIndex(scroller.getValue()) ;
  }

  private static IMatrixDataset createDataset() {
    DefaultMatrixDataset dataset = new DefaultMatrixDataset() ;
    dataset.addColKey("Scheduled") ;

    Calendar cal = Calendar.getInstance() ;
    long t1 = cal.getTimeInMillis() ;
    long t2 = 0 ;
    for (int i = 0 ; i < 50 ; i++) {
      dataset.addRowKey("Task " + i) ;
      int j = (int) (Math.random() * 10D) + 1 ;
      cal.add(Calendar.DAY_OF_MONTH, j) ;
      t2 = cal.getTimeInMillis() ;
      dataset.addRowData() ;
      dataset.addData(i, new GranttChartData(t1, t2, new Double(0))) ;

      t1 = t2 ;
    }

    return dataset ;
  }

}
