package demo.linecharts ;

import java.awt.BasicStroke ;
import java.awt.BorderLayout ;
import java.awt.Color ;
import java.awt.Font ;
import java.awt.Shape ;
import java.awt.geom.Ellipse2D ;

import javax.swing.JPanel ;

import com.ijchart.xychart.axis.value.NumberValueAxis ;
import com.ijchart.xychart.client.ChartFactory ;
import com.ijchart.xychart.client.ChartPanel ;
import com.ijchart.xychart.client.IJChart ;
import com.ijchart.xychart.constant.ChartOrientation ;
import com.ijchart.xychart.constant.HorizontalAlignment ;
import com.ijchart.xychart.data.NumberChartData ;
import com.ijchart.xychart.dataset.DefaultMatrixDataset ;
import com.ijchart.xychart.plot.CategoryPlot ;
import com.ijchart.xychart.render.category.LineAndShapeRender ;
import com.ijchart.xychart.tickunit.LongTickUnit ;
import com.ijchart.xychart.title.TextTitle ;
import com.ijchart.xychart.ui.RectangleEdge ;
import com.ijchart.xychart.ui.supplier.DefaultDrawingSupplier ;

/**
 * <p>Title: IJChart</p>
 *
 * <p>Description: a chart library for the Java(tm) platform.</p>
 *
 * <p>Copyright: Copyright (c) 2013</p>
 *
 * <p>Company: </p>
 *
 * @author zhang feng min,shanghai china.
 * <a href="mailto:ijchartsh@gmail.com">Contract me by email.</a>
 * @version 1.0
 */
public class LineChartDemo1 extends JPanel {
  private ChartOrientation orientation = ChartOrientation.VERTICAL ;

  public LineChartDemo1() {
    setLayout(new BorderLayout()) ;
    addPanel1() ;
  }

  private void addPanel1() {
    DefaultMatrixDataset dataset = createDataset() ;
    IJChart chart = ChartFactory.createCategoryLineAndShapeChart("Java Standard Class Library",
                                                                 null, "Class Count", dataset,
                                                                 orientation, false, true) ;
    chart.addSubtitle(new TextTitle("Number of Classes By Release")) ;
    TextTitle texttitle = new TextTitle(
      "Source: Java In A Nutshell (5th Edition) by David Flanagan (O'Reilly)") ;
    texttitle.setTextFont(new Font("SansSerif", Font.BOLD, 10)) ;
    texttitle.setPosition(RectangleEdge.BOTTOM) ;
    texttitle.setHorizontalAlignment(HorizontalAlignment.RIGHT) ;
    chart.addSubtitle(texttitle) ;

    CategoryPlot plot = (CategoryPlot) chart.getPlot() ;
    plot.setValueGridlineVisible(false) ;
    plot.setDomainGridlineVisible(false) ;

    NumberValueAxis numberaxis = (NumberValueAxis) plot.getValueAxis() ;
    numberaxis.setTickUnit(new LongTickUnit()) ;

    LineAndShapeRender render = (LineAndShapeRender) plot.getRender() ;
    render.setAutoSelectFillPaint(false) ;
    render.setBaseItemFillPaint(Color.white) ;
    render.setAutoSelectBorderStroke(false) ;
    render.setRowItemBorderStroke(0, new BasicStroke(3F)) ;

    plot.setDrawingSupplier(new DefaultDrawingSupplier() {
      public Shape getNextShape() {
        return new java.awt.geom.Ellipse2D.Double( -5D, -5D, 10D, 10D) ;
      }
    }) ;

    ChartPanel chartpanel = new ChartPanel(chart) ;
    add(chartpanel) ;
  }

  private static DefaultMatrixDataset createDataset() {
    DefaultMatrixDataset dataset = new DefaultMatrixDataset() ;
    dataset.addRowKey("Classes") ;
    dataset.addColKey("JDK 1.0") ;
    dataset.addColKey("JDK 1.1") ;
    dataset.addColKey("SDK 1.2") ;
    dataset.addColKey("SDK 1.3") ;
    dataset.addColKey("SDK 1.4") ;
    dataset.addRowData(1) ;

    dataset.addData(0, new NumberChartData(212D)) ;
    dataset.addData(0, new NumberChartData(504D)) ;
    dataset.addData(0, new NumberChartData(1520D)) ;
    dataset.addData(0, new NumberChartData(1842D)) ;
    dataset.addData(0, new NumberChartData(2991D)) ;
    dataset.addData(0, new NumberChartData(3500D)) ;

    return dataset ;
  }

}
