package demo.miscellaneous;

import java.awt.BorderLayout ;
import java.awt.Color ;
import java.util.Calendar ;

import javax.swing.JPanel ;

import com.ijchart.xychart.axis.value.NumberValueAxis ;
import com.ijchart.xychart.axis.value.SymbolValueAxis ;
import com.ijchart.xychart.axis.value.XYNumberValueAxis ;
import com.ijchart.xychart.client.ChartFactory ;
import com.ijchart.xychart.client.ChartPanel ;
import com.ijchart.xychart.client.IJChart ;
import com.ijchart.xychart.constant.ChartOrientation ;
import com.ijchart.xychart.constant.IAngle ;
import com.ijchart.xychart.constant.RenderDrawOrder ;
import com.ijchart.xychart.constant.XYValueAxisType ;
import com.ijchart.xychart.data.XYNumberChartData ;
import com.ijchart.xychart.dataset.DefaultMatrixDataset ;
import com.ijchart.xychart.dataset.IMatrixDataset ;
import com.ijchart.xychart.plot.XYPlot ;
import com.ijchart.xychart.range.NumberRange ;
import com.ijchart.xychart.render.xy.XYAreaRender ;
import com.ijchart.xychart.render.xy.XYLineAndShapeRender ;

/**
 * <p>Title: IJChart</p>
 *
 * <p>Description: a chart library for the Java(tm) platform.</p>
 *
 * <p>Copyright: Copyright (c) 2013</p>
 *
 * <p>Company: </p>
 *
 * @author zhang feng min,shanghai china.
 * <a href="mailto:ijchartsh@gmail.com">Contract me by email.</a>
 * @version 1.0
 */
public class CompassFormatDemo1 extends JPanel {
  private ChartOrientation orientation = ChartOrientation.VERTICAL ;

  public CompassFormatDemo1() {
    setLayout(new BorderLayout()) ;
    addPanel1() ;
  }

  private void addPanel1() {
    IMatrixDataset dataset = createDirectionDataset(600) ;
    final IJChart chart = ChartFactory.createXYLineAndShapeChart("Time", "Date", "Direction", dataset,
                                                         orientation, true, true, true, false) ;
    XYPlot plot = (XYPlot) chart.getPlot() ;
    plot.setRenderDrawOrder(RenderDrawOrder.REVERSE) ;

    String[] symbols = new String[] {"ESE", "SE", "SSE", "S", "SSW", "SW", "WSW"} ;
    SymbolValueAxis valueAxis1 = new SymbolValueAxis("Direction",
                                                     (NumberValueAxis) plot.getValueAxis(), symbols) ;
    valueAxis1.setAxisLabelAngle(IAngle.ANGLE_UP_90) ;
    valueAxis1.setAxisRangeIncludeZero(false) ;
    plot.setValueAxis(0, valueAxis1) ;

    XYLineAndShapeRender render = (XYLineAndShapeRender) plot.getRender() ;
    render.setBaseItemShapeVisible(false) ;
    render.setAutoSelectBorderPaint(false) ;
    render.setRowItemBorderPaint(0, Color.red) ;

    IMatrixDataset dataset2 = createForceDataset(600) ;
    XYNumberValueAxis valueAxis2 = new XYNumberValueAxis("Force", dataset2,
                                                         XYValueAxisType.XY_VALUE_AXIS_Y, false) ;
    valueAxis2.setUserDefineDatasetRange(new NumberRange(0, 12)) ;
    valueAxis2.setAxisLabelAngle(IAngle.ANGLE_DOWN_90) ;
    plot.setValueAxis(1, valueAxis2) ;

    XYAreaRender render2 = new XYAreaRender(dataset2) ;
    render2.setAutoSelectFillPaint(false) ;
    render2.setRowItemFillPaint(0, new Color(0, 0, 255, 128)) ;
    plot.setRender(1, render2) ;

    ChartPanel chartpanel = new ChartPanel(chart) ;
    add(chartpanel) ;
  }

  private static IMatrixDataset createDirectionDataset(int i) {
    DefaultMatrixDataset dataset = new DefaultMatrixDataset() ;
    dataset.addRowKey("Wind Direction") ;
    dataset.addRowData() ;

    Calendar cal = Calendar.getInstance() ;
    cal.set(Calendar.SECOND, 0) ;
    cal.set(Calendar.MILLISECOND, 0) ;

    double d = 180D ;
    for (int j = 0 ; j < i ; j++) {
      dataset.addData(0, new XYNumberChartData(cal.getTimeInMillis(), d)) ;
      cal.add(Calendar.MINUTE, 1) ;

      d += (Math.random() - 0.5D) * 15D ;
      if (d < 0.0D) {
        d += 360D ;
        continue ;
      }
      if (d > 360D) {
        d -= 360D ;
      }
    }

    return dataset ;
  }

  private static IMatrixDataset createForceDataset(int i) {
    DefaultMatrixDataset dataset = new DefaultMatrixDataset() ;
    dataset.addRowKey("Wind Force") ;
    dataset.addRowData() ;

    Calendar cal = Calendar.getInstance() ;
    cal.set(Calendar.SECOND, 0) ;
    cal.set(Calendar.MILLISECOND, 0) ;

    double d = 3D ;
    for (int j = 0 ; j < i ; j++) {
      dataset.addData(0, new XYNumberChartData(cal.getTimeInMillis(), d)) ;
      cal.add(Calendar.MINUTE, 1) ;
      d = Math.max(0.5D, d + (Math.random() - 0.5D) * 0.5D) ;
    }

    return dataset ;
  }

}
