package demo.miscellaneous ;

import java.awt.BorderLayout ;
import java.awt.Color ;
import java.util.Calendar ;

import javax.swing.JPanel ;
import javax.swing.JSlider ;
import javax.swing.event.ChangeEvent ;
import javax.swing.event.ChangeListener ;

import com.ijchart.xychart.axis.value.XYNumberValueAxis ;
import com.ijchart.xychart.client.ChartFactory ;
import com.ijchart.xychart.client.ChartPanel ;
import com.ijchart.xychart.client.IJChart ;
import com.ijchart.xychart.constant.ChartOrientation ;
import com.ijchart.xychart.data.IChartData ;
import com.ijchart.xychart.data.XYNumberChartData ;
import com.ijchart.xychart.dataset.DefaultMatrixDataset ;
import com.ijchart.xychart.dataset.IMatrixDataset ;
import com.ijchart.xychart.plot.XYPlot ;
import com.ijchart.xychart.render.xy.XYLineAndShapeRender ;
import com.ijchart.xychart.ui.RectangleEdge ;
import com.ijchart.xychart.ui.RectangleInsets ;

/**
 * <p>Title: IJChart</p>
 *
 * <p>Description: a chart library for the Java(tm) platform.</p>
 *
 * <p>Copyright: Copyright (c) 2013</p>
 *
 * <p>Company: </p>
 *
 * @author zhang feng min,shanghai china.
 * <a href="mailto:ijchartsh@gmail.com">Contract me by email.</a>
 * @version 1.0
 */
public class TranslateDemo1 extends JPanel implements ChangeListener {
  private TranslateDataset renderDataset ;

  private JSlider slider ;

  public TranslateDemo1() {
    setLayout(new BorderLayout()) ;
    addPanel1() ;

    slider = new JSlider( -200, 200, 0) ;
    slider.setPaintLabels(true) ;
    slider.setMajorTickSpacing(50) ;
    slider.setPaintTicks(true) ;
    slider.addChangeListener(this) ;
    add(slider, BorderLayout.SOUTH) ;

  }

  private void addPanel1() {
    XYNumberChartData[] datas = createDatas() ;
    IMatrixDataset axisDataset = new TranslateDataset(datas) ;
    renderDataset = new TranslateDataset(datas) ;
    IJChart chart = ChartFactory.createXYLineAndShapeChart("Translate Demo 1", "Time of Day",
                                                           "Value", renderDataset,
                                                           ChartOrientation.VERTICAL, true, true, true, false) ;
    chart.getLegendTitle().setPosition(RectangleEdge.RIGHT) ;
    XYPlot xyplot = (XYPlot) chart.getPlot() ;
    xyplot.setBackgroundPaint(Color.white) ;
    xyplot.setDomainGridlinePaint(Color.white) ;
    xyplot.setValueGridlinePaint(Color.white) ;
    xyplot.setAxisOffset(new RectangleInsets(5D, 5D, 5D, 5D)) ;
    xyplot.setDomainCrosshairVisible(true) ;

    XYNumberValueAxis xAxis = (XYNumberValueAxis) xyplot.getDomainAxis() ;
    xAxis.setDataset(axisDataset) ;
    XYNumberValueAxis yAxis = (XYNumberValueAxis) xyplot.getValueAxis() ;
    yAxis.setDataset(axisDataset) ;

    XYLineAndShapeRender render = (XYLineAndShapeRender) xyplot.getRender() ;
    render.setBaseItemShapeVisible(false) ;

    ChartPanel chartpanel = new ChartPanel(chart) ;
    add(chartpanel, BorderLayout.CENTER) ;
  }

  public void stateChanged(ChangeEvent changeevent) {
    int i = slider.getValue() ;
    renderDataset.setTranslate(i * 60 * 1000D) ;
  }

  private static XYNumberChartData[] createDatas() {
    final int len = 200 ;
    XYNumberChartData[] datas = new XYNumberChartData[len] ;

    Calendar cal = Calendar.getInstance() ;
    cal.set(Calendar.SECOND, 0) ;
    cal.set(Calendar.MILLISECOND, 0) ;

    double value = 100 ;
    for (int i = 0 ; i < len ; i++) {
      datas[i] = new XYNumberChartData(cal.getTimeInMillis(), value) ;
      cal.add(Calendar.MINUTE, 1) ;
      value *= 1.0D + (Math.random() - 0.495D) / 10D ;
    }

    return datas ;
  }

  private class TranslateDataset extends DefaultMatrixDataset {
    private XYNumberChartData[] datas ;

    private double translate ;

    TranslateDataset(XYNumberChartData[] datas) {
      this.datas = datas ;
    }

    public double getTranslate() {
      return this.translate ;
    }

    public void setTranslate(double v) {
      this.translate = v ;
      fireDatasetChanged() ;
    }

    public int getRowCount() {
      return 1 ;
    }

    public int getColumnCount() {
      return this.datas.length ;
    }

    public int getColumnCount(int rowIdx) {
      return getColumnCount() ;
    }

    public Object getRowKey(int index) {
      return "Random 1" ;
    }

    public IChartData getData(int rowIndex, int colIndex) {
      XYNumberChartData data = this.datas[colIndex] ;
      return new XYNumberChartData(data.getX() + this.translate, data.getY()) ;
    }
  }
}
