package demo.miscellaneous.dynamiccharts;

import java.awt.BorderLayout ;
import java.awt.Color ;
import java.awt.FlowLayout ;
import java.awt.event.ActionEvent ;
import java.awt.event.ActionListener ;

import javax.swing.BorderFactory ;
import javax.swing.JButton ;
import javax.swing.JPanel ;

import com.ijchart.xychart.axis.value.XYNumberValueAxis ;
import com.ijchart.xychart.client.ChartPanel ;
import com.ijchart.xychart.client.IJChart ;
import com.ijchart.xychart.constant.UnitType ;
import com.ijchart.xychart.constant.XYValueAxisType ;
import com.ijchart.xychart.data.XYNumberChartData ;
import com.ijchart.xychart.dataset.DefaultMatrixDataset ;
import com.ijchart.xychart.plot.XYPlot ;
import com.ijchart.xychart.plot.combine.CombineDomainAxisXYPlot ;
import com.ijchart.xychart.render.xy.XYLineAndShapeRender ;
import com.ijchart.xychart.title.LegendTitle ;
import com.ijchart.xychart.ui.RectangleEdge ;
import com.ijchart.xychart.ui.RectangleInsets ;

/**
 * <p>Title: IJChart</p>
 *
 * <p>Description: a chart library for the Java(tm) platform.</p>
 *
 * <p>Copyright: Copyright (c) 2013</p>
 *
 * <p>Company: </p>
 *
 * @author zhang feng min,shanghai china.
 * <a href="mailto:ijchartsh@gmail.com">Contract me by email.</a>
 * @version 1.0
 */
public class DynamicDataDemo3 extends JPanel implements ActionListener {

  private DefaultMatrixDataset datasets[] ;

  private double lastValue[] ;

  public DynamicDataDemo3() {
    setLayout(new BorderLayout()) ;
    lastValue = new double[3] ;
    CombineDomainAxisXYPlot combineddomainxyplot = new CombineDomainAxisXYPlot() ;
    datasets = new DefaultMatrixDataset[3] ;

    for (int i = 0 ; i < 3 ; i++) {
      lastValue[i] = 100D ;
      datasets[i] = new DefaultMatrixDataset() ;
      datasets[i].addRowKey("Random " + i) ;
      datasets[i].addRowData() ;

      XYNumberValueAxis xAxis = new XYNumberValueAxis(datasets[i],
                                                      XYValueAxisType.XY_VALUE_AXIS_X, true) ;
      xAxis.setAxisLabel("Time") ;
      XYNumberValueAxis yAxis = new XYNumberValueAxis(datasets[i],
                                                      XYValueAxisType.XY_VALUE_AXIS_Y, false) ;
      yAxis.setAxisLabel("Y" + i) ;
      yAxis.setAxisRangeIncludeZero(false) ;

      XYLineAndShapeRender render = new XYLineAndShapeRender(datasets[i]) ;
      render.setBaseItemShapeVisible(false);
      XYPlot plot = new XYPlot(xAxis, yAxis, render) ;
      plot.setDomainGridlinePaint(Color.white) ;
      plot.setValueGridlinePaint(Color.white) ;

      combineddomainxyplot.addSubPlot(plot) ;
    }

    IJChart chart = new IJChart("Dynamic Data Demo 3", combineddomainxyplot) ;
    LegendTitle legendtitle = chart.getLegendTitle() ;
    legendtitle.setPosition(RectangleEdge.RIGHT) ;
    legendtitle.setPadding(new RectangleInsets(UnitType.ABSOLUTE, 0.0D, 4D, 0.0D, 4D)) ;
    chart.setBorderPaint(Color.black) ;
    chart.setBorderVisible(true) ;
    combineddomainxyplot.setAxisOffset(new RectangleInsets(4D, 4D, 4D, 4D)) ;

    ChartPanel chartpanel = new ChartPanel(chart) ;
    add(chartpanel, BorderLayout.CENTER) ;
    JPanel jpanel = new JPanel(new FlowLayout()) ;
    for (int j = 0 ; j < 3 ; j++) {
      JButton jbutton1 = new JButton("Series " + j) ;
      jbutton1.setActionCommand("ADD_DATA_" + j) ;
      jbutton1.addActionListener(this) ;
      jpanel.add(jbutton1) ;
    }

    JButton jbutton = new JButton("ALL") ;
    jbutton.setActionCommand("ADD_ALL") ;
    jbutton.addActionListener(this) ;
    jpanel.add(jbutton) ;
    add(jpanel, BorderLayout.SOUTH) ;
    chartpanel.setBorder(BorderFactory.createEmptyBorder(5, 5, 5, 5)) ;
  }

  public void actionPerformed(ActionEvent actionevent) {
    for (int i = 0 ; i < 3 ; i++) {
      if (actionevent.getActionCommand().endsWith(String.valueOf(i))) {
        lastValue[i] = lastValue[i] * (0.90000000000000002D + 0.20000000000000001D * Math.random()) ;
        datasets[i].addData(0, new XYNumberChartData(System.currentTimeMillis(), lastValue[i])) ;
      }
    }

    if (actionevent.getActionCommand().equals("ADD_ALL")) {
      for (int j = 0 ; j < 3 ; j++) {
        lastValue[j] = lastValue[j] * (0.90000000000000002D + 0.20000000000000001D * Math.random()) ;
        datasets[j].addData(0, new XYNumberChartData(System.currentTimeMillis(), lastValue[j])) ;
      }
    }
  }

}
