package demo.others.shapes ;

import java.awt.Color ;
import java.awt.Dimension ;
import java.awt.Graphics ;
import java.awt.Graphics2D ;
import java.awt.GridLayout ;
import java.awt.RenderingHints ;
import java.awt.event.MouseEvent ;
import java.awt.event.MouseMotionAdapter ;
import java.awt.geom.GeneralPath ;
import java.awt.geom.Point2D ;
import java.util.HashMap ;
import java.util.Map ;

import javax.swing.BorderFactory ;
import javax.swing.JLabel ;
import javax.swing.JPanel ;

import com.ijchart.xychart.constant.TextPosition ;
import com.ijchart.xychart.shape.GeneralPathChartShape ;

/**
 * <p>Title: IJChart</p>
 *
 * <p>Description: a chart library for the Java(tm) platform.</p>
 *
 * <p>Copyright: Copyright (c) 2013</p>
 *
 * <p>Company: </p>
 *
 * @author zhang feng min,shanghai china.
 * <a href="mailto:ijchartsh@gmail.com">Contract me by email.</a>
 * @version 1.0
 */
public class GeneralPathChartShapePanel extends JPanel {
  private Map hashMap = new HashMap() ;

  private JLabel label ;

  public GeneralPathChartShapePanel() {
    init() ;
  }

  private void init() {
    hashMap.put(RenderingHints.KEY_ANTIALIASING, RenderingHints.VALUE_ANTIALIAS_ON) ;

    setLayout(new GridLayout(5, 3)) ;
    addPanel1() ;
    addPanel2() ;
    addPanel3() ;

    label = new JLabel("begin") ;
    add(label) ;
  }

  private void addPanel1() {
    GeneralPath path = new GeneralPath() ;
    path.moveTo(20, 20) ;
    path.lineTo(35, 15) ;
    path.lineTo(35, 50) ;
    path.lineTo(20, 45) ;
    path.closePath() ;

    final GeneralPathChartShape pathShape = new GeneralPathChartShape(path) ;
    pathShape.setText("abc123") ;

    JPanel panel = new JPanel() {
      public void paint(Graphics g) {
        Graphics2D g2 = (Graphics2D) g ;
        g2.addRenderingHints(hashMap) ;

        setBackground(Color.white) ;
        super.paintComponent(g) ;

        pathShape.setDrawBorder(true) ;
        pathShape.setFillShape(true) ;
        pathShape.setFillPaint(Color.cyan) ;
        pathShape.draw(g2) ;
        pathShape.setDrawText(true) ;
        pathShape.setTextPosition(TextPosition.OUTSIDE_BOTTOM) ;

        Point2D p = pathShape.getLockPoint() ;
        g.setColor(Color.blue) ;
        g.fillRect((int) p.getX(), (int) p.getY(), 2, 2) ;
      }

    } ;

    panel.addMouseMotionListener(new MouseMotionAdapter() {
      public void mouseMoved(MouseEvent e) {
        label.setText("(" + e.getX() + "," + e.getY() + ")" + pathShape.contains(e.getX(), e.getY())) ;
      }
    }) ;

    panel.setPreferredSize(new Dimension(110, 110)) ;
    panel.setBorder(BorderFactory.createLineBorder(Color.red)) ;
    add(panel) ;
  }

  private void addPanel2() {

    GeneralPath path = new GeneralPath() ;
    path.moveTo(20, 20) ;
    path.lineTo(35, 15) ;
    path.lineTo(35, 50) ;
    path.lineTo(20, 45) ;
    path.closePath() ;

    final GeneralPathChartShape pathShape = new GeneralPathChartShape(path) ;
    JPanel panel = new JPanel() {

      public void paint(Graphics g) {
        Graphics2D g2 = (Graphics2D) g ;
        g2.addRenderingHints(hashMap) ;

        setBackground(Color.white) ;
        super.paintComponent(g) ;

        pathShape.setDrawBorder(true) ;
        pathShape.setFillShape(true) ;
        pathShape.setFillPaint(Color.cyan) ;
        pathShape.setRotate(Math.PI / 16) ;
        pathShape.setSelected(true) ;
        pathShape.draw(g2) ;

        Point2D p = pathShape.getLockPoint() ;
        g.setColor(Color.blue) ;
        g.fillRect((int) p.getX(), (int) p.getY(), 2, 2) ;
      }

    } ;

    panel.addMouseMotionListener(new MouseMotionAdapter() {
      public void mouseMoved(MouseEvent e) {
        label.setText("(" + e.getX() + "," + e.getY() + ")" + pathShape.contains(e.getX(), e.getY())) ;
      }
    }) ;

    panel.setPreferredSize(new Dimension(110, 110)) ;
    panel.setBorder(BorderFactory.createLineBorder(Color.red)) ;
    add(panel) ;
  }

  private void addPanel3() {

    GeneralPath path = new GeneralPath() ;
    path.moveTo(56, 104) ;
    path.lineTo(56, 31) ;
    path.lineTo(56, 31) ;
    path.lineTo(56, 104) ;
    path.closePath() ;

    final GeneralPathChartShape pathShape = new GeneralPathChartShape(path) ;
    JPanel panel = new JPanel() {

      public void paint(Graphics g) {
        Graphics2D g2 = (Graphics2D) g ;
        g2.addRenderingHints(hashMap) ;

        setBackground(Color.white) ;
        super.paintComponent(g) ;

        pathShape.setDrawBorder(true) ;
        pathShape.setFillShape(true) ;
        pathShape.setFillPaint(Color.cyan) ;
        pathShape.setSelected(true) ;
        pathShape.draw(g2) ;

        Point2D p = pathShape.getLockPoint() ;
        g.setColor(Color.blue) ;
        g.fillRect((int) p.getX(), (int) p.getY(), 2, 2) ;

      }

    } ;

    panel.setPreferredSize(new Dimension(110, 110)) ;
    panel.setBorder(BorderFactory.createLineBorder(Color.red)) ;
    add(panel) ;
  }

}
