package demo.others.shapes ;

import java.awt.Color ;
import java.awt.Dimension ;
import java.awt.Graphics ;
import java.awt.Graphics2D ;
import java.awt.GridLayout ;
import java.awt.RenderingHints ;
import java.awt.event.MouseEvent ;
import java.awt.event.MouseMotionAdapter ;
import java.awt.geom.GeneralPath ;
import java.awt.geom.Point2D ;
import java.awt.geom.Rectangle2D ;
import java.util.HashMap ;
import java.util.Map ;

import javax.swing.BorderFactory ;
import javax.swing.JLabel ;
import javax.swing.JPanel ;

import com.ijchart.xychart.shape.TextAndPointerChartShape ;

/**
 * <p>Title: IJChart</p>
 *
 * <p>Description: a chart library for the Java(tm) platform.</p>
 *
 * <p>Copyright: Copyright (c) 2013</p>
 *
 * <p>Company: </p>
 *
 * @author zhang feng min,shanghai china.
 * <a href="mailto:ijchartsh@gmail.com">Contract me by email.</a>
 * @version 1.0
 */
public class TextAndPointChartShapePanel extends JPanel {
  private Map hashMap = new HashMap() ;

  private JLabel label ;

  public TextAndPointChartShapePanel() {
    init() ;
  }

  private void init() {
    hashMap.put(RenderingHints.KEY_ANTIALIASING, RenderingHints.VALUE_ANTIALIAS_ON) ;

    setLayout(new GridLayout(2, 2)) ;
    addPanel1() ;
    addPanel2() ;
    addPanel3() ;

    label = new JLabel("begin") ;
    add(label) ;
  }

  private void addPanel1() {

    final float halfArrowWidth = 5.5F ;
    final float arrowLength = 5F ;
    Rectangle2D rect = new Rectangle2D.Double(5, 5, 40, 25) ;
    final GeneralPath arrow = new GeneralPath() ;
    arrow.moveTo(25, 30) ;
    arrow.lineTo(25, 60) ;
    arrow.lineTo(25 + halfArrowWidth + 0.5F, 60) ;
    arrow.lineTo(25, 60 + arrowLength) ;
    arrow.lineTo(25 - halfArrowWidth, 60) ;
    arrow.lineTo(25, 60) ;
    arrow.lineTo(25, 30) ;
    arrow.closePath() ;

    final TextAndPointerChartShape pathShape = new TextAndPointerChartShape(rect, "arrow text",
                                                                            arrow) ;
    pathShape.setForceInsideTextNoWrap(true) ;
    JPanel panel = new JPanel() {

      public void paint(Graphics g) {
        Graphics2D g2 = (Graphics2D) g ;
        g2.addRenderingHints(hashMap) ;
        g2.draw(arrow) ;

        setBackground(Color.white) ;
        super.paintComponent(g) ;

        pathShape.setSelected(true) ;
        pathShape.draw(g2) ;

        Point2D p = pathShape.getLockPoint() ;
        g.setColor(Color.red) ;
        g.fillRect((int) p.getX(), (int) p.getY(), 2, 2) ;
      }

    } ;

    panel.addMouseMotionListener(new MouseMotionAdapter() {
      public void mouseMoved(MouseEvent e) {
        label.setText("(" + e.getX() + "," + e.getY() + ")" + pathShape.contains(e.getX(), e.getY())) ;
      }
    }) ;

    panel.setPreferredSize(new Dimension(110, 110)) ;
    panel.setBorder(BorderFactory.createLineBorder(Color.red)) ;
    add(panel) ;
  }

  private void addPanel2() {

    final float halfArrowWidth = 5.5F ;
    final float arrowLength = 5F ;
    Rectangle2D rect = new Rectangle2D.Double(5, 5, 1, 1) ;
    final GeneralPath arrow = new GeneralPath() ;
    arrow.moveTo(25, 30) ;
    arrow.lineTo(25, 60) ;
    arrow.lineTo(25 + halfArrowWidth + 0.5F, 60) ;
    arrow.lineTo(25, 60 + arrowLength) ;
    arrow.lineTo(25 - halfArrowWidth, 60) ;
    arrow.lineTo(25, 60) ;
    arrow.lineTo(25, 30) ;
    arrow.closePath() ;

    final TextAndPointerChartShape pathShape = new TextAndPointerChartShape(rect, "arrow text",
                                                                            arrow) ;
    pathShape.setForceInsideTextNoWrap(true) ;
    JPanel panel = new JPanel() {

      public void paint(Graphics g) {
        Graphics2D g2 = (Graphics2D) g ;
        g2.addRenderingHints(hashMap) ;
        g2.draw(arrow) ;

        setBackground(Color.white) ;
        super.paintComponent(g) ;

        pathShape.setSelected(true) ;
        pathShape.draw(g2) ;

        Point2D p = pathShape.getLockPoint() ;
        g.setColor(Color.red) ;
        g.fillRect((int) p.getX(), (int) p.getY(), 2, 2) ;
      }

    } ;

    panel.addMouseMotionListener(new MouseMotionAdapter() {
      public void mouseMoved(MouseEvent e) {
        label.setText("(" + e.getX() + "," + e.getY() + ")" + pathShape.contains(e.getX(), e.getY())) ;
      }
    }) ;

    panel.setPreferredSize(new Dimension(110, 110)) ;
    panel.setBorder(BorderFactory.createLineBorder(Color.red)) ;
    add(panel) ;
  }

  private void addPanel3() {
    final float halfArrowWidth = 5.5F ;
    final float arrowLength = 5F ;
    Rectangle2D rect = new Rectangle2D.Double(5, 5, 1, 1) ;
    final GeneralPath arrow = new GeneralPath() ;
    arrow.moveTo(25, 30) ;
    arrow.lineTo(25, 60) ;
    arrow.lineTo(25 + halfArrowWidth + 0.5F, 60) ;
    arrow.lineTo(25, 60 + arrowLength) ;
    arrow.lineTo(25 - halfArrowWidth, 60) ;
    arrow.lineTo(25, 60) ;
    arrow.lineTo(25, 30) ;
    arrow.closePath() ;

    final TextAndPointerChartShape pathShape = new TextAndPointerChartShape(rect, "arrow text",
                                                                            arrow) ;
    pathShape.setRotate(Math.PI / -5) ;

    pathShape.setForceInsideTextNoWrap(true) ;
    JPanel panel = new JPanel() {

      public void paint(Graphics g) {
        Graphics2D g2 = (Graphics2D) g ;
        g2.addRenderingHints(hashMap) ;
        g2.draw(arrow) ;

        setBackground(Color.white) ;
        super.paintComponent(g) ;
        g.translate(50, 50) ;

        pathShape.setSelected(true) ;
        pathShape.draw(g2) ;

        Point2D p = pathShape.getLockPoint() ;
        g.setColor(Color.red) ;
        g.fillRect((int) p.getX(), (int) p.getY(), 2, 2) ;
      }

    } ;

    panel.addMouseMotionListener(new MouseMotionAdapter() {
      public void mouseMoved(MouseEvent e) {
        label.setText("(" + e.getX() + "," + e.getY() + ")" + pathShape.contains(e.getX(), e.getY())) ;
      }
    }) ;

    panel.setPreferredSize(new Dimension(150, 150)) ;
    panel.setBorder(BorderFactory.createLineBorder(Color.red)) ;
    add(panel) ;
  }

}
