package demo.stackedbarcharts ;

import java.awt.GridLayout ;

import javax.swing.JPanel ;

import com.ijchart.xychart.axis.category.CategoryAxis ;
import com.ijchart.xychart.axis.value.StackedNumberValueAxis ;
import com.ijchart.xychart.client.ChartFactory ;
import com.ijchart.xychart.client.ChartPanel ;
import com.ijchart.xychart.client.IJChart ;
import com.ijchart.xychart.constant.AxisLocation ;
import com.ijchart.xychart.constant.ChartOrientation ;
import com.ijchart.xychart.data.NumberChartData ;
import com.ijchart.xychart.dataset.DefaultMatrixDataset ;
import com.ijchart.xychart.dataset.IMatrixDataset ;
import com.ijchart.xychart.label.DefaultLabelGenerator ;
import com.ijchart.xychart.plot.CategoryPlot ;
import com.ijchart.xychart.render.category.StackedBarRender ;
import com.ijchart.xychart.tickunit.LongTickUnit ;

/**
 * <p>Title: IJChart</p>
 *
 * <p>Description: a chart library for the Java(tm) platform.</p>
 *
 * <p>Copyright: Copyright (c) 2013</p>
 *
 * <p>Company: </p>
 *
 * @author zhang feng min,shanghai china.
 * <a href="mailto:ijchartsh@gmail.com">Contract me by email.</a>
 * @version 1.0
 */
public class StackedBarChartDemo5 extends JPanel {
  private ChartOrientation orientation = ChartOrientation.VERTICAL ;

  public StackedBarChartDemo5() {
    setLayout(new GridLayout(1, 1)) ;
    addPanel1() ;
  }

  private void addPanel1() {
    IMatrixDataset dataset = createDataset1() ;
    IJChart chart = ChartFactory.createStackedBarChart("Stacked Bar Chart Demo 5", null,
                                                       "Value", dataset, orientation, true, true) ;
    CategoryPlot plot = (CategoryPlot) chart.getPlot() ;
    StackedNumberValueAxis numberaxis = (StackedNumberValueAxis) plot.getValueAxis() ;
    numberaxis.setTickUnit(new LongTickUnit()) ;

    CategoryAxis xAxis2 = new CategoryAxis("Category / Group", createDataset2()) ;
    xAxis2.setAxisLineVisible(false) ;
    xAxis2.setTickMarkVisible(false) ;
    plot.setCategoryAxis(1, xAxis2) ;
    plot.setCategoryAxisLocation(1, AxisLocation.BOTTOM_OR_RIGHT) ;

    StackedBarRender render = (StackedBarRender) plot.getRender() ;
    render.setBaseItemLabelVisible(true) ;
    render.setLabelGenerator(new DefaultLabelGenerator() {
      public String generateLabel(IMatrixDataset dataset, int row, int column) {
        NumberChartData data = (NumberChartData) dataset.getData(row, column) ;
        return String.valueOf((int) data.getValue()) ;
      }
    }) ;

    ChartPanel chartpanel = new ChartPanel(chart) ;
    add(chartpanel) ;
  }

  private static IMatrixDataset createDataset1() {
    DefaultMatrixDataset dataset = new DefaultMatrixDataset() ;
    dataset.addColKey("S1") ;
    dataset.addColKey("S2") ;
    dataset.addColKey("S3") ;
    dataset.addColKey("S4") ;

    for (int i = 1 ; i < 7 ; i++) {
      dataset.addRowKey("G " + (i % 3 == 0 ? 3 : i % 3)) ;
    }

    double dataArr[][] = { {3396D, 1580D}, {Double.NaN, Double.NaN, 76D}, {Double.NaN, Double.NaN,
                         Double.NaN, 10100D}, {3429D, 1562D}, {Double.NaN, Double.NaN, 61D},
                         {Double.NaN, Double.NaN, Double.NaN, -10100D},
    } ;

    dataset.addRowData(dataArr.length) ;
    for (int i = 0 ; i < dataArr.length ; i++) {
      for (int j = 0 ; j < dataArr[i].length ; j++) {
        dataset.addData(i, new NumberChartData(dataArr[i][j])) ;
      }
    }

    return dataset ;
  }

  private IMatrixDataset createDataset2() {
    DefaultMatrixDataset dataset = new DefaultMatrixDataset() ;
    String[] rowKeys = {"", "C1", "", "", "C2", ""} ;

    for (int i = 0 ; i < rowKeys.length ; i++) {
      dataset.addRowKey(rowKeys[i]) ;
    }
    dataset.addRowData(rowKeys.length) ;

    return dataset ;
  }

}
