package demo.statisticalcharts ;

import java.awt.BasicStroke ;
import java.awt.GridLayout ;

import javax.swing.JPanel ;

import com.ijchart.xychart.annotation.XYPointerAnnotation ;
import com.ijchart.xychart.axis.value.XYNumberValueAxis ;
import com.ijchart.xychart.client.ChartFactory ;
import com.ijchart.xychart.client.ChartPanel ;
import com.ijchart.xychart.client.IJChart ;
import com.ijchart.xychart.constant.ChartOrientation ;
import com.ijchart.xychart.constant.TextPosition ;
import com.ijchart.xychart.data.XYNumberChartData ;
import com.ijchart.xychart.dataset.DefaultMatrixDataset ;
import com.ijchart.xychart.dataset.IMatrixDataset ;
import com.ijchart.xychart.function.IFunction ;
import com.ijchart.xychart.function.NormalDistributionFunction ;
import com.ijchart.xychart.plot.XYPlot ;
import com.ijchart.xychart.render.xy.XYLineAndShapeRender ;
import com.ijchart.xychart.tickunit.LongTickUnit ;
import com.ijchart.xychart.util.DatasetUtil ;

/**
 * <p>Title: IJChart</p>
 *
 * <p>Description: a chart library for the Java(tm) platform.</p>
 *
 * <p>Copyright: Copyright (c) 2013</p>
 *
 * <p>Company: </p>
 *
 * @author zhang feng min,shanghai china.
 * <a href="mailto:ijchartsh@gmail.com">Contract me by email.</a>
 * @version 1.0
 */
public class NormalDistributionDemo2 extends JPanel {

  public NormalDistributionDemo2() {
    setLayout(new GridLayout(1, 1)) ;
    addPanel1() ;
  }

  private void addPanel1() {
    IMatrixDataset dataset = createDataset() ;
    IJChart chart = ChartFactory.createXYLineAndShapeChart("Normal Distribution 2", "X", "Y",
                                                           dataset, ChartOrientation.VERTICAL, true, true,
                                                           false, false) ;
    XYPlot plot = (XYPlot) chart.getPlot() ;
    XYLineAndShapeRender render = (XYLineAndShapeRender) plot.getRender() ;
    render.setBaseItemShapeVisible(false) ;

    XYNumberValueAxis xAxis = (XYNumberValueAxis) plot.getDomainAxis() ;
    xAxis.setTickUnit(new LongTickUnit()) ;

    plot.setDomainZeroBaselineVisible(true) ;
    plot.setValueZeroBaselineVisible(true) ;
    XYNumberValueAxis yAxis = (XYNumberValueAxis) plot.getDomainAxis() ;
    yAxis.setLowerMargin(0.0D) ;
    yAxis.setUpperMargin(0.0D) ;
    plot.getValueAxis().setUpperMargin(0.03) ;

    render.setAutoSelectBorderStroke(false) ;
    render.setRowItemBorderStroke(0, new BasicStroke(1.5F)) ;
    render.setRowItemBorderStroke(1, new BasicStroke(2.0F, 1, 1, 1.0F, new float[] {6F, 4F}, 0.0F)) ;
    render.setRowItemBorderStroke(2, new BasicStroke(2.0F, 1, 1, 1.0F,
                                                     new float[] {6F, 4F, 3F, 3F}, 0.0F)) ;
    render.setRowItemBorderStroke(3, new BasicStroke(2.0F, 1, 1, 1.0F, new float[] {4F, 4F}, 0.0F)) ;

    XYPointerAnnotation annotation = new XYPointerAnnotation(" = -2.0, \262 = 0.5", -2D,
                                                             0.56399999999999995D,
                                                             3.9269908169872414D) ;
    annotation.setLabelOffset(4D) ;
    annotation.setTextPosition(TextPosition.OUTSIDE_LEFT) ;
    render.addAnnotation(annotation) ;
    XYPointerAnnotation annotation1 = new XYPointerAnnotation(" = 0.0, \262 = 0.2",
                                                              0.22500000000000001D,
                                                              0.80000000000000004D, 0.0D) ;
    annotation1.setLabelOffset(4D) ;
    render.addAnnotation(annotation1) ;
    XYPointerAnnotation annotation2 = new XYPointerAnnotation(" = 0.0, \262 = 1.0",
                                                              0.75D, 0.29999999999999999D,
                                                              5.497787143782138D) ;
    annotation2.setLabelOffset(4D) ;
    render.addAnnotation(annotation2) ;
    XYPointerAnnotation annotation3 = new XYPointerAnnotation(" = 0.0, \262 = 5.0", 3D,
                                                              0.074999999999999997D,
                                                              4.7123889803846897D) ;
    annotation3.setLabelOffset(4D) ;
    render.addAnnotation(annotation3) ;

    ChartPanel chartpanel = new ChartPanel(chart) ;
    add(chartpanel) ;
  }

  private static IMatrixDataset createDataset() {
    IFunction function1 = new NormalDistributionFunction(0.0D, 1.0D) ;
    XYNumberChartData[] datas1 = DatasetUtil.getDatasByFunction(function1, -5.0999999999999996D,
                                                                5.0999999999999996D, 121) ;
    IFunction function2 = new NormalDistributionFunction(0.0D, Math.sqrt(0.20000000000000001D)) ;
    XYNumberChartData[] datas2 = DatasetUtil.getDatasByFunction(function2, -5.0999999999999996D,
                                                                5.0999999999999996D, 121) ;
    IFunction function3 = new NormalDistributionFunction(0.0D, Math.sqrt(5D)) ;
    XYNumberChartData[] datas3 = DatasetUtil.getDatasByFunction(function3, -5.0999999999999996D,
                                                                5.0999999999999996D, 121) ;
    IFunction function4 = new NormalDistributionFunction( -2D, Math.sqrt(0.5D)) ;
    XYNumberChartData[] datas4 = DatasetUtil.getDatasByFunction(function4, -5.0999999999999996D,
                                                                5.0999999999999996D, 121) ;

    DefaultMatrixDataset dataset = new DefaultMatrixDataset() ;
    dataset.addRowKey("N1") ;
    dataset.addRowKey("N2") ;
    dataset.addRowKey("N3") ;
    dataset.addRowKey("N4") ;

    dataset.addRowData(4) ;
    dataset.addDatas(0, datas1) ;
    dataset.addDatas(1, datas2) ;
    dataset.addDatas(2, datas3) ;
    dataset.addDatas(3, datas4) ;

    return dataset ;
  }

}
