package demo.xycharts ;

import java.awt.Color ;
import java.awt.GridLayout ;
import java.util.Calendar ;

import javax.swing.JPanel ;

import com.ijchart.xychart.axis.value.NumberValueAxis ;
import com.ijchart.xychart.client.ChartFactory ;
import com.ijchart.xychart.client.ChartPanel ;
import com.ijchart.xychart.client.IJChart ;
import com.ijchart.xychart.constant.ChartOrientation ;
import com.ijchart.xychart.data.XYZNumberChartData ;
import com.ijchart.xychart.dataset.DefaultMatrixDataset ;
import com.ijchart.xychart.dataset.IMatrixDataset ;
import com.ijchart.xychart.plot.XYPlot ;
import com.ijchart.xychart.render.paint.LookupPaintScale ;
import com.ijchart.xychart.render.xy.XYBlockRender ;
import com.ijchart.xychart.tickunit.LongTickUnit ;
import com.ijchart.xychart.title.LegendTitle ;

/**
 * <p>Title: IJChart</p>
 *
 * <p>Description: a chart library for the Java(tm) platform.</p>
 *
 * <p>Copyright: Copyright (c) 2013</p>
 *
 * <p>Company: </p>
 *
 * @author zhang feng min,shanghai china.
 * <a href="mailto:ijchartsh@gmail.com">Contract me by email.</a>
 * @version 1.0
 */
public class XYBlockChartDemo2 extends JPanel {

  public XYBlockChartDemo2() {
    setLayout(new GridLayout(1, 1)) ;
    addPanel1() ;
  }

  private void addPanel1() {
    IMatrixDataset dataset = createDataset() ;
    IJChart chart = ChartFactory.createXYBlockChart(
      "XYBlockChartDemo2", "Date", "Hour", null, dataset, ChartOrientation.HORIZONTAL,
      true, true, true, false) ;

    LegendTitle legendTitle = chart.getLegendTitle() ;
    legendTitle.setBorderVisible(false) ;
    XYPlot plot = (XYPlot) chart.getPlot() ;
    ((NumberValueAxis) plot.getValueAxis()).setTickUnit(new LongTickUnit()) ;

    String[] symbols = new String[] {"", "Unavailable", "", "Free", "",
                       "Group 1", "", "Group 2", ""} ;
    LookupPaintScale lookuppaintscale = new LookupPaintScale(0.0D, 4.5D, Color.white, symbols) ;
    lookuppaintscale.add(0.0D, Color.red) ;
    lookuppaintscale.add(1D, Color.green) ;
    lookuppaintscale.add(2D, Color.blue) ;
    lookuppaintscale.add(3D, Color.yellow) ;

    XYBlockRender render = (XYBlockRender) plot.getRender() ;
    render.setBackgroundPaint(Color.white) ;
    render.setPaintScale(lookuppaintscale) ;
    render.setBlockWidth(86400000D) ;

    ChartPanel chartpanel = new ChartPanel(chart) ;
    add(chartpanel) ;
  }

  private static IMatrixDataset createDataset() {
    DefaultMatrixDataset dataset = new DefaultMatrixDataset() ;
    dataset.addRowData() ;
    dataset.addRowKey("Series 1") ;

    Calendar cal = Calendar.getInstance() ;
    cal.set(Calendar.HOUR_OF_DAY, 0) ;
    cal.set(Calendar.MINUTE, 0) ;
    cal.set(Calendar.SECOND, 0) ;
    cal.set(Calendar.MILLISECOND, 0) ;

    for (int i = 0 ; i < 100 ; i++) {
      double d = 1.0D ;
      for (int j = 0 ; j <= 24 ; j++) {
        if (Math.random() < 0.10000000000000001D) {
          d = Math.random() * 4D ;
        }

        dataset.addData(0, new XYZNumberChartData(cal.getTime().getTime(), j, d)) ;
      }

      cal.add(Calendar.DAY_OF_MONTH, 1) ;
    }

    return dataset ;
  }

}
