package com.ijchart.xychart.annotation ;

import java.awt.BasicStroke ;
import java.awt.Color ;
import java.awt.Paint ;
import java.awt.Stroke ;
import java.awt.geom.Rectangle2D ;
import java.io.Serializable ;

import com.ijchart.xychart.axis.category.CategoryAxis ;
import com.ijchart.xychart.axis.value.AbstractValueAxis ;
import com.ijchart.xychart.chartinfo.AnnotationShapeCollection ;
import com.ijchart.xychart.constant.ChartOrientation ;
import com.ijchart.xychart.shape.LineChartShape ;
import com.ijchart.xychart.ui.RectangleEdge ;
import com.ijchart.xychart.util.EqualsUtil ;
import com.ijchart.xychart.util.HashcodeUtil ;

/**
 * <p>Title: IJChart</p>
 * <zh>{@link com.ijchart.xychart.plot.CategoryPlot}ע(annotation).</zh>
 * <en>A line annotation that can be placed on a {@link com.ijchart.xychart.plot.CategoryPlot}.</en>
 * <p>Description: a chart library for the Java(tm) platform.</p>
 *
 * <p>Copyright: Copyright (c) 2013</p>
 *
 * <p>Company: </p>
 *
 * @author zhang feng min,shanghai china.
 * <a href="mailto:ijchartsh@gmail.com">Contract me by email.</a>
 * @version 1.0
 */
public class CategoryLineAnnotation extends AbstractAnnotation implements ICategoryAnnotation,
  Cloneable, Serializable {

  /**
   * <zh>л汾ID.</zh>
   * <en>For serialization.</en>
   */
  private static final long serialVersionUID = 6126983644321821246L ;

  /**
   * <zh>ֱʹõȱʡpaint.</zh>
   * <en>The default line paint.</en>
   */
  public static final Paint DEFAULT_LINE_PAINT = Color.black ;

  /**
   * <zh>ֱʹõȱʡstroke.</zh>
   * <en>The default line stroke.</en>
   */
  public static final Stroke DEFAULT_LINE_STROKE = new BasicStroke(1.0F) ;

  /**
   * <zh>(category).</zh>
   * <en>The category for the start of the line.</en>
   */
  private Object startCategory ;

  /**
   * <zh>yֵ.</zh>
   * <en>The start y value.</en>
   */
  private double startValue ;

  /**
   * <zh>յ(category).</zh>
   * <en>The category for the end of the line.</en>
   */
  private Object endCategory ;

  /**
   * <zh>յyֵ.</zh>
   * <en>The end y value.</en>
   */
  private double endValue ;

  /**
   * <zh>ֱʹõpaint.</zh>
   * <en>The line paint.</en>
   */
  private Paint paint ;

  /**
   * <zh>ֱʹõstroke.</zh>
   * <en>The line stroke.</en>
   */
  private Stroke stroke ;

  /**
   * <zh>ƴ(startCategory,startValue)յ(endCategory,endValue)ֱ.</zh>
   * <en>Draw a line from (startCategory,startValue) to (endCategory,endValue).</en>
   *
   * @param startCategory Objec <zh>(category) (Ϊnull).</zh>
   * <en>The category for the start of the line (null not permitted).</en>
   * @param startValue double <zh>yֵ.</zh>
   * <en>The start y value.</en>
   * @param endCategory Object <zh>յ(category) (Ϊnull).</zh>
   * <en>The category for the end of the line (null not permitted).</en>
   * @param endValue double <zh>յyֵ.</zh>
   * <en>The end y value.</en>
   */
  public CategoryLineAnnotation(Object startCategory, double startValue,
                                Object endCategory, double endValue) {
    this(startCategory, startValue, endCategory, endValue,
         DEFAULT_LINE_PAINT, DEFAULT_LINE_STROKE) ;
  }

  /**
   * <zh>ƴ(startCategory,startValue)յ(endCategory,endValue)ֱ.</zh>
   * <en>Draw a line from (startCategory,startValue) to (endCategory,endValue).</en>
   *
   * @param startCategory Objec <zh>(category) (Ϊnull).</zh>
   * <en>The category for the start of the line (null not permitted).</en>
   * @param startValue double <zh>yֵ.</zh>
   * <en>The start y value.</en>
   * @param endCategory Object <zh>յ(category) (Ϊnull).</zh>
   * <en>The category for the end of the line (null not permitted).</en>
   * @param endValue double <zh>յyֵ.</zh>
   * <en>The end y value.</en>
   * @param paint Paint <zh>ֱʹõpaint (Ϊnull).</zh>
   * <en>The line paint (null not permitted).</en>
   * @param stroke Stroke <zh>ֱʹõstroke (Ϊnull).</zh>
   * <en>The line stroke (null not permitted).</en>
   */
  public CategoryLineAnnotation(Object startCategory, double startValue,
                                Object endCategory, double endValue,
                                Paint paint, Stroke stroke) {
    if (startCategory == null) {
      throw new IllegalArgumentException("Null 'startCategory' argument.") ;
    }
    if (endCategory == null) {
      throw new IllegalArgumentException("Null 'endCategory' argument.") ;
    }
    if (paint == null) {
      throw new IllegalArgumentException("Null 'paint' argument.") ;
    }
    if (stroke == null) {
      throw new IllegalArgumentException("Null 'stroke' argument.") ;
    }

    this.startCategory = startCategory ;
    this.startValue = startValue ;

    this.endCategory = endCategory ;
    this.endValue = endValue ;

    this.paint = paint ;
    this.stroke = stroke ;
  }

  /**
   * <zh>(category).</zh>
   * <en>Returns the category for the start of the line.</en>
   *
   * @return Object <zh>(category) (Ϊnull).</zh>
   * <en>The category (never null).</en>
   */
  public Object getStartCategory() {
    return this.startCategory ;
  }

  /**
   * <zh>(category),м
   * {@link com.ijchart.xychart.event.AnnotationChangeEvent}¼.</zh>
   * <en>Sets the category for the start of the line and sends an
   * {@link com.ijchart.xychart.event.AnnotationChangeEvent} to all registered listeners.</en>
   *
   * @param category Object <zh>(category) (Ϊnull).</zh>
   * <en>The category (null not permitted).</en>
   */
  public void setStartCategory(Object category) {
    if (category == null) {
      throw new IllegalArgumentException("Null 'category' argument.") ;
    }

    this.startCategory = category ;
    fireAnnotationChanged() ;
  }

  /**
   * <zh>yֵ.</zh>
   * <en>Returns the start y value.</en>
   *
   * @return double <zh>ֵ.</zh>
   * <en>The start value.</en>
   */
  public double getStartValue() {
    return this.startValue ;
  }

  /**
   * <zh>yֵ,м
   * {@link com.ijchart.xychart.event.AnnotationChangeEvent}¼.</zh>
   * <en>Sets the start y value and sends an
   * {@link com.ijchart.xychart.event.AnnotationChangeEvent} to all registered listeners.</en>
   *
   * @param value double <zh>ֵ.</zh>
   * <en>The start value.</en>
   */
  public void setStartValue(double value) {
    if (this.startValue != value) {
      this.startValue = value ;
      fireAnnotationChanged() ;
    }
  }

  /**
   * <zh>յ(category).</zh>
   * <en>Returns the category for the end of the line.</en>
   *
   * @return Object <zh>(category) (Ϊnull).</zh>
   * <en>The category (never null).</en>
   */
  public Object getEndCategory() {
    return this.endCategory ;
  }

  /**
   * <zh>յ(category),м
   * {@link com.ijchart.xychart.event.AnnotationChangeEvent}¼.</zh>
   * <en>Sets the category for the end of the line and sends an
   * {@link com.ijchart.xychart.event.AnnotationChangeEvent} to all registered listeners.</en>
   *
   * @param category Object <zh>(category) (Ϊnull).</zh>
   * <en>The category (null not permitted).</en>
   */
  public void setEndCategory(Object category) {
    if (category == null) {
      throw new IllegalArgumentException("Null 'category' argument.") ;
    }

    this.endCategory = category ;
    fireAnnotationChanged() ;
  }

  /**
   * <zh>յyֵ.</zh>
   * <en>Returns the end y value.</en>
   *
   * @return double <zh>յֵ.</zh>
   * <en>The end value.</en>
   */
  public double getEndValue() {
    return this.endValue ;
  }

  /**
   * <zh>յyֵ,м
   * {@link com.ijchart.xychart.event.AnnotationChangeEvent}¼.</zh>
   * <en>Sets the end y value and sends an
   * {@link com.ijchart.xychart.event.AnnotationChangeEvent} to all registered listeners.</en>
   *
   * @param value double <zh>յֵ.</zh>
   * <en>The end value.</en>
   */
  public void setEndValue(double value) {
    if (this.endValue != value) {
      this.endValue = value ;
      fireAnnotationChanged() ;
    }
  }

  /**
   * <zh>ػֱʹõpaint.</zh>
   * <en>Returns the line paint.</en>
   *
   * @return Paint <zh>Paint (Ϊnull).</zh>
   * <en>The line paint (never null).</en>
   */
  public Paint getPaint() {
    return this.paint ;
  }

  /**
   * <zh>ûֱʹõpaint,м
   * {@link com.ijchart.xychart.event.AnnotationChangeEvent}¼.</zh>
   * <en>Sets the line paint and sends an
   * {@link com.ijchart.xychart.event.AnnotationChangeEvent} to all registered listeners.</en>
   *
   * @param paint Paint <zh>Paint (Ϊnull).</zh>
   * <en>The paint (null not permitted).</en>
   */
  public void setPaint(Paint paint) {
    if (paint == null) {
      throw new IllegalArgumentException("Null 'paint' argument.") ;
    }

    this.paint = paint ;
    fireAnnotationChanged() ;
  }

  /**
   * <zh>ػֱʹõstroke.</zh>
   * <en>Returns the line stroke.</en>
   *
   * @return Stroke <zh>Stroke (Ϊnull).</zh>
   * <en>The line stroke (never null).</en>
   */
  public Stroke getStroke() {
    return this.stroke ;
  }

  /**
   * <zh>ûֱʹõstroke,м
   * {@link com.ijchart.xychart.event.AnnotationChangeEvent}¼.</zh>
   * <en>Sets the line stroke and sends an
   * {@link com.ijchart.xychart.event.AnnotationChangeEvent} to all registered listeners.</en>
   *
   * @param stroke Stroke <zh>Stroke (Ϊnull).</zh>
   * <en>The line stroke (null not permitted).</en>
   */
  public void setStroke(Stroke stroke) {
    if (stroke == null) {
      throw new IllegalArgumentException("Null 'stroke' argument.") ;
    }

    this.stroke = stroke ;
    fireAnnotationChanged() ;
  }

  /**
   * <zh>ע(annotation)ͼ.</zh>
   * <en>Draws the annotation.</en>
   *
   * @param categoryAxis CategoryAxis <zh>category (Ϊnull).</zh>
   * <en>The category axis (null not permitted).</en>
   * @param valueAxis AbstractValueAxis <zh>value (Ϊnull).</zh>
   * <en>The value axis (null not permitted).</en>
   * @param valueAxisEdge RectangleEdge <zh>valueλ (Ϊnull).</zh>
   * <en>The value axis edge (null not permitted).</en>
   * @param dataArea Rectangle2D <zh> (Ϊnull).</zh>
   * <en>The chart data area (null not permitted).</en>
   * @param orientation ChartOrientation <zh>ͼķ (Ϊnull).</zh>
   * <en>The chart orientation (null not permitted).</en>
   * @param annotationShapes AnnotationShapeCollection <zh>ע(annotation)ͼμ (Ϊnull).</zh>
   * <en>Collects information about the annotation (null not permitted).</en>
   */
  public void drawAnnotation(CategoryAxis categoryAxis, AbstractValueAxis valueAxis,
                             RectangleEdge valueAxisEdge, Rectangle2D dataArea,
                             ChartOrientation orientation,
                             AnnotationShapeCollection annotationShapes) {
    if (categoryAxis == null) {
      throw new IllegalArgumentException("Null 'categoryAxis' argument.") ;
    }
    if (valueAxis == null) {
      throw new IllegalArgumentException("Null 'valueAxis' argument.") ;
    }
    if (valueAxisEdge == null) {
      throw new IllegalArgumentException("Null 'valueAxisEdge' argument.") ;
    }
    if (dataArea == null) {
      throw new IllegalArgumentException("Null 'dataArea' argument.") ;
    }
    if (annotationShapes == null) {
      throw new IllegalArgumentException("Null 'annotationShapes' argument.") ;
    }

    double startXPos = categoryAxis.categoryToJava2D(this.startCategory, null) ;
    if (startXPos < 0) { //categoryҲ
      return ;
    }

    double endXPos = categoryAxis.categoryToJava2D(this.endCategory, null) ;
    if (endXPos < 0) { //categoryҲ
      return ;
    }

    double startYPos = valueAxis.valueToJava2D(dataArea, valueAxisEdge, this.startValue, false) ;
    if (startYPos == AbstractValueAxis.INVALID_COORDINATE_VALUE) {
      return ;
    }

    double endYPos = valueAxis.valueToJava2D(dataArea, valueAxisEdge, this.endValue, false) ;
    if (endYPos == AbstractValueAxis.INVALID_COORDINATE_VALUE) {
      return ;
    }

    LineChartShape lineShape = null ;
    if (orientation.equals(ChartOrientation.VERTICAL)) {
      lineShape = new LineChartShape(startXPos, startYPos, endXPos, endYPos) ;
    } else if (orientation.equals(ChartOrientation.HORIZONTAL)) {
      lineShape = new LineChartShape(startYPos, startXPos, endYPos, endXPos) ;
    }

    if (lineShape != null) {
      lineShape.setBorderPaint(this.paint) ;
      lineShape.setBorderStroke(this.stroke) ;
      lineShape.setDrawBorder(true) ;
      lineShape.setTooltipText(getTooltipText()) ;

      annotationShapes.addAnnotationShape(lineShape) ;
    }
  }

  /**
   * <zh>Ƿ.</zh>
   * <en>Tests this instance for equality with an arbitrary object.</en>
   *
   * @param obj  the object (null permitted).
   *
   * @return A boolean.
   */
  public boolean equals(Object obj) {
    if (obj == this) {
      return true ;
    }
    if (!(obj instanceof CategoryLineAnnotation)) {
      return false ;
    }
    if (!super.equals(obj)) {
      return false ;
    }

    CategoryLineAnnotation that = (CategoryLineAnnotation) obj ;
    if (EqualsUtil.equals(this.startCategory, that.startCategory)) {
      return false ;
    }
    if (this.startValue != that.startValue) {
      return false ;
    }
    if (EqualsUtil.equals(this.endCategory, that.endCategory)) {
      return false ;
    }
    if (this.endValue != that.endValue) {
      return false ;
    }
    if (EqualsUtil.equals(this.paint, that.paint)) {
      return false ;
    }
    if (EqualsUtil.equals(this.stroke, that.stroke)) {
      return false ;
    }

    return true ;
  }

  /**
   * <zh>hashcodeֵ.</zh>
   * <en>Returns a hash code value for the object.</en>
   *
   * @return int <zh>hashcodeֵ.</zh>
   * <en>A hash code value for this object.</en>
   */
  public int hashCode() {
    double hashcode = super.hashCode() ;
    hashcode = 37 * hashcode + HashcodeUtil.hashCode(this.startCategory) ;
    hashcode = 37 * hashcode + 100 * this.startValue ;
    hashcode = 37 * hashcode + HashcodeUtil.hashCode(this.endCategory) ;
    hashcode = 37 * hashcode + 100 * this.endValue ;
    hashcode = 37 * hashcode + HashcodeUtil.hashCode(this.paint) ;
    hashcode = 37 * hashcode + HashcodeUtil.hashCode(this.stroke) ;

    return (int) hashcode ;
  }

  /**
   * <zh>ؿ¡Ķ.</zh>
   * <en>Creates and returns a copy of this object.</en>
   * @return object   a clone of this instance.
   * @exception  CloneNotSupportedException  if the object's class does not
   *               support the Cloneable interface. Subclasses
   *               that override the clone method can also
   *               throw this exception to indicate that an instance cannot
   *               be cloned.
   */
  public Object clone() throws CloneNotSupportedException {
    return super.clone() ;
  }

  /**
   * <zh>toString.</zh>
   * <en>Returns a string representation of the object.</en>
   *
   * @return  a string representation of the object.
   */
  public String toString() {
    return "[startCategory=" + this.startCategory
      + ",startValue=" + this.startValue
      + ",endCategory=" + this.endCategory
      + ",endValue=" + this.endValue
      + ",paint=" + this.paint
      + ",stroke=" + this.stroke
      + "]" ;
  }

}
