package com.ijchart.xychart.axis.value ;

import java.io.Serializable ;
import java.util.List ;

import com.ijchart.xychart.data.group.NumberGroupChartData ;
import com.ijchart.xychart.dataset.IMatrixDataset ;
import com.ijchart.xychart.exception.InvalidChartDataException ;
import com.ijchart.xychart.range.NumberRange ;
import com.ijchart.xychart.render.category.StackedBarRender ;

/**
 * <p>Title: IJChart</p>
 *
 * <p>Description: a chart library for the Java(tm) platform.</p>
 * <zh>ջ͵ͼʹõֵ.</zh>
 * <en>The axis used for stack bar chart.</en>
 *
 * <p>Copyright: Copyright (c) 2013</p>
 *
 * <p>Company: </p>
 *
 * @author zhang feng min,shanghai china.
 * <a href="mailto:ijchartsh@gmail.com">Contract me by email.</a>
 * @version 1.0
 */
public class GroupStackedNumberValueAxis extends StackedNumberValueAxis implements Cloneable,
  Serializable {
  /**
   * <zh>л汾ID.</zh>
   * <en>For serialization.</en>
   */
  private static final long serialVersionUID = 3147973847602967466L ;

  /**
   * <zh>ݼ.</zh>
   * <en>Constructs an axis using a dataset.</en>
   *
   * @param dataset IMatrixDataset <zh>άݼ (Ϊnull).</zh>
   * <en>The two dimension matrix dataset (null not permitted).</en>
   * @param render StackedBarRender <zh>ջ״ͼƶ (Ϊnull).</zh>
   * <en>The stacked bar render object (null not permitted).</en>
   */
  public GroupStackedNumberValueAxis(IMatrixDataset dataset, StackedBarRender render) {
    this(null, dataset, render) ;
  }

  /**
   * <zh>ǩݼ.</zh>
   * <en>Constructs an axis using label and a dataset.</en>
   *
   * @param label <zh>ǩ.</zh>
   * <en>The axis label (null permitted).</en>
   * @param dataset IMatrixDataset <zh>άݼ (Ϊnull).</zh>
   * <en>The two dimension matrix dataset (null not permitted).</en>
   * @param render StackedBarRender <zh>ջ״ͼƶ (Ϊnull).</zh>
   * <en>The stacked bar render object (null not permitted).</en>
   */
  public GroupStackedNumberValueAxis(String label, IMatrixDataset dataset, StackedBarRender render) {
    super(label, dataset, render) ;
  }

  /**
   * <zh>õݼݷΧ.</zh>
   * <en>Returns the value range of a dataset.</en>
   *
   * @return NumberRange <zh>ݷΧ (Ϊnull).</zh>
   * <en>The value range of a dataset (never null).</en>
   */
  public NumberRange getDatasetRange() {
    if (getUserDefineDatasetRange() != null) {
      return getUserDefineDatasetRange() ;
    }

    double minValue = 0.0 ;
    double maxValue = 0.0 ;
    IMatrixDataset dataset = getDataset() ;

    if (dataset != null) {
      if (getStackedBarRender() != null && getStackedBarRender().isRenderAsPercentages()) {
        boolean positive = false, negative = false ; //Ƿֵ͸ֵ
        int rows = dataset.getRowCount() ;

        for (int i = 0 ; i < rows ; i++) {
          int currRowCols = dataset.getColumnCount(i) ;
          for (int j = 0 ; j < currRowCols ; j++) {
            Object data = dataset.getData(i, j) ;
            if (data != null) {
              if (!(data instanceof NumberGroupChartData)) {
                throw new InvalidChartDataException(
                  "Invalid chart data type,musts be 'NumberGroupChartData'.") ;
              } else { //Сֵиֵ֮,ֵֵ֮
                NumberGroupChartData groupData = (NumberGroupChartData) data ;
                List datas = groupData.getDatas() ;
                if (datas != null) {
                  for (int m = 0 ; m < datas.size() ; m++) {
                    Number number = (Number) datas.get(m) ;
                    if (number != null && !Double.isNaN(number.doubleValue())) {
                      double value = number.doubleValue() ;
                      if (value > 0) {
                        positive = true ;
                      } else if (value < 0) {
                        negative = true ;
                      }

                      if (positive && negative) {
                        break ;
                      }
                    }
                  }
                }

                if (positive && negative) {
                  return new NumberRange( -100, 100) ;
                }
              }
            }
          }
        }

        if (negative && !positive) { //
          return new NumberRange( -100, 0) ;
        } else if (positive && !negative) { //
          return new NumberRange(0, 100) ;
        }
      } else {
        int rows = dataset.getRowCount() ;
        for (int i = 0 ; i < rows ; i++) {
          int currRowCols = dataset.getColumnCount(i) ;
          for (int j = 0 ; j < currRowCols ; j++) {
            Object data = dataset.getData(i, j) ;
            if (data != null) {
              if (!(data instanceof NumberGroupChartData)) {
                throw new InvalidChartDataException(
                  "Invalid chart data type,musts be 'NumberGroupChartData'.") ;
              } else { //Сֵиֵ֮,ֵֵ֮
                NumberGroupChartData numberData = (NumberGroupChartData) data ;

                double groupMinValue = 0, groupMaxValue = 0 ; //һݵСֵ
                List datas = numberData.getDatas() ;
                if (datas != null) {
                  for (int m = 0 ; m < datas.size() ; m++) {
                    Number number = (Number) datas.get(m) ;
                    if (number != null && !Double.isNaN(number.doubleValue())) {
                      double value = number.doubleValue() ;
                      groupMinValue += Math.min(0, value) ;
                      groupMaxValue += Math.max(0, value) ;
                    }
                  }

                  minValue = Math.min(minValue, groupMinValue) ;
                  maxValue = Math.max(maxValue, groupMaxValue) ;
                }
              }
            }
          }

        }
      }
    }

    return new NumberRange(minValue, maxValue) ;
  }

}
