package com.ijchart.xychart.axis.value ;

import java.awt.Graphics2D ;
import java.awt.geom.Rectangle2D ;
import java.io.Serializable ;
import java.util.ArrayList ;
import java.util.List ;

import com.ijchart.xychart.range.NumberRange ;
import com.ijchart.xychart.ui.RectangleEdge ;
import com.ijchart.xychart.util.EqualsUtil ;
import com.ijchart.xychart.util.HashcodeUtil ;

/**
 * <p>Title: IJChart</p>
 *
 * <p>Description: a chart library for the Java(tm) platform.</p>
 * <zh>Էŵʽʾֵ.</zh>
 * <en>A value axis that replaces values with symbols.</en>
 * <p>Copyright: Copyright (c) 2013</p>
 *
 * <p>Company: </p>
 *
 * @author zhang feng min,shanghai china.
 * <a href="mailto:ijchartsh@gmail.com">Contract me by email.</a>
 * @version 1.0
 */
public class SymbolValueAxis extends NumberValueAxis implements Cloneable, Serializable {
  /**
   * <zh>л汾ID.</zh>
   * <en>For serialization.</en>
   */
  private static final long serialVersionUID = 762013884978039822L ;

  /**
   * <zh>value.</zh>
   * <en>The value axis.</en>
   */
  private NumberValueAxis valueAxis ;

  /**
   * <zh>.</zh>
   * <en>The symbols.</en>
   */
  private String[] symbols ;

  /**
   * <zh>ݼ.</zh>
   * <en>Constructs an axis using a dataset.</en>
   *
   * @param valueAxis NumberValueAxis <zh>value (Ϊnull).</zh>
   * <en>The value axis (null not permitted).</en>
   * @param symbols String[] <zh> (Ϊnull).</zh>
   * <en>The symbols (null not permitted).</en>
   */
  public SymbolValueAxis(NumberValueAxis valueAxis, String[] symbols) {
    this(null, valueAxis, symbols) ;
  }

  /**
   * <zh>ǩݼ.</zh>
   * <en>Constructs an axis using label and a dataset.</en>
   *
   * @param label <zh>ǩ.</zh>
   * <en>The axis label (null permitted).</en>
   * @param valueAxis NumberValueAxis <zh>value (Ϊnull).</zh>
   * <en>The value axis (null not permitted).</en>
   * @param symbols String[] <zh> (Ϊnull).</zh>
   * <en>The symbols (null not permitted).</en>
   */
  public SymbolValueAxis(String label, NumberValueAxis valueAxis, String[] symbols) {
    super(label, valueAxis.getDataset()) ;
    if (symbols == null) {
      throw new IllegalArgumentException("Null 'symbols' argument.") ;
    }
    if (symbols.length == 0) {
      throw new IllegalArgumentException("Argument 'symbols' length must be >0.") ;
    }

    this.valueAxis = valueAxis ;
    this.symbols = symbols ;

    setGridBandVisible(true) ;
    setLowerMargin(valueAxis.getLowerMargin()) ;
    setUpperMargin(valueAxis.getUpperMargin()) ;

    setAxisLabelAngle(valueAxis.getAxisLabelAngle()) ;
    setAxisLabelFont(valueAxis.getAxisLabelFont()) ;

    setTickLabelAngle(valueAxis.getTickLabelAngle()) ;
    setTickLabelFont(valueAxis.getTickLabelFont()) ;
    setAxisLabelType(valueAxis.getAxisLabelType()) ;

    setRender(valueAxis.getRender()) ;
  }

  /**
   * <zh>value.</zh>
   * <en>Returns the value axis.</en>
   *
   * @return NumberValueAxis <zh>value.</zh>
   * <en>The value axis.</en>
   */
  public NumberValueAxis getValueAxis() {
    return this.valueAxis ;
  }

  /**
   * <zh>value,м
   * {@link com.ijchart.xychart.event.AxisChangeEvent}¼.</zh>
   * <en>Sets the value axis and sends a
   * {@link com.ijchart.xychart.event.AxisChangeEvent} to all registered listeners.</en>
   *
   * @param valueAxis NumberValueAxis <zh>value.</zh>
   * <en>The value axis.</en>
   */
  public void setValueAxis(NumberValueAxis valueAxis) {
    if (valueAxis == null) {
      throw new IllegalArgumentException("Null 'valueAxis' argument") ;
    }
    if (symbols.length == 0) {
      throw new IllegalArgumentException("Argument 'symbols' length must be >0.") ;
    }

    this.valueAxis = valueAxis ;
    fireAxisChanged() ;
  }

  /**
   * <zh>ط.</zh>
   * <en>Returns the symbols.</en>
   *
   * @return String[] <zh> (Ϊnull).</zh>
   * <en>The symbols (never null).</en>
   */
  public String[] getSymbols() {
    return this.symbols ;
  }

  /**
   * <zh>÷,м{@link com.ijchart.xychart.event.AxisChangeEvent}¼.</zh>
   * <en>Sets the symbols and notifies registered listeners that the axis has changed.</en>
   *
   * @param symbols String[] <zh> (Ϊnull).</zh>
   * <en>The symbols (null not permitted).</en>
   */
  public void setSymbols(String[] symbols) {
    if (symbols == null) {
      throw new IllegalArgumentException("Null 'symbols' argument.") ;
    }
    if (symbols.length == 0) {
      throw new IllegalArgumentException("Argument 'symbols' length must be >0.") ;
    }

    this.symbols = symbols ;
    fireAxisChanged() ;
  }

  /**
   * <zh>õݼݷΧ.</zh>
   * <en>Returns the value range of a dataset.</en>
   *
   * @return NumberRange <zh>ݷΧ (Ϊnull).</zh>
   * <en>The value range of a dataset (never null).</en>
   */
  public NumberRange getDatasetRange() {
    return this.valueAxis.getDatasetRange() ;
  }

  /**
   * <zh>ʵʵıǩֵ.</zh>
   * <en>Calculate the </en>
   * @param g2 Graphics2D
   * @param axisArea Rectangle2D
   * @param edge RectangleEdge
   */
  protected void calculateLabels(Graphics2D g2, Rectangle2D axisArea, RectangleEdge edge) {
    NumberRange range = getDatasetRange() ;
    int labelCount = this.symbols.length ;
    double[] tickValues = new double[labelCount] ;
    double step = (labelCount == 1) ? 0 : range.getLength() / (labelCount - 1) ;
    for (int i = 0 ; i < labelCount ; i++) {
      tickValues[i] = range.getLowerBound() + i * step ;
    }

    NumberRange realRange = (getZoomDatasetRange() == null) ? range : getZoomDatasetRange() ;
    List tmpLabels = new ArrayList() ;
    List tmpValueTicks = new ArrayList() ;
    boolean ignoreLeft = false ; //ߵǷ
    boolean addData = false ; //ǷѾ
    for (int i = 0 ; i < labelCount ; i++) {
      if (realRange.contains(tickValues[i])) {
        addData = true ;
        if (ignoreLeft && i > 0) {
          tmpLabels.add(null) ;
          tmpValueTicks.add(new Double(tickValues[i - 1])) ;
        }
        tmpLabels.add(this.symbols[i]) ;
        tmpValueTicks.add(new Double(tickValues[i])) ;
      } else {
        if (!addData) {
          ignoreLeft = true ;
        } else {
          tmpLabels.add(null) ;
          tmpValueTicks.add(new Double(tickValues[i])) ;
          break ;
        }
      }
    }

    if (tmpValueTicks.size() == 0) { //ѡķΧλ̶֮
      tmpValueTicks.add(new Double(realRange.getLowerBound())) ;
      tmpValueTicks.add(new Double(realRange.getUpperBound())) ;
    }

    this.tickLabels = (String[]) tmpLabels.toArray(new String[0]) ;
    this.valueTicks = new double[tmpValueTicks.size()] ;
    this.numberTicks = new String[tmpValueTicks.size()] ;
    for (int i = 0 ; i < tmpValueTicks.size() ; i++) {
      this.valueTicks[i] = ((Double) tmpValueTicks.get(i)).doubleValue() ;
      this.numberTicks[i] = tmpValueTicks.get(i).toString() ;
    }
  }

  /**
   * <zh>¡.</zh>
   * <en>Creates a clone of the axis.</en>
   *
   * @return A clone.
   *
   * @throws CloneNotSupportedException if some component of the axis does
   *         not support cloning.
   */
  public Object clone() throws CloneNotSupportedException {
    return super.clone() ;
  }

  /**
   * <zh>ԱǷһ.</zh>
   * <en>Tests this axis for equality with an arbitrary object.</en>
   *
   * @param obj <zh>ҪԵĶ (Ϊnull).</zh>
   * <en>The object (null permitted).</en>
   *
   * @return A boolean.
   */
  public boolean equals(Object obj) {
    if (obj == this) {
      return true ;
    }
    if (!(obj instanceof SymbolValueAxis)) {
      return false ;
    }
    if (!super.equals(obj)) {
      return false ;
    }

    SymbolValueAxis that = (SymbolValueAxis) obj ;
    if (!EqualsUtil.equals(this.symbols, that.symbols)) {
      return false ;
    }
    if (!EqualsUtil.equals(this.valueAxis, that.valueAxis)) {
      return false ;
    }

    return true ;
  }

  /**
   * <zh>hashcode.</zh>
   * <en>Returns a hash code for this object.</en>
   *
   * @return <zh>hashcode.</zh>
   * <en>A hash code.</en>
   */
  public int hashCode() {
    int result = super.hashCode() ;
    result = 29 * result + HashcodeUtil.hashCode(this.valueAxis) ;
    result = 29 * result + HashcodeUtil.hashCode(this.symbols) ;

    return result ;
  }

  /**
   * <zh>toString.</zh>
   * <en>Returns a string representation of the object.</en>
   *
   * @return A string representation of the object.
   */
  public String toString() {
    return "SymbolValueAxis" ;
  }

}
