package com.ijchart.xychart.chartinfo ;

import java.awt.Graphics2D ;
import java.awt.geom.Point2D ;
import java.io.Serializable ;
import java.util.ArrayList ;
import java.util.Collection ;
import java.util.Collections ;
import java.util.List ;

import com.ijchart.xychart.chartinfo.draw.IChangeChartShape ;
import com.ijchart.xychart.client.IJChart ;
import com.ijchart.xychart.constant.ChartShapeDrawType ;
import com.ijchart.xychart.event.ChartProgressEvent ;
import com.ijchart.xychart.event.ChartShapeChangeEvent ;
import com.ijchart.xychart.shape.IChartShape ;
import com.ijchart.xychart.util.EqualsUtil ;
import com.ijchart.xychart.util.HashcodeUtil ;
import com.ijchart.xychart.web.IAreaFragment ;

/**
 * <p>Title: IJChart</p>
 *
 * <p>Description: a chart library for the Java(tm) platform.</p>
 * <zh>ͼеÿԪضһͼζ{@link com.ijchart.xychart.shape.IChartShape} ,
 * ¼:⡢ͼplot.</zh>
 * <en>All elements in chart is a {@link com.ijchart.xychart.shape.IChartShape} ,
 * type is:title,legend,plot. </en>
 *
 * <p>Copyright: Copyright (c) 2013</p>
 *
 * <p>Company: </p>
 *
 * @author zhang feng min,shanghai china.
 * <a href="mailto:ijchartsh@gmail.com">Contract me by email.</a>
 * @version 1.0
 */
public class ChartShapeCollection extends AbstractShapeCollection implements Serializable,
  Cloneable {
  /**
   * <zh>л汾ID.</zh>
   * <en>For serialization.</en>
   */
  private static final long serialVersionUID = 4424033962018298524L ;

  /**
   * <zh>ȱʡx᷽.</zh>
   * <en>The default scale value of x-axis.</en>
   */
  public static final double DEFAULT_SCALE_X_AXIS = 1.0 ;

  /**
   * <zh>ȱʡy᷽.</zh>
   * <en>The default scale value of y-axis.</en>
   */
  public static final double DEFAULT_SCALE_Y_AXIS = 1.0 ;

  /**
   * <zh>߿ͼ.</zh>
   * <en>The border shape.</en>
   */
  private IChartShape borderShape ;

  /**
   * <zh>ɫͼ.</zh>
   * <en>The background shape.</en>
   */
  private IChartShape backgroundShape ;

  /**
   * <zh>ͼƬͼ.</zh>
   * <en>The background image shape.</en>
   */
  private IChartShape backgroundImageShape ;

  /**
   * <zh>ͼμ.</zh>
   * <en>The title shapes collection.</en>
   */
  private List titleShapesList ;

  /**
   * <zh>ͼͼμ.</zh>
   * <en>The legend shapes collection.</en>
   */
  private LegendShapeCollection legendShapes ;

  /**
   * <zh>plotͼμ.</zh>
   * <en>The plot shapes collection.</en>
   */
  private List plotShapesCollection ;

  /**
   * <zh>ͼεIJChart.</zh>
   * <en>The IJChart.</en>
   */
  private IJChart ijchart ;

  /**
   * <zh>ͼǷиı.</zh>
   * <en>A boolean flag that controls whether or not the shape is changed.</en>
   */
  private boolean shapeChanged ;

  /**
   * <zh>x᷽.</zh>
   * <en>The scale value of x-axis.</en>
   */
  private double scaleX ;

  /**
   * <zh>y᷽.</zh>
   * <en>The scale value of y-axis.</en>
   */
  private double scaleY ;

  /**
   * <zh>ͼθı.</zh>
   * <en>The shape change object.</en>
   */
  private IChangeChartShape changeChartShape ;

  /**
   * <zh>ȱʡ.</zh>
   * <en>No argument constructor.</en>
   */
  public ChartShapeCollection() {
    this.borderShape = null ;
    this.backgroundShape = null ;
    this.backgroundImageShape = null ;

    this.titleShapesList = new ArrayList() ;
    this.legendShapes = null ;
    this.plotShapesCollection = new ArrayList() ;

    this.ijchart = null ;
    this.shapeChanged = false ;
    this.changeChartShape = null ;

    this.scaleX = DEFAULT_SCALE_X_AXIS ;
    this.scaleY = DEFAULT_SCALE_Y_AXIS ;
  }

  /**
   * <zh>رͼƬͼ.</zh>
   * <en>Returns the background shape.</en>
   *
   * @return IChartShape <zh>ͼƬͼ(Ϊnull).</zh>
   * <en>The background shape (maybe null).</en>
   */
  public IChartShape getBackgroundShape() {
    return this.backgroundShape ;
  }

  /**
   * <zh>ñͼƬͼ.</zh>
   * <en>Sets the background shape.</en>
   *
   * @param shape IChartShape <zh>ͼƬͼ(Ϊnull).</zh>
   * <en>The background shape (null permitted).</en>
   */
  public void setBackgroundShape(IChartShape shape) {
    if (shape != null) {
      shape.setId(shapeIdSequence++) ;
      shape.setShapeDrawType(ChartShapeDrawType.CHART_BACKGROUND_SHAPE) ;
      shape.addChangeListener(this) ;
    }

    this.backgroundShape = shape ;
    fireChartShapeChanged(shape) ;
  }

  /**
   * <zh>رͼƬͼ.</zh>
   * <en>Returns the background image shape.</en>
   *
   * @return IChartShape <zh>ͼƬͼ(Ϊnull).</zh>
   * <en>The background image shape (maybe null).</en>
   */
  public IChartShape getBackgroundImageShape() {
    return this.backgroundImageShape ;
  }

  /**
   * <zh>ñͼƬͼ.</zh>
   * <en>Sets the background image shape.</en>
   *
   * @param shape IChartShape <zh>ͼƬͼ(Ϊnull).</zh>
   * <en>The background image shape (null permitted).</en>
   */
  public void setBackgroundImageShape(IChartShape shape) {
    if (shape != null) {
      shape.setId(shapeIdSequence++) ;
      shape.setShapeDrawType(ChartShapeDrawType.CHART_BACKGROUND_IMAGE_SHAPE) ;
      shape.addChangeListener(this) ;
    }

    this.backgroundImageShape = shape ;
    fireChartShapeChanged(shape) ;
  }

  /**
   * <zh>ñ߿ͼ.</zh>
   * <en>Sets the border shape.</en>
   *
   * @param borderShape IChartShape <zh>߿ͼ (Ϊnull).</zh>
   * <en>The border shape (null permitted).</en>
   */
  public void setBorderShape(IChartShape borderShape) {
    if (borderShape != null) {
      borderShape.setId(shapeIdSequence++) ;
      borderShape.setShapeDrawType(ChartShapeDrawType.CHART_BORDER_SHAPE) ;
      borderShape.addChangeListener(this) ;
    }

    this.borderShape = borderShape ;
    fireChartShapeChanged(borderShape) ;
  }

  /**
   * <zh>ñ߿ͼ.</zh>
   * <en>The border shape.</en>
   *
   * @return IChartShape <zh>߿ͼ (Ϊnull).</zh>
   * <en>The border shape (maybe null).</en>
   */
  public IChartShape getBorderShape() {
    return this.borderShape ;
  }

  /**
   * <zh>رͼμ,˼ϲ޸.</zh>
   * <en>Returns the list of title shapes and is unmodifiable.</en>
   *
   * @return List <zh>ͼμ,˼ϲ޸.</zh>
   * <en>The list of title shapes and is unmodifiable.</en>
   */
  public List getTitleShapesList() {
    return Collections.unmodifiableList(this.titleShapesList) ;
  }

  /**
   * <zh>һͼ.</zh>
   * <en>Add a title shape.</en>
   *
   * @param titleShape TitleShapeCollection <zh>ͼ (Ϊnull).</zh>
   * <en>A title shape (null permitted).</en>
   */
  public void addTitleShapes(TitleShapeCollection titleShape) {
    if (titleShape != null) {
      titleShape.addChangeListener(this) ;
    }

    this.titleShapesList.add(titleShape) ;
    fireChartShapeChanged(null) ;
  }

  /**
   * <zh>ص0titleͼμ.</zh>
   * <en>Returns the first title shapes collection.</en>
   *
   * @return TitleShapeCollection <zh>titleͼμ.</zh>
   * <en>The title shapes collection.</en>
   */
  public TitleShapeCollection getTitleShapes() {
    return getTitleShapes(0) ;
  }

  /**
   * <zh>ָtitleͼμ.</zh>
   * <en>Returns the title shapes collection of the specified index.</en>
   *
   * @param index int <zh>ֵ.</zh>
   * <en>The index.</en>
   *
   * @return TitleShapeCollection <zh>titleͼμ.</zh>
   * <en>The title shapes collection.</en>
   */
  public TitleShapeCollection getTitleShapes(int index) {
    return (TitleShapeCollection)this.titleShapesList.get(index) ;
  }

  /**
   * <zh>ͼͼμ.</zh>
   * <en>Returns the legend shapes collection.</en>
   *
   * @return LegendShapeCollection <zh>ͼͼμ.</zh>
   * <en>The legend shapes collection.</en>
   */
  public LegendShapeCollection getLegendShapes() {
    return this.legendShapes ;
  }

  /**
   * <zh>ͼͼμ.</zh>
   * <en>Sets the legend shapes collection.</en>
   *
   * @param legendShapes LegendShapeCollection <zh>ͼͼμ (Ϊnull).</zh>
   * <en>The legend shapes collection (null permitted).</en>
   */
  public void setLegendShapes(LegendShapeCollection legendShapes) {
    if (legendShapes != null) {
      legendShapes.addChangeListener(this) ;
    }

    this.legendShapes = legendShapes ;
    fireChartShapeChanged(null) ;
  }

  /**
   * <zh>plotͼμ.</zh>
   * <en>Adds the plot shapes collection.</en>
   *
   * @param plotShapes PlotShapeCollection <zh>plotͼμ (Ϊnull).</zh>
   * <en>The plot shapes collection (null permitted).</en>
   */
  public void addPlotShapes(PlotShapeCollection plotShapes) {
    if (plotShapes != null) {
      plotShapes.addChangeListener(this) ;
    }

    this.plotShapesCollection.add(plotShapes) ;
    fireChartShapeChanged(null) ;
  }

  /**
   * <zh>ص0plotͼμ.</zh>
   * <en>Returns the first plot shapes collection.</en>
   *
   * @return PlotShapeCollection <zh>plotͼμ.</zh>
   * <en>The plot shapes collection.</en>
   */
  public PlotShapeCollection getPlotShapes() {
    return getPlotShapes(0) ;
  }

  /**
   * <zh>ָplotͼμ.</zh>
   * <en>Returns the plot shapes collection of the specified index.</en>
   *
   * @param index int <zh>ֵ.</zh>
   * <en>The index.</en>
   *
   * @return PlotShapeCollection <zh>plotͼμ.</zh>
   * <en>The plot shapes collection.</en>
   */
  public PlotShapeCollection getPlotShapes(int index) {
    return (PlotShapeCollection)this.plotShapesCollection.get(index) ;
  }

  /**
   * <zh>еplotͼμ.</zh>
   * <en>Returns the shapes collection of all plot.</en>
   *
   * @return Collection <zh>˼ϲ޸.</zh>
   * <en>The collection is unmodifiable.</en>
   */
  public Collection getAllPlotShapes() {
    return Collections.unmodifiableCollection(this.plotShapesCollection) ;
  }

  /**
   * <zh>ɾеplotͼμ.</zh>
   * <en>Remove all plots shape collection.</en>
   */
  public void clearPlotShapes() {
    this.plotShapesCollection.clear() ;
    fireChartShapeChanged(null) ;
  }

  /**
   * <zh>زͼεIJChart.</zh>
   * <en>Returns the IJChart object.</en>
   *
   * @return IJChart <zh>ͼεIJChart (Ϊnull).</zh>
   * <en>The IJChart (never null).</en>
   */
  public IJChart getIJChart() {
    return this.ijchart ;
  }

  /**
   * <zh>òͼεIJChart.</zh>
   * <en>Sets the IJChart object.</en>
   *
   * @param chart IJChart <zh>ͼεIJChart (Ϊnull).</zh>
   * <en>The IJChart (null not permitted).</en>
   */
  public void setIJChart(IJChart chart) {
    if (chart == null) {
      throw new IllegalArgumentException("Null 'chart' argument.") ;
    }

    this.ijchart = chart ;
  }

  /**
   * <zh>ͼǷиı.</zh>
   * <en>Returns the boolean flag that controls whether or not the shape is changed.</en>
   *
   * @return boolean <zh>ֵ.</zh>
   * <en>A boolean.</en>
   */
  public boolean isShapeChanged() {
    return this.shapeChanged ;
  }

  /**
   * <zh>ͼǷиı,ıͼλ».</zh>
   * <en>Sets the boolean flag that controls whether or not the shape is changed.</en>
   *
   * @param flag boolean <zh>ֵ.</zh>
   * <en>A boolean.</en>
   */
  public void setShapeChanged(boolean flag) {
    this.shapeChanged = flag ;
  }

  /**
   * <zh>ǿͼǷиı,ûᵼͼ».</zh>
   * <en>Sets the boolean flag that the chart is changed.</en>
   */
  public void forceChartChanged() {
    this.shapeChanged = true ;
    fireChartShapeChanged(null) ;
  }

  /**
   * <zh>x᷽.</zh>
   * <en>Returns the scale value of x-axis.</en>
   *
   * @return double <zh>.</zh>
   * <en>The scale value.</en>
   */
  public double getScaleX() {
    return this.scaleX ;
  }

  /**
   * <zh>x᷽.</zh>
   * <en>Sets the scale value of x-axis.</en>
   *
   * @param scale double <zh>.</zh>
   * <en>The scale value.</en>
   */
  public void setScaleX(double scale) {
    if (scale <= 0 || scale > 1.0) {
      throw new IllegalArgumentException("Argument 'scale' must be in (0.0,1.0].") ;
    }

    this.scaleX = scale ;
  }

  /**
   * <zh>y᷽.</zh>
   * <en>Returns the scale value of y-axis.</en>
   *
   * @return double <zh>.</zh>
   * <en>The scale value.</en>
   */
  public double getScaleY() {
    return this.scaleY ;
  }

  /**
   * <zh>y᷽.</zh>
   * <en>Sets the scale value of y-axis.</en>
   *
   * @param scale double <zh>.</zh>
   * <en>The scale value.</en>
   */
  public void setScaleY(double scale) {
    if (scale <= 0 || scale > 1.0) {
      throw new IllegalArgumentException("Argument 'scale' must be in (0.0,1.0].") ;
    }

    this.scaleY = scale ;
  }

  /**
   * <zh>ͼθı.</zh>
   * <en>Returns the shape change object.</en>
   *
   * @return IChangeChartShape <zh>ͼθı (Ϊnull).</zh>
   * <en>The shape change object (maybe null).</en>
   */
  public IChangeChartShape getChangeChartShape() {
    return this.changeChartShape ;
  }

  /**
   * <zh>ͼθı.</zh>
   * <en>Sets the shape change object.</en>
   *
   * @param change IChangeChartShape <zh>ͼθı.</zh>
   * <en>The shape change object.</en>
   */
  public void setChangeChartShape(IChangeChartShape change) {
    this.changeChartShape = change ;
  }

  /**
   * <zh>λƼÿһͼ.</zh>
   * <en>Drawing all ChartShape in this collection.</en>
   *
   * @param g2 Graphics2D <zh>ͼ豸(ӡĻ) (Ϊnull).</zh>
   * <en>The graphics device (null not permitted).</en>
   */
  public void draw(Graphics2D g2) {
    if (this.changeChartShape != null) {
      this.changeChartShape.beforeDraw(this) ;
    }
    notifyProgressListener(ChartProgressEvent.PROGRESS_DRAWING_SHAPES_STARTED) ; //¼
    if (this.backgroundShape != null) {
      this.backgroundShape.draw(g2) ;
    }
    if (this.backgroundImageShape != null) {
      this.backgroundImageShape.draw(g2) ;
    }
    if (this.borderShape != null) {
      this.borderShape.draw(g2) ;
    }

    notifyProgressListener(ChartProgressEvent.PROGRESS_DRAWING_TITLE_SHAPES_STARTED) ; //¼
    for (int i = 0 ; i < this.titleShapesList.size() ; i++) {
      TitleShapeCollection titleShapes = (TitleShapeCollection)this.titleShapesList.get(i) ;
      if (titleShapes != null) {
        titleShapes.draw(g2) ;
      }
    }

    notifyProgressListener(ChartProgressEvent.PROGRESS_DRAWING_LEGEND_SHAPES_STARTED) ; //¼
    if (this.legendShapes != null) {
      this.legendShapes.draw(g2) ;
    }

    notifyProgressListener(ChartProgressEvent.PROGRESS_DRAWING_PLOT_SHAPES_STARTED) ; //¼
    if (this.plotShapesCollection != null) {
      for (int i = 0 ; i < this.plotShapesCollection.size() ; i++) {
        PlotShapeCollection plotShapes = (PlotShapeCollection)this.plotShapesCollection.get(i) ;
        plotShapes.draw(g2) ;
      }
    }

    notifyProgressListener(ChartProgressEvent.PROGRESS_DRAWING_SHAPES_FINISHED) ; //¼

    if (this.changeChartShape != null) {
      this.changeChartShape.afterDraw(this) ;
    }
  }

  /**
   * <zh>յͼζ仯֪ͨ.</zh>
   * <en>Receives notification of an {@link com.ijchart.xychart.shape.IChartShape} change event.</en>
   *
   * @param event ChartShapeChangeEvent <zh>ͼζ仯Ķ</zh>
   * <en>Information about the event.</en>
   */
  public void chartShapeChanged(ChartShapeChangeEvent event) {
    notifyListeners(event) ;
    this.shapeChanged = true ;
  }

  /**
   * <zh>֪ͨȼ.</zh>
   * <en>Notify progress listener.</en>
   *
   * @param eventType int <zh>¼.</zh>
   * <en>The chart progress event.</en>
   */
  private void notifyProgressListener(int eventType) {
    if (this.ijchart != null) {
      this.ijchart.notifyListeners(new ChartProgressEvent(this.ijchart, this, eventType)) ;
    }
  }

  /**
   * <zh>ƶͼεԭ㵽(x,y).</zh>
   * <en>Translates the origin of the all shape in this chart to the point (x,&nbsp;y)
   * in the current coordinate system. </en>
   *
   * @param  x double <zh>x.</zh>
   * <en>The x coordinate.</en>
   * @param  y double <zh>y.</zh>
   * <en>The y coordinate.</en>
   */
  public void translate(double x, double y) {
    if (x == 0 && y == 0) {
      return ;
    }

    super.translate(x, y) ;
    if (this.borderShape != null) {
      this.borderShape.translate(x, y) ;
    }
    if (this.backgroundShape != null) {
      this.backgroundShape.translate(x, y) ;
    }
    if (this.backgroundImageShape != null) {
      this.backgroundImageShape.translate(x, y) ;
    }

    for (int i = 0 ; i < this.titleShapesList.size() ; i++) {
      TitleShapeCollection titleShapes = (TitleShapeCollection)this.titleShapesList.get(i) ;
      if (titleShapes != null) {
        titleShapes.translate(x, y) ;
      }
    }
    if (this.legendShapes != null) {
      this.legendShapes.translate(x, y) ;
    }
    if (this.plotShapesCollection != null) {
      for (int i = 0 ; i < this.plotShapesCollection.size() ; i++) {
        PlotShapeCollection plotShapes = (PlotShapeCollection)this.plotShapesCollection.get(i) ;
        plotShapes.translate(x, y) ;
      }
    }
  }

  /**
   * <zh>еͼεļ.</zh>
   * <en>Remove all the shapes of the listener.</en>
   */
  public void clearShapesListener() {
    if (this.borderShape != null) {
      this.borderShape.removeAllChangeListener() ;
    }
    if (this.backgroundShape != null) {
      this.backgroundShape.removeAllChangeListener() ;
    }
    if (this.backgroundImageShape != null) {
      this.backgroundImageShape.removeAllChangeListener() ;
    }

    for (int i = 0 ; i < this.titleShapesList.size() ; i++) {
      TitleShapeCollection titleShapes = (TitleShapeCollection)this.titleShapesList.get(i) ;
      if (titleShapes != null) {
        titleShapes.clearShapesListener() ;
      }
    }
    if (this.legendShapes != null) {
      this.legendShapes.clearShapesListener() ;
    }
    if (this.plotShapesCollection != null) {
      for (int i = 0 ; i < this.plotShapesCollection.size() ; i++) {
        PlotShapeCollection plotShapes = (PlotShapeCollection)this.plotShapesCollection.get(i) ;
        plotShapes.clearShapesListener() ;
      }
    }
  }

  /**
   * <zh>ذpointͼ.</zh>
   * <en>Returns the {@link IChartShape} whose area contains the specified point.</en>
   *
   * @param point Point2D <zh>ҪԵĵ (Ϊnull).</zh>
   * <en>The point (null not permitted).</en>
   * @param drawType ChartShapeDrawType <zh>ͼεĻ (Ϊnull).</zh>
   * <en>The shape draw shape (null permitted).</en>
   *
   * @return IChartShape <zh>ͼ (Ϊnull).</zh>
   * <en>The chart shape (maybe null).</en>
   */
  public IChartShape findChartShape(Point2D point, ChartShapeDrawType drawType) {
    if (point == null) {
      throw new IllegalArgumentException("Null 'point' argument.") ;
    }
    if (getDrawArea() != null && !getDrawArea().contains(point)) {
      return null ;
    }

    Point2D newPoint = new Point2D.Double(point.getX() / this.scaleX, point.getY() / this.scaleY) ;
    IChartShape shape = null ;
    if (this.plotShapesCollection != null) {
      for (int i = 0 ; i < this.plotShapesCollection.size() ; i++) {
        PlotShapeCollection plotShapes = (PlotShapeCollection)this.plotShapesCollection.get(i) ;
        shape = plotShapes.findChartShape(newPoint, drawType) ;
        if (shape != null) {
          return shape ;
        }
      }
    }

    if (this.legendShapes != null) {
      shape = this.legendShapes.findChartShape(newPoint, drawType) ;
      if (shape != null) {
        return shape ;
      }
    }

    shape = findShape(this.titleShapesList, newPoint, drawType) ;
    if (shape != null) {
      return shape ;
    }

    boolean flag = drawType == null || drawType.equals(ChartShapeDrawType.ANY_SHAPE) ;
    if (flag || drawType.equals(ChartShapeDrawType.CHART_BORDER_SHAPE)) {
      if (this.borderShape != null && this.borderShape.contains(newPoint)) {
        return this.borderShape ;
      }
    }

    if (flag || drawType.equals(ChartShapeDrawType.CHART_BACKGROUND_IMAGE_SHAPE)) {
      if (this.backgroundImageShape != null && this.backgroundImageShape.contains(newPoint)) {
        return this.backgroundImageShape ;
      }
    }

    if (flag || drawType.equals(ChartShapeDrawType.CHART_BACKGROUND_SHAPE)) {
      if (this.backgroundShape != null && this.backgroundShape.contains(newPoint)) {
        return this.backgroundShape ;
      }
    }

    return null ;
  }

  /**
   * <zh>ͼεȵArea.</zh>
   * <en>Returns the hotspot areas.</en>
   *
   * @param drawTypes List <zh>ʾȵͼ.</zh>
   * <en>The types of hotspot shape.</en>
   * @param fragment IAreaFragment <zh>ʵֽӿ{@link IAreaFragment}Ķ,(Ϊnull)</zh>
   * <en>The {@link IAreaFragment} object (maybe null).</en>
   *
   * @return List <zh>ͼεȵArea (Ϊnull).</zh>
   * <en>The hotspot areas (never null).</en>
   */
  public List getHotspotAreas(List drawTypes, IAreaFragment fragment) {
    if (drawTypes == null) {
      throw new IllegalArgumentException("Null 'drawTypes' argument.") ;
    }

    List result = new ArrayList() ;
    if (drawTypes.contains(ChartShapeDrawType.CHART_BACKGROUND_SHAPE)) {
      addHotspotArea(result, this.backgroundShape, fragment,
                     ChartShapeDrawType.CHART_BACKGROUND_SHAPE) ;
    }
    if (drawTypes.contains(ChartShapeDrawType.CHART_BACKGROUND_IMAGE_SHAPE)) {
      addHotspotArea(result, this.backgroundImageShape, fragment,
                     ChartShapeDrawType.CHART_BACKGROUND_IMAGE_SHAPE) ;
    }
    if (drawTypes.contains(ChartShapeDrawType.CHART_BORDER_SHAPE)) {
      addHotspotArea(result, this.borderShape, fragment, ChartShapeDrawType.CHART_BORDER_SHAPE) ;
    }

    for (int i = 0 ; i < this.titleShapesList.size() ; i++) {
      TitleShapeCollection titleShapes = (TitleShapeCollection)this.titleShapesList.get(i) ;
      if (titleShapes != null) {
        result.addAll(titleShapes.getHotspotAreas(drawTypes, fragment)) ;
      }
    }
    if (this.legendShapes != null) {
      result.addAll(this.legendShapes.getHotspotAreas(drawTypes, fragment)) ;
    }
    if (this.plotShapesCollection != null) {
      for (int i = 0 ; i < this.plotShapesCollection.size() ; i++) {
        PlotShapeCollection plotShapes = (PlotShapeCollection)this.plotShapesCollection.get(i) ;
        if (plotShapes != null) {
          result.addAll(plotShapes.getHotspotAreas(drawTypes, fragment)) ;
        }
      }
    }

    return result ;
  }

  /**
   * <zh>еͼΪѡ״̬.</zh>
   * <en>Sets all shapes to unselected.</en>
   */
  public void clearSelectedShapes() {
    clearSelectedShape(this.borderShape) ;
    clearSelectedShape(this.backgroundShape) ;
    clearSelectedShape(this.backgroundImageShape) ;

    for (int i = 0 ; i < this.titleShapesList.size() ; i++) {
      TitleShapeCollection titleShapes = (TitleShapeCollection)this.titleShapesList.get(i) ;
      if (titleShapes != null) {
        titleShapes.clearSelectedShapes() ;
      }
    }
    this.legendShapes.clearSelectedShapes() ;
    if (this.plotShapesCollection != null) {
      for (int i = 0 ; i < this.plotShapesCollection.size() ; i++) {
        PlotShapeCollection plotShapes = (PlotShapeCollection)this.plotShapesCollection.get(i) ;
        plotShapes.clearSelectedShapes() ;
      }
    }
  }

  /**
   * <zh>hashCode.</zh>
   * <en>Override hasCode method.</en>
   * @return int
   */
  public int hashCode() {
    double result = HashcodeUtil.hashCode(this.borderShape) ;
    result = 29 * result + HashcodeUtil.hashCode(this.backgroundShape) ;
    result = 29 * result + HashcodeUtil.hashCode(this.backgroundImageShape) ;
    result = 29 * result + HashcodeUtil.hashCode(this.titleShapesList) ;
    result = 29 * result + HashcodeUtil.hashCode(this.legendShapes) ;
    result = 29 * result + HashcodeUtil.hashCode(this.plotShapesCollection) ;
    result = 29 * result + HashcodeUtil.hashCode(this.shapeChanged) ;
    result = 29 * result + 37 * this.scaleX ;
    result = 29 * result + 37 * this.scaleY ;
    result = 29 * result + HashcodeUtil.hashCode(this.changeChartShape) ;

    return (int) result ;
  }

  /**
   * <zh>equals.</zh>
   * <en>Override equals method.</en>
   * @param obj Object
   * @return boolean
   */
  public boolean equals(Object obj) {
    if (this == obj) {
      return true ;
    }

    if (!(obj instanceof ChartShapeCollection)) {
      return false ;
    }
    if (!super.equals(obj)) {
      return false ;
    }

    ChartShapeCollection other = (ChartShapeCollection) obj ;
    if (!EqualsUtil.equals(this.borderShape, other.borderShape)) {
      return false ;
    }
    if (!EqualsUtil.equals(this.backgroundShape, other.backgroundShape)) {
      return false ;
    }
    if (!EqualsUtil.equals(this.backgroundImageShape, other.backgroundImageShape)) {
      return false ;
    }
    if (!EqualsUtil.equals(this.titleShapesList, other.titleShapesList)) {
      return false ;
    }
    if (!EqualsUtil.equals(this.legendShapes, other.legendShapes)) {
      return false ;
    }
    if (!EqualsUtil.equals(this.plotShapesCollection, other.plotShapesCollection)) {
      return false ;
    }
    if (this.shapeChanged != other.shapeChanged) {
      return false ;
    }
    if (this.scaleX != other.scaleX) {
      return false ;
    }
    if (this.scaleY != other.scaleY) {
      return false ;
    }
    if (!EqualsUtil.equals(this.changeChartShape, other.changeChartShape)) {
      return false ;
    }

    return true ;
  }

  /**
   * <zh>¡.</zh>
   * <en>Creates and returns a copy of this object.</en>
   *
   * @return object   a clone of this instance.
   * @throws java.lang.CloneNotSupportedException
   */
  public Object clone() throws CloneNotSupportedException {
    return super.clone() ;
  }

  /**
   * <zh>toString.</zh>
   * <en>Override toString method.</en>
   * @return String
   */
  public String toString() {
    return "[borderShape=" + this.borderShape
      + ",backgroundShape=" + this.backgroundShape
      + ",backgroundImageShape=" + this.backgroundImageShape
      + ",titleShape=" + this.titleShapesList
      + ",legendShape=" + this.legendShapes
      + ",plotShape=" + this.plotShapesCollection
      + ",scaleX=" + this.scaleX
      + ",scaleY=" + this.scaleY
      + "]" ;
  }

}
