package com.ijchart.xychart.data ;

import java.io.Serializable ;
import java.util.Date ;

import com.ijchart.xychart.util.EqualsUtil ;
import com.ijchart.xychart.util.HashcodeUtil ;

/**
 * <p>Title: IJChart</p>
 *
 * <p>Description: a chart library for the Java(tm) platform.</p>
 * <zh>ͼݶ,ⲿǲ޸ĵ.</zh>
 * <en>Date ChartData object,this class is immutable.</en>
 *
 * <p>Copyright: Copyright (c) 2013</p>
 *
 * <p>Company: </p>
 *
 * @author zhang feng min,shanghai china.
 * <a href="mailto:ijchartsh@gmail.com">Contract me by email.</a>
 * @version 1.0
 */
public class DateChartData extends AbstractChartData implements Cloneable, Serializable {
  /**
   * <zh>л汾ID.</zh>
   * <en>For serialization.</en>
   */
  private static final long serialVersionUID = -2970092047088215956L ;

  /**
   * <zh>ֵ.</zh>
   * <en>chart data field.</en>
   */
  private Date date ;

  /**
   * <zh></zh>
   * <en>Constructor</en>
   *
   * @param date Date <zh>ڶ (Ϊnull)</zh>
   * <en>Date object (null permitted).</en>
   */
  public DateChartData(Date date) {
    this.date = date ;
  }

  /**
   * <zh>ڶ.</zh>
   * <en>Get Date type of chart data.</en>
   *
   * @return Date <zh> (Ϊnull).</zh>
   * <en>The date (maybe null).</en>
   */
  public Date getDate() {
    return this.date ;
  }

  /**
   * <zh>رǩַ.</zh>
   * <en>Returns the label text.</en>
   *
   * @return String <zh>ǩַ (Ϊnull).</zh>
   * <en>The label text (maybe null).</en>
   */
  public String getLabelText() {
    return formatDate(this.date) ;
  }

  /**
   * <zh>ͼȵareaĺ.</zh>
   * <en>Returns the function arguments of hotspot area.</en>
   *
   * @return String <zh> (Ϊnull).</zh>
   * <en>The function arguments (maybe null).</en>
   */
  public String getAreaFunctionArguments() {
    return formatDateForFunction(this.date) ;
  }

  /**
   * <zh>ͼȵareaurl.</zh>
   * <en>Returns the url for hotspot area.</en>
   *
   * @return String <zh>url (Ϊnull).</zh>
   * <en>The url fragment (maybe null).</en>
   */
  public String getAreaURLFragment() {
    return "date=" + formatDate(this.date) ;
  }

  /**
   * <zh>ʵComparableӿڵcompareTo().</zh>
   * <en>Implements the compareTo() method of interface Comparable.</en>
   *
   * @param o Object <zh>ҪȽϵĶ.</zh>
   * <en>The Object to be compared.</en>
   * @return int <zh>ֵΪʾС,0ʾ,ʾ.</zh>
   * <en>A negative integer, zero, or a positive integer as this object	is less than, equal to, or
   * greater than the specified object.</en>
   *
   * @throws ClassCastException if the specified object's type prevents it
   *         from being compared to this Object.
   */
  public int compareTo(Object o) {
    if (o == null) {
      return -1 ;
    } else {
      if (!(o instanceof DateChartData)) {
        throw new ClassCastException("Must be 'DateChartData'.") ;
      }

      DateChartData other = (DateChartData) o ;
      if (this.date == null) {
        return -1 ;
      }

      return this.date.compareTo(other.date) ;
    }
  }

  /**
   * <zh>Ƿ.</zh>
   * <en>Tests this instance for equality with an arbitrary object.</en>
   *
   * @param obj  the object (null permitted).
   *
   * @return A boolean.
   */
  public boolean equals(Object obj) {
    if (obj == this) {
      return true ;
    }
    if (!(obj instanceof DateChartData)) {
      return false ;
    }
    if (!super.equals(obj)) {
      return false ;
    }

    DateChartData that = (DateChartData) obj ;
    if (EqualsUtil.equals(this.date, that.date)) {
      return false ;
    }

    return true ;
  }

  /**
   * <zh>hashcodeֵ.</zh>
   * <en>Returns a hash code value for the object.</en>
   *
   * @return int <zh>hashcodeֵ.</zh>
   * <en>A hash code value for this object.</en>
   */
  public int hashCode() {
    int result = super.hashCode() ;
    result = 37 * result + HashcodeUtil.hashCode(this.date) ;

    return result ;
  }

  /**
   * <zh>ؿ¡Ķ.</zh>
   * <en>Creates and returns a copy of this object.</en>
   * @return object   a clone of this instance.
   * @exception  CloneNotSupportedException  if the object's class does not
   *               support the Cloneable interface. Subclasses
   *               that override the clone method can also
   *               throw this exception to indicate that an instance cannot
   *               be cloned.
   */
  public Object clone() throws CloneNotSupportedException {
    return super.clone() ;
  }

  /**
   * <zh>toString.</zh>
   * <en>Returns a string representation of the object.</en>
   *
   * @return  a string representation of the object.
   */
  public String toString() {
    return "[date=" + getLabelText() + "]" ;
  }
}
