package com.ijchart.xychart.data.group ;

import java.io.Serializable ;
import java.util.List ;

import com.ijchart.xychart.data.IChartData ;
import com.ijchart.xychart.util.EqualsUtil ;
import com.ijchart.xychart.util.HashcodeUtil ;

/**
 * <p>Title: IJChart</p>
 *
 * <p>Description: a chart library for the Java(tm) platform.</p>
 * <zh>ݵĻ,˶ǲ޸ĵ..</zh>
 * <en>The base class for group data,this class is immutable.</en>
 * <p>Copyright: Copyright (c) 2013</p>
 *
 * <p>Company: </p>
 *
 * @author zhang feng min,shanghai china.
 * <a href="mailto:ijchartsh@gmail.com">Contract me by email.</a>
 * @version 1.0
 */
public abstract class AbstractGroupChartData implements IChartData, Cloneable, Serializable {
  /**
   * <zh>л汾ID.</zh>
   * <en>For serialization.</en>
   */
  private static final long serialVersionUID = -6066733976026205484L ;

  /**
   * <zh>.</zh>
   * <en>The group of this chart data.</en>
   */
  private Object group ;

  /**
   * <zh>ݼ.</zh>
   * <en>The collection of datas.</en>
   */
  private List datas ;

  /**
   * <zh>.</zh>
   * <en>The constructor.</en>
   *
   * @param group Object <zh> (Ϊnull).</zh>
   * <en>The group of this chart data (null permitted).</en>
   */
  public AbstractGroupChartData(Object group) {
    this(group, null) ;
  }

  /**
   * <zh>.</zh>
   * <en>The constructor.</en>
   *
   * @param group Object <zh> (Ϊnull).</zh>
   * <en>The group of this chart data (null permitted).</en>
   * @param datas List <zh>ݼ (Ϊnull).</zh>
   * <en>The collection of datas (null permitted).</en>
   */
  public AbstractGroupChartData(Object group, List datas) {
    this(group, datas, null) ;
  }

  /**
   * <zh>.</zh>
   * <en>The constructor.</en>
   *
   * @param group Object <zh> (Ϊnull).</zh>
   * <en>The group of this chart data (null permitted).</en>
   * @param datas List <zh>ݼ (Ϊnull).</zh>
   * <en>The collection of datas (null permitted).</en>
   * @param constraint Class <zh>datasÿClass (Ϊnull).</zh>
   * <en>The data in collection datas must be constraint (null permitted).</en>
   */
  public AbstractGroupChartData(Object group, List datas, Class constraint) {
    this.group = group ;
    this.datas = datas ;

    if (datas != null && constraint != null) {
      for (int i = 0 ; i < datas.size() ; i++) {
        Object obj = datas.get(i) ;
        if (obj != null && !constraint.isInstance(obj)) {
          String msg = "Must be '" + constraint.getClass().getName() + "',but is '"
                       + obj.getClass().getName() + "'" ;
          throw new IllegalArgumentException(msg) ;
        }
      }
    }
  }

  /**
   * <zh>ط.</zh>
   * <en>Returns the group of this chart data.</en>
   *
   * @return Object <zh> (Ϊnull).</zh>
   * <en>The group of this chart data (never null).</en>
   */
  public Object getGroup() {
    return this.group ;
  }

  /**
   * <zh>ݼ.</zh>
   * <en>Returns the collection of datas.</en>
   *
   * @return List <zh>ݼ (Ϊnull).</zh>
   * <en>The collection of datas (maybe null).</en>
   */
  public List getDatas() {
    return this.datas ;
  }

  /**
   * <zh>رǩַ.</zh>
   * <en>Returns the label text.</en>
   *
   * @return String <zh>ǩַ (Ϊnull).</zh>
   * <en>The label text (maybe null).</en>
   */
  public String getLabelText() {
    return null ;
  }

  /**
   * <zh>ͼȵareaĺ.</zh>
   * <en>Returns the function arguments of hotspot area.</en>
   *
   * @return String <zh> (Ϊnull).</zh>
   * <en>The function arguments (maybe null).</en>
   */
  public String getAreaFunctionArguments() {
    return null ;
  }

  /**
   * <zh>ͼȵareaurl.</zh>
   * <en>Returns the url for hotspot area.</en>
   *
   * @return String <zh>url (Ϊnull).</zh>
   * <en>The url fragment (maybe null).</en>
   */
  public String getAreaURLFragment() {
    return null ;
  }

  /**
   * <zh>رǩַ.</zh>
   * <en>Returns the label text.</en>
   *
   * @param index int <zh>ֵ.</zh>
   * <en>The data index.</en>
   * @return String <zh>ǩַ (Ϊnull).</zh>
   * <en>The label text (maybe null).</en>
   */
  public String toLabelText(int index) {
    Object value = null ;
    if (index < this.datas.size()) {
      value = this.datas.get(index) ;
    }

    return group + "," + ((value == null) ? "" : value.toString()) ;
  }

  /**
   * <zh>ʵComparableӿڵcompareTo().</zh>
   * <en>Implements the compareTo() method of interface Comparable.</en>
   *
   * @param o Object <zh>ҪȽϵĶ.</zh>
   * <en>The Object to be compared.</en>
   * @return int <zh>ֵΪʾС,0ʾ,ʾ.</zh>
   * <en>A negative integer, zero, or a positive integer as this object	is less than, equal to, or
   * greater than the specified object.</en>
   *
   * @throws ClassCastException if the specified object's type prevents it
   *         from being compared to this Object.
   */
  public int compareTo(Object o) {
    throw new UnsupportedOperationException() ;
  }

  /**
   * <zh>Ƿ.</zh>
   * <en>Tests this instance for equality with an arbitrary object.</en>
   *
   * @param obj  the object (null permitted).
   *
   * @return A boolean.
   */
  public boolean equals(Object obj) {
    if (obj == this) {
      return true ;
    }
    if (!(obj instanceof AbstractGroupChartData)) {
      return false ;
    }
    if (!super.equals(obj)) {
      return false ;
    }

    AbstractGroupChartData that = (AbstractGroupChartData) obj ;
    if (EqualsUtil.equals(this.group, that.group)) {
      return false ;
    }
    if (EqualsUtil.equals(this.datas, that.datas)) {
      return false ;
    }

    return true ;
  }

  /**
   * <zh>hashcodeֵ.</zh>
   * <en>Returns a hash code value for the object.</en>
   *
   * @return int <zh>hashcodeֵ.</zh>
   * <en>A hash code value for this object.</en>
   */
  public int hashCode() {
    int result = HashcodeUtil.hashCode(this.group) ;
    result = 37 * result + HashcodeUtil.hashCode(this.datas) ;

    return result ;
  }

  /**
   * <zh>ؿ¡Ķ.</zh>
   * <en>Creates and returns a copy of this object.</en>
   * @return object   a clone of this instance.
   * @exception  CloneNotSupportedException  if the object's class does not
   *               support the Cloneable interface. Subclasses
   *               that override the clone method can also
   *               throw this exception to indicate that an instance cannot
   *               be cloned.
   */
  public Object clone() throws CloneNotSupportedException {
    return super.clone() ;
  }

  /**
   * <zh>toString.</zh>
   * <en>Returns a string representation of the object.</en>
   *
   * @return  a string representation of the object.
   */
  public String toString() {
    return "[group=" + group
      + ",datas size=" + ((this.datas == null) ? 0 : this.datas.size())
      + "]" ;
  }

}
