package com.ijchart.xychart.format ;

import java.io.Serializable ;

import com.ijchart.xychart.util.EqualsUtil ;
import com.ijchart.xychart.util.HashcodeUtil ;

/**
 * <p>Title: IJChart</p>
 *
 * <p>Description: a chart library for the Java(tm) platform.</p>
 * <zh>滻ĳֵΪһֵ.</zh>
 * <en>Replace a value to other.</en>
 *
 * <p>Copyright: Copyright (c) 2013</p>
 *
 * <p>Company: </p>
 *
 * @author zhang feng min,shanghai china.
 * <a href="mailto:ijchartsh@gmail.com">Contract me by email.</a>
 * @version 1.0
 */
public class ReplaceFormatter implements IFormatter, Serializable {
  /**
   * <zh>л汾ID.</zh>
   * <en>For serialization.</en>
   */
  private static final long serialVersionUID = -1474052070375729515L ;

  /**
   * <zh>Դֵ.</zh>
   * <en>The source values.</en>
   */
  private Object[] sourceValues ;

  /**
   * <zh>滻ֵ.</zh>
   * <en>The replaced values.</en>
   */
  private Object[] replaceValues ;

  /**
   * <zh>.</zh>
   * <en>The constructor.</en>
   *
   * @param sourceValues Object[] <zh>Դֵ (Ϊnull).</zh>
   * <en>The source values (null not permitted).</en>
   * @param replaceValues Object[] <zh>滻ֵ (Ϊnull).</zh>
   * <en>The replaced values (null not permitted).</en>
   */
  public ReplaceFormatter(Object[] sourceValues, Object[] replaceValues) {
    if (sourceValues == null) {
      throw new IllegalArgumentException("Null 'sourceValues' argument.") ;
    }
    if (replaceValues == null) {
      throw new IllegalArgumentException("Null 'replaceValues' argument.") ;
    }

    this.sourceValues = sourceValues ;
    this.replaceValues = replaceValues ;
  }

  /**
   * <zh>Դֵ.</zh>
   * <en>Returns the source values.</en>
   *
   * @return Object[] <zh>Դֵ.</zh>
   * <en>The source values.</en>
   */
  public Object[] getSourceValues() {
    return this.sourceValues ;
  }

  /**
   * <zh>ر滻ֵ.</zh>
   * <en>Returns the replaced values.</en>
   *
   * @return Object[] <zh>滻ֵ.</zh>
   * <en>The replaced values.</en>
   */
  public Object[] getReplaceValues() {
    return this.replaceValues ;
  }

  /**
   * <zh>ʽһjava.lang.NumberΪַ.ĳֵһϵж(ϵıǩ
   * ̶ֵ),ʽʱԸindexֵĲͬʽΪַͬ.numberֵΪLong.MAX_VALUE.
   * indexʱûʹ,Ҫʹ,ɼ̳д.</zh>
   * <en>Formatted a java.lang.Number object to a string. If a value is composed of a
   * series of objects (such as the coordinates of the axis labels or scale value),
   * the format could be based on index.The max value of this object is Long.MAX_VALUE.
   * Parameter index do not have to use,to use,extends this class.</en>
   *
   * @param obj Object <zh>ҪʽĶ (Ϊnull).</zh>
   * <en>The object to be formatted (null permitted).</en>
   * @param index int <zh>һϵֵ˳,0ʼ.˲ûʹ.</zh>
   * <en>The index value of a series objects.
   * The first is at index 0, the next is at 1, and so on.This parameter is not used.</en>
   *
   * @return String <zh>ʽַ.</zh>
   * <en>A formatted Number string.</en>
   */
  public String format(Object obj, int index) {
    if (obj == null) {
      return "" ;
    }

    for (int i = 0 ; i < this.sourceValues.length ; i++) {
      if (this.sourceValues[i].equals(obj)) {
        Object toValue = this.replaceValues[i] ;
        if (toValue != null) {
          return toValue.toString() ;
        } else {
          return "" ;
        }
      }
    }

    return null ;
  }

  /**
   * <zh>Ƿ.</zh>
   * <en>Tests this object for equality with an arbitrary object.</en>
   *
   * @param obj <zh>ҪȽϵһ (Ϊnull).</zh>
   * <en>The object to test against (null permitted).</en>
   *
   * @return A boolean.
   */
  public boolean equals(Object obj) {
    if (this == obj) {
      return true ;
    }
    if (!(obj instanceof ReplaceFormatter)) {
      return false ;
    }
    if (!super.equals(obj)) {
      return false ;
    }

    ReplaceFormatter that = (ReplaceFormatter) obj ;
    if (!EqualsUtil.equals(this.sourceValues, that.sourceValues)) {
      return false ;
    }
    if (!EqualsUtil.equals(this.replaceValues, that.replaceValues)) {
      return false ;
    }

    return true ;
  }

  /**
   * <zh>hashcode.</zh>
   * <en>Returns a hash code.</en>
   *
   * @return A hash code.
   */
  public int hashCode() {
    int hashcode = super.hashCode() ;
    hashcode = 37 * hashcode + HashcodeUtil.hashCode(this.sourceValues) ;
    hashcode = 37 * hashcode + HashcodeUtil.hashCode(this.replaceValues) ;

    return hashcode ;
  }

  /**
   * <zh>toString.</zh>
   * <en>Returns a string representation of this Range.</en>
   *
   * @return A String.
   */
  public String toString() {
    return "ReplaceFormatter" ;
  }

}
