package com.ijchart.xychart.marker ;

import java.awt.Paint ;
import java.awt.Stroke ;
import java.io.Serializable ;

import com.ijchart.xychart.event.MarkerChangeEvent ;
import com.ijchart.xychart.util.HashcodeUtil ;

/**
 * <p>Title: IJChart</p>
 *
 * <p>Description: a chart library for the Java(tm) platform.</p>
 * <zh>ʾ(category)marker.
 * <br>
 * Ϊ˱֤ȷл,datasetRowKeyǿл.
 * </zh>
 * <en>A marker for a category.
 * <br>
 * Note that for serialization to work correctly, the category key must be an
 * instance of a serializable class.</en>
 *
 * <p>Copyright: Copyright (c) 2013</p>
 *
 * <p>Company: </p>
 *
 * @author zhang feng min,shanghai china.
 * <a href="mailto:ijchartsh@gmail.com">Contract me by email.</a>
 * @version 1.0
 */
public class CategoryMarker extends AbstractMarker implements Cloneable, Serializable {
  /**
   * <zh>л汾ID.</zh>
   * <en>For serialization.</en>
   */
  private static final long serialVersionUID = -3407236529360020764L ;

  /**
   * <zh>ȱʡǷʾΪһֱ,һ.</zh>
   * <en>A default flag that the marker should be drawn as a line rather than a region.</en>
   */
  public static final boolean DEFAULT_DRAW_AS_LINE = false ;

  /**
   * <zh>ݼrowKeycolKey.</zh>
   * <en>The dataset rowKey or colKey value.</en>
   */
  private Object datasetRowOrColumnKey ;

  /**
   * <zh>ʾΪһֱ,һ.</zh>
   * <en>A hint that the marker should be drawn as a line rather than a region.</en>
   */
  private boolean drawAsLine = false ;

  /**
   * <zh>һµmarker.</zh>
   * <en>Creates a new category marker for the specified category.</en>
   *
   * @param datasetRowOrColumnKey Object <zh>ݼrowKeycolKey (Ϊnull).</zh>
   * <en>The dataset rowKey or colKey value (null not permitted).</en>
   */
  public CategoryMarker(Object datasetRowOrColumnKey) {
    if (datasetRowOrColumnKey == null) {
      throw new IllegalArgumentException("Null 'datasetRowOrColumnKey' argument.") ;
    }

    this.datasetRowOrColumnKey = datasetRowOrColumnKey ;
    this.drawAsLine = DEFAULT_DRAW_AS_LINE ;
  }

  /**
   * <zh>һµmarker.</zh>
   * <en>Creates a new category marker.</en>
   *
   * @param datasetRowOrColumnKey Object <zh>ݼrowKeycolKey.</zh>
   * <en>The dataset rowKey or colKey value.</en>
   * @param fillPaint <zh>ʹõPaint (Ϊnull).</zh>
   * <en>The fill paint (null not permitted).</en>
   */
  public CategoryMarker(Object datasetRowOrColumnKey, Paint fillPaint) {
    super(fillPaint) ;
    if (datasetRowOrColumnKey == null) {
      throw new IllegalArgumentException("Null 'datasetRowOrColumnKey' argument.") ;
    }

    this.datasetRowOrColumnKey = datasetRowOrColumnKey ;
    this.drawAsLine = DEFAULT_DRAW_AS_LINE ;
  }

  /**
   * <zh>һµmarker.</zh>
   * <en>Creates a new category marker.</en>
   *
   * @param datasetRowOrColumnKey Object <zh>ݼrowKeycolKey.</zh>
   * <en>The dataset rowKey or colKey value.</en>
   * @param fillPaint <zh>ʹõpaint (Ϊnull).</zh>
   * <en>The fill paint (null not permitted).</en>
   * @param borderPaint <zh>ʹõPaint (Ϊnull).</zh>
   * <en>The border paint (null permitted).</en>
   * @param borderStroke <zh>ʹõStroke (Ϊnull).</zh>
   * <en>The border stroke (null permitted).</en>
   * @param fillAlpha <zh>䱳alphaֵ,ڱ[0.0,1.0].</zh>
   * <en>The alpha transparency (must be in the range 0.0f to 1.0f).</en>
   */
  public CategoryMarker(Object datasetRowOrColumnKey, Paint fillPaint, Paint borderPaint,
                        Stroke borderStroke, float fillAlpha) {
    super(fillPaint, borderPaint, borderStroke, fillAlpha) ;
    if (datasetRowOrColumnKey == null) {
      throw new IllegalArgumentException("Null 'datasetRowOrColumnKey' argument.") ;
    }

    this.datasetRowOrColumnKey = datasetRowOrColumnKey ;
    this.drawAsLine = DEFAULT_DRAW_AS_LINE ;
  }

  /**
   * <zh>ݼrowKeycolKey.</zh>
   * <en>Returns the dataset rowKey or colKey value.</en>
   *
   * @return Object <zh>ݼrowKeycolKey (Ϊnull).</zh>
   * <en>The dataset rowKey or colKey value (never null).</en>
   */
  public Object getKey() {
    return this.datasetRowOrColumnKey ;
  }

  /**
   * <zh>ݼrowKeycolKey,м{@link MarkerChangeEvent}¼.</zh>
   * <en>Sets the dataset rowKey and sends a {@link MarkerChangeEvent}
   * to all registered listeners.</en>
   *
   * @param datasetRowOrColumnKey <zh>ݼrowKeycolKey.</zh>
   * <en>The dataset rowKey or colKey value.</en>
   */
  public void setKey(Object datasetRowOrColumnKey) {
    if (datasetRowOrColumnKey == null) {
      throw new IllegalArgumentException("Null 'datasetRowOrColumnKey' argument.") ;
    }

    this.datasetRowOrColumnKey = datasetRowOrColumnKey ;
    notifyListeners(new MarkerChangeEvent(this)) ;
  }

  /**
   * <zh>ǷʾΪһֱ,һ.</zh>
   * <en>Returns the flag that controls whether the marker is drawn as a region or a line.</en>
   *
   * @return boolean <zh>ֵ.</zh>
   * <en>A boolean.</en>
   */
  public boolean isDrawAsLine() {
    return this.drawAsLine ;
  }

  /**
   * <zh>ǷʾΪһֱ,һ,м{@link MarkerChangeEvent}¼.</zh>
   * <en>Sets the flag that controls whether the marker is drawn as a region or
   * as a line, and sends a {@link MarkerChangeEvent} to all registered
   * listeners.</en>
   *
   * @param drawAsLine boolean <zh>ֵ.</zh>
   * <en>A boolean.</en>
   */
  public void setDrawAsLine(boolean drawAsLine) {
    if (this.drawAsLine != drawAsLine) {
      this.drawAsLine = drawAsLine ;
      notifyListeners(new MarkerChangeEvent(this)) ;
    }
  }

  /**
   * <zh>hashcodeֵ.</zh>
   * <en>Returns a hash code value for the object.</en>
   *
   * @return int The hashcode value.
   */
  public int hashCode() {
    int result = super.hashCode() ;
    result = 37 * result + HashcodeUtil.hashCode(this.datasetRowOrColumnKey) ;
    result = 37 * result + HashcodeUtil.hashCode(this.drawAsLine) ;

    return result ;
  }

  /**
   * <zh>Ƿ.</zh>
   * <en>Tests the marker for equality with an arbitrary object.</en>
   *
   * @param obj  the object (null permitted).
   *
   * @return A boolean.
   */
  public boolean equals(Object obj) {
    if (obj == null) {
      return false ;
    }
    if (!(obj instanceof CategoryMarker)) {
      return false ;
    }
    if (!super.equals(obj)) {
      return false ;
    }

    CategoryMarker that = (CategoryMarker) obj ;
    if (!this.datasetRowOrColumnKey.equals(that.datasetRowOrColumnKey)) {
      return false ;
    }
    if (this.drawAsLine != that.drawAsLine) {
      return false ;
    }

    return true ;
  }

  /**
   * <zh>Ʊ.</zh>
   * <en>Creates a clone of the marker.</en>
   *
   * @return <zh>ƺĶ</zh>
   * <en>A clone.</en>
   *
   * @throws CloneNotSupportedException never.
   */
  public Object clone() throws CloneNotSupportedException {
    return super.clone() ;
  }

  /**
   * <zh>toString.</zh>
   * <en>Returns a string representation of the object.  </en>
   *
   * @return  a string representation of the object.
   */
  public String toString() {
    return "[datasetRowOrColumnKey=" + this.datasetRowOrColumnKey
      + ",drawAsLine=" + this.drawAsLine + "]" ;
  }
}
