package com.ijchart.xychart.plot ;

import java.awt.Color ;
import java.awt.Graphics2D ;
import java.awt.Paint ;
import java.awt.geom.GeneralPath ;
import java.awt.geom.Rectangle2D ;
import java.io.Serializable ;
import java.util.ArrayList ;
import java.util.List ;

import com.ijchart.xychart.axis.AbstractAxis ;
import com.ijchart.xychart.axis.IAxis ;
import com.ijchart.xychart.axis.category.CategoryAxis ;
import com.ijchart.xychart.axis.category.CategoryAxis3D ;
import com.ijchart.xychart.axis.value.AbstractValueAxis ;
import com.ijchart.xychart.chartinfo.AxisShapeCollection ;
import com.ijchart.xychart.chartinfo.PlotShapeCollection ;
import com.ijchart.xychart.chartinfo.RenderShapeCollection ;
import com.ijchart.xychart.constant.ChartOrientation ;
import com.ijchart.xychart.constant.PlotType ;
import com.ijchart.xychart.render.category.AbstractCategoryRender ;
import com.ijchart.xychart.shape.GeneralPathChartShape ;
import com.ijchart.xychart.shape.IChartShape ;
import com.ijchart.xychart.shape.LineChartShape ;
import com.ijchart.xychart.shape.ParallelepipedTextChartShape ;
import com.ijchart.xychart.ui.IEffect3D ;
import com.ijchart.xychart.ui.RectangleEdge ;
import com.ijchart.xychart.util.StrokeUtil ;

/**
 * <p>Title: IJChart</p>
 *
 * <p>Description: a chart library for the Java(tm) platform.</p>
 * <zh>ʾֱ{@link com.ijchart.xychart.axis.category.CategoryAxis3D},
 * {@link com.ijchart.xychart.axis.value.AbstractValueAxis}plot.</zh>
 * <en>Used to display the two coordinate axis are
 *  (@link com.ijchart.xychart.axis.category.CategoryAxis3D),
 * (@Link com.ijchart.xychart.axis.value.AbstractValueAxis) of the plot.</en>
 *
 * <p>Copyright: Copyright (c) 2013</p>
 *
 * <p>Company: </p>
 *
 * @author zhang feng min,shanghai china.
 * <a href="mailto:ijchartsh@gmail.com">Contract me by email.</a>
 * @version 1.0
 */
public class Category3DPlot extends CategoryPlot implements Cloneable, Serializable, IEffect3D {
  /**
   * <zh>л汾ID.</zh>
   * <en>For serialization.</en>
   */
  private static final long serialVersionUID = -1771424243934790323L ;

  /**
   * <zh>ȱʡxƫֵ.</zh>
   * <en>The default x-offset for the 3D effect. </en>
   */
  public static final double DEFAULT_X_OFFSET = ParallelepipedTextChartShape.DEFAULT_X_OFFSET ;

  /**
   * <zh>ȱʡyƫֵ.</zh>
   * <en>The default y-offset for the 3D effect. </en>
   */
  public static final double DEFAULT_Y_OFFSET = ParallelepipedTextChartShape.DEFAULT_Y_OFFSET ;

  /**
   * <zh>xƫֵ.</zh>
   * <en>The x-offset for the 3D effect. </en>
   */
  private double xOffset ;

  /**
   * <zh>ȱʡyƫֵ.</zh>
   * <en>The default y-offset for the 3D effect. </en>
   */
  private double yOffset ;

  /**
   * <zh>ȱʡĹ.</zh>
   * <en>Default constructor.</en>
   */
  public Category3DPlot() {
    this(null, null, null) ;
  }

  /**
   * <zh>һµĶ.</zh>
   * <en>Creates a new plot.</en>
   *
   * @param categoryAxis CategoryAxis3D <zh>category (Ϊnull).</zh>
   * <en>The category axis (null permitted).</en>
   * @param valueAxis AbstractValueAxis <zh>value (Ϊnull).</zh>
   * <en>The value axis (null permitted).</en>
   * @param render AbstractCategoryRender <zh> (Ϊnull).</zh>
   * <en>The item render (null permitted).</en>
   *
   */
  public Category3DPlot(CategoryAxis3D categoryAxis, AbstractValueAxis valueAxis,
                        AbstractCategoryRender render) {
    super(categoryAxis, valueAxis, render) ;

    this.xOffset = DEFAULT_X_OFFSET ;
    this.yOffset = DEFAULT_Y_OFFSET ;
  }

  /**
   * <zh>ػ3DЧʱX᷽ƫֵ.</zh>
   * <en>Returns the x-offset (in Java2D units) for the 3D effect.</en>
   *
   * @return double <zh>X᷽ƫֵ.</zh>
   * <en>The offset.</en>
   */
  public double getXOffset() {
    return this.xOffset ;
  }

  /**
   * <zh>xƫֵ,м{@link com.ijchart.xychart.event.ChartShapeChangeEvent}¼.</zh>
   * <en>Sets the x-offset for the 3D effect,and sends
   * a {@link com.ijchart.xychart.event.ChartShapeChangeEvent} to all registered listeners. </en>
   *
   * @param offset double <zh>ƫֵ,0.</zh>
   * <en>The x-offset,must greater than 0. </en>
   */
  public void setXOffset(double offset) {
    if (this.xOffset != offset) {
      if (offset <= 0) {
        throw new IllegalArgumentException("Argument 'offset' must be greater than 0.") ;
      }

      this.xOffset = offset ;
      firePlotChanged() ;
    }
  }

  /**
   * <zh>ػ3DЧʱY᷽ƫֵ.</zh>
   * <en>Returns the y-offset (in Java2D units) for the 3D effect.</en>
   *
   * @return double <zh>Y᷽ƫֵ.</zh>
   * <en>The offset.</en>
   */
  public double getYOffset() {
    return this.yOffset ;
  }

  /**
   * <zh>yƫֵ,м{@link com.ijchart.xychart.event.ChartShapeChangeEvent}¼.</zh>
   * <en>Sets the y-offset for the 3D effect,and sends
   * a {@link com.ijchart.xychart.event.ChartShapeChangeEvent} to all registered listeners. </en>
   *
   * @param offset double <zh>ƫֵ,0.</zh>
   * <en>The y-offset,must greater than 0. </en>
   */
  public void setYOffset(double offset) {
    if (this.yOffset != offset) {
      if (offset <= 0) {
        throw new IllegalArgumentException("Argument 'offset' must be greater than 0.") ;
      }

      this.yOffset = offset ;
      firePlotChanged() ;
    }
  }

  /**
   * <zh>plotַ.</zh>
   * <en>Returns a short string describing the plot type.</en>
   *
   * @return <zh>plot.</zh>
   * <en>The plot type.</en>
   */
  public PlotType getPlotType() {
    return PlotType.PLOT_TYPE_CATEGORY3D ;
  }

  /**
   * <zh>е.</zh>
   * <en>A utility method for drawing the plot's axes.</en>
   *
   * @param g2 Graphics2D <zh>ͼ豸(ӡĻ) (Ϊnull).</zh>
   * <en>The graphics device (null not permitted).</en>
   * @param plotArea Rectangle2D <zh>plot (Ϊnull).</zh>
   * <en>The plot draw area (null not permitted).</en>
   * @param plotShapes PlotShapeCollection <zh>plotͼμ (Ϊnull).</zh>
   * <en>Collects information about the plot (null not permitted).</en>
   *
   * @return Rectangle2D <zh>.</zh>
   * <en>The data area.</en>
   */
  protected Rectangle2D drawAxes(Graphics2D g2, Rectangle2D plotArea,
                                 PlotShapeCollection plotShapes) {
    if (g2 == null) {
      throw new IllegalArgumentException("Null 'g2' argument.") ;
    }
    if (plotArea == null) {
      throw new IllegalArgumentException("Null 'plotArea' argument.") ;
    }
    if (plotShapes == null) {
      throw new IllegalArgumentException("Null 'plotShapes' argument.") ;
    }

    checkHas3DAxis() ;
    //Ԥ3d
    Rectangle2D newArea = new Rectangle2D.Double(plotArea.getX(), plotArea.getY() + this.yOffset,
                                                 plotArea.getWidth() - this.xOffset,
                                                 plotArea.getHeight() - this.yOffset) ;
    if (newArea.getWidth() <= 0 || newArea.getHeight() <= 0) { //ûпʾ
      return null ;
    }

    Rectangle2D result = super.drawAxes(g2, newArea, plotShapes) ;
    addBottomAxis3DShape(plotShapes, result) ;
    addLeftAxis3DShape(plotShapes, result) ;

    translateAxesShapes(plotShapes) ; //ƫ
    //ƫ
    result.setRect(result.getX() + this.xOffset, result.getY() - this.yOffset,
                   result.getWidth(), result.getHeight()) ;
    drawCloseRenderAreaLine(result, plotShapes) ; //Ʒ3D

    return result ;
  }

  /**
   * <zh>ȱʡķ3D.</zh>
   * <en>Draw the line shapes for close render area of 3D chart.</en>
   *
   * @param renderArea Rectangle2D <zh>render.</zh>
   * <en>The render draw area.</en>
   * @param plotShapes PlotShapeCollection <zh>plotͼμ (Ϊnull).</zh>
   * <en>Collects information about the plot (null not permitted).</en>
   */
  private void drawCloseRenderAreaLine(Rectangle2D renderArea, PlotShapeCollection plotShapes) {
    ChartOrientation orientation = getOrientation() ;
    //»categoryӦ
    if (orientation.equals(ChartOrientation.VERTICAL)) {
      int categoryAxisCount = getCategoryAxisCount() ;
      boolean drawTop = false ;
      AbstractAxis firstAxis = null ; //һʾ
      for (int i = 0 ; i < categoryAxisCount ; i++) {
        CategoryAxis categoryAxis = getCategoryAxis(i) ;
        if (categoryAxis != null) {
          if (firstAxis == null) { //¼һ
            firstAxis = categoryAxis ;
          }
          if (getCategoryAxisEdge(i).equals(RectangleEdge.TOP)) {
            drawTop = true ;
            break ;
          }
        }
      }

      if (!drawTop) { //Ҫ
        LineChartShape closeLine = new LineChartShape(renderArea.getX(), renderArea.getY(),
                                                      renderArea.getMaxX(), renderArea.getY()) ;
        closeLine.setDrawBorder(true) ;
        closeLine.setBorderPaint(firstAxis.getAxisLinePaint()) ;
        closeLine.setBorderStroke(firstAxis.getAxisLineStroke()) ;

        List renderShapesListLocal = plotShapes.getAllRenderShapes() ;
        int renderShapesListLocalSize = renderShapesListLocal.size() ;
        for (int i = 0 ; i < renderShapesListLocalSize ; i++) {
          RenderShapeCollection renderShapes = (RenderShapeCollection) renderShapesListLocal.get(i) ;
          if (renderShapes != null) {
            renderShapes.addChart3DCloseLinesShape(closeLine) ;
            break ;
          }
        }
      }

    }

    //------------------------------------------------------------------------
    //»valueӦ
    if (orientation.equals(ChartOrientation.VERTICAL)) {
      int valueAxisCount = getValueAxisCount() ;
      boolean drawRight = false ;
      AbstractAxis firstAxis = null ; //һʾ
      for (int i = 0 ; i < valueAxisCount ; i++) {
        AbstractValueAxis valueAxis = getValueAxis(i) ;
        if (valueAxis != null) {
          if (firstAxis == null) { //¼һ
            firstAxis = valueAxis ;
          }
          if (getValueAxisEdge(i).equals(RectangleEdge.RIGHT)) {
            drawRight = true ;
            break ;
          }
        }
      }

      if (!drawRight) { //Ҫ
        LineChartShape closeLine = new LineChartShape(renderArea.getMaxX(), renderArea.getY(),
                                                      renderArea.getMaxX(), renderArea.getMaxY()) ;
        closeLine.setDrawBorder(true) ;
        closeLine.setBorderPaint(firstAxis.getAxisLinePaint()) ;
        closeLine.setBorderStroke(firstAxis.getAxisLineStroke()) ;

        List renderShapesListLocal = plotShapes.getAllRenderShapes() ;
        int renderShapesListLocalSize = renderShapesListLocal.size() ;
        for (int i = 0 ; i < renderShapesListLocalSize ; i++) {
          RenderShapeCollection renderShapes = (RenderShapeCollection) renderShapesListLocal.get(i) ;
          if (renderShapes != null) {
            renderShapes.addChart3DCloseLinesShape(closeLine) ;
            break ;
          }
        }
      }

    } else if (orientation.equals(ChartOrientation.HORIZONTAL)) {
      int valueAxisCount = getValueAxisCount() ;
      boolean drawTop = false ;
      AbstractAxis firstAxis = null ; //һʾ
      for (int i = 0 ; i < valueAxisCount ; i++) {
        AbstractValueAxis valueAxis = getValueAxis(i) ;
        if (valueAxis != null) {
          if (firstAxis == null) { //¼һ
            firstAxis = valueAxis ;
          }
          if (getValueAxisEdge(i).equals(RectangleEdge.TOP)) {
            drawTop = true ;
            break ;
          }
        }
      }

      if (!drawTop) { //Ҫ
        LineChartShape closeLine = new LineChartShape(renderArea.getX(), renderArea.getY(),
                                                      renderArea.getMaxX(), renderArea.getY()) ;
        closeLine.setDrawBorder(true) ;
        closeLine.setBorderPaint(firstAxis.getAxisLinePaint()) ;
        closeLine.setBorderStroke(firstAxis.getAxisLineStroke()) ;

        List renderShapesListLocal = plotShapes.getAllRenderShapes() ;
        int renderShapesListLocalSize = renderShapesListLocal.size() ;
        for (int i = 0 ; i < renderShapesListLocalSize ; i++) {
          RenderShapeCollection renderShapes = (RenderShapeCollection) renderShapesListLocal.get(i) ;
          if (renderShapes != null) {
            renderShapes.addChart3DCloseLinesShape(closeLine) ;
            break ;
          }
        }
      }

    }

  }

  /**
   * <zh>ǷһXἰY.</zh>
   * <en>Check whether or not has a category axis and a value axis at least.</en>
   */
  private void checkHas3DAxis() {
    //һ3DXἰY
    int categoryAxisCount = getCategoryAxisCount() ;
    for (int i = 0 ; i < categoryAxisCount ; i++) {
      IAxis axis = getCategoryAxis(i) ;
      if (axis != null) {
        if (axis instanceof IEffect3D) {
          break ;
        }
        if (i == categoryAxisCount - 1 && !(axis instanceof IEffect3D)) {
          throw new RuntimeException("No 3D category axis found.") ;
        }
      }
    }

    int valueAxisCount = getValueAxisCount() ;
    for (int i = 0 ; i < valueAxisCount ; i++) {
      IAxis axis = getCategoryAxis(i) ;
      if (axis != null) {
        if (axis instanceof IEffect3D) {
          break ;
        }
        if (i == valueAxisCount - 1 && !(axis instanceof IEffect3D)) {
          throw new RuntimeException("No 3D value axis found.") ;
        }
      }
    }
  }

  /**
   * <zh>ûеײ3D͵,һ3Dͼ.</zh>
   * <en>If no 3D axis at bottom edge,add a 3d axis line shape.</en>
   *
   * @param plotShapes PlotShapeCollection <zh>plotͼμ (Ϊnull).</zh>
   * <en>Collects information about the plot (null not permitted).</en>
   * @param renderArea Rectangle2D <zh>render.</zh>
   * <en>The render draw area.</en>
   */
  private void addBottomAxis3DShape(PlotShapeCollection plotShapes, Rectangle2D renderArea) {
    List xAxisShapes = plotShapes.getXAxisShapeList() ;
    for (int i = 0 ; i < xAxisShapes.size() ; i++) {
      AxisShapeCollection axisShapes = (AxisShapeCollection) xAxisShapes.get(i) ;
      if (axisShapes.getEdge().equals(RectangleEdge.BOTTOM)) {
        if (axisShapes.getAxisLine3DShapes().size() > 0) {
          return ;
        }
      }
    }

    List yAxisShapes = plotShapes.getYAxisShapeList() ;
    for (int i = 0 ; i < yAxisShapes.size() ; i++) {
      AxisShapeCollection axisShapes = (AxisShapeCollection) yAxisShapes.get(i) ;
      if (axisShapes.getEdge().equals(RectangleEdge.BOTTOM)) {
        if (axisShapes.getAxisLine3DShapes().size() > 0) {
          return ;
        }
      }
    }

    int categoryAxisCount = getCategoryAxisCount() ;
    final float correct = 0.5F ; //ֵ
    for (int i = 0 ; i < categoryAxisCount ; i++) {
      CategoryAxis axis = getCategoryAxis(i) ;
      if (axis != null) {
        float strokeWidth = (float) (StrokeUtil.getStrokeWidth(axis.getAxisLineStroke()) * 0.5) ;

        GeneralPath path = new GeneralPath() ;
        path.moveTo((float) (renderArea.getX() - correct),
                    (float) (renderArea.getMaxY() + correct)) ;
        path.lineTo((float) (renderArea.getX() + this.xOffset - correct),
                    (float) (renderArea.getMaxY() - this.yOffset + strokeWidth)) ;
        path.lineTo((float) (renderArea.getMaxX() + this.xOffset),
                    (float) (renderArea.getMaxY() - this.yOffset + strokeWidth)) ;
        path.lineTo((float) renderArea.getMaxX(), (float) (renderArea.getMaxY() + correct)) ;
        path.closePath() ;

        IChartShape axisLine = new GeneralPathChartShape(path) ;
        set3DAxisLineProperties(axis, axisLine) ;

        List axisShapes = plotShapes.getXAxisShapeList() ;
        if (axisShapes.size() > 0) {
          ((AxisShapeCollection) axisShapes.get(0)).addAxisLine3DShape(axisLine) ;
        }

        break ;
      }
    }

  }

  /**
   * <zh>ûߵ3D͵,һ3Dͼ.</zh>
   * <en>If no 3D axis at left edge,add a 3d Y-axis line shape.</en>
   *
   * @param plotShapes PlotShapeCollection <zh>plotͼμ (Ϊnull).</zh>
   * <en>Collects information about the plot (null not permitted).</en>
   * @param renderArea Rectangle2D <zh>render.</zh>
   * <en>The render draw area.</en>
   */
  private void addLeftAxis3DShape(PlotShapeCollection plotShapes, Rectangle2D renderArea) {
    List xAxisShapes = plotShapes.getXAxisShapeList() ;
    for (int i = 0 ; i < xAxisShapes.size() ; i++) {
      AxisShapeCollection axisShapes = (AxisShapeCollection) xAxisShapes.get(i) ;
      if (axisShapes.getEdge().equals(RectangleEdge.LEFT)) {
        if (axisShapes.getAxisLine3DShapes().size() > 0) {
          return ;
        }
      }
    }

    List yAxisShapes = plotShapes.getYAxisShapeList() ;
    for (int i = 0 ; i < yAxisShapes.size() ; i++) {
      AxisShapeCollection axisShapes = (AxisShapeCollection) yAxisShapes.get(i) ;
      if (axisShapes.getEdge().equals(RectangleEdge.LEFT)) {
        if (axisShapes.getAxisLine3DShapes().size() > 0) {
          return ;
        }
      }
    }

    int categoryAxisCount = getCategoryAxisCount() ;
    final float correct = 0.5F ; //ֵ
    for (int i = 0 ; i < categoryAxisCount ; i++) {
      CategoryAxis axis = getCategoryAxis(i) ;
      if (axis != null) {
        //һײͼ
        float strokeWidth = (float) (StrokeUtil.getStrokeWidth(axis.getAxisLineStroke()) * 0.5) ;

        GeneralPath path = new GeneralPath() ;
        path.moveTo((float) (renderArea.getX() + this.xOffset - strokeWidth),
                    (float) (renderArea.getY() - this.yOffset - 2 * correct)) ;
        path.lineTo((float) (renderArea.getX() + this.xOffset - strokeWidth),
                    (float) (renderArea.getMaxY() - this.yOffset + correct)) ;
        path.lineTo((float) (renderArea.getX() - correct), (float) (renderArea.getMaxY() + correct)) ;
        path.lineTo((float) (renderArea.getX() - correct), (float) (renderArea.getY() - 2 * correct)) ;
        path.closePath() ;

        IChartShape axisLine = new GeneralPathChartShape(path) ;
        set3DAxisLineProperties(axis, axisLine) ;

        List axisShapes = plotShapes.getYAxisShapeList() ;
        if (axisShapes.size() > 0) {
          ((AxisShapeCollection) axisShapes.get(0)).addAxisLine3DShape(axisLine) ;
        }

        break ;
      }
    }

  }

  /**
   * <zh>ͼε.</zh>
   * <en>Sets the properties of the 3d axis line shape.</en>
   *
   * @param axis AbstractAxis
   * @param shape IChartShape
   */
  private static void set3DAxisLineProperties(AbstractAxis axis, IChartShape shape) {
    Paint linePaint = axis.getAxisLinePaint() ;
    shape.setBorderPaint(linePaint) ;
    shape.setBorderStroke(axis.getAxisLineStroke()) ;
    shape.setDrawBorder(true) ;
    shape.setFillShape(true) ;
    if (linePaint instanceof Color) {
      shape.setFillPaint(((Color) linePaint).brighter()) ;
    }
  }

  /**
   * <zh>ƫϲҲͼ.</zh>
   * <en>Translate position of axis shapes,the axis position is at top edge or right edge.</en>
   *
   * @param plotShapes PlotShapeCollection
   */
  private void translateAxesShapes(PlotShapeCollection plotShapes) {
    List newList = new ArrayList() ;

    List xAxes = plotShapes.getXAxisShapeList() ;
    newList.addAll(xAxes) ;
    List yAxes = plotShapes.getYAxisShapeList() ;
    newList.addAll(yAxes) ;

    for (int i = 0 ; i < newList.size() ; i++) {
      AxisShapeCollection axisShapes = (AxisShapeCollection) newList.get(i) ;
      if (axisShapes != null) {
        RectangleEdge edge = axisShapes.getEdge() ;
        if (edge != null && edge.equals(RectangleEdge.TOP) || edge.equals(RectangleEdge.RIGHT)) {
          axisShapes.translate(this.xOffset, -this.yOffset) ;
        }
      }
    }
  }

  /**
   * <zh>Ƿ.</zh>
   * <en>Indicates whether some other object is "equal to" this one.</en>
   *
   * @param obj <zh>ҪȽϵĶ</zh>
   * <en>the reference object with which to compare.</en>
   * @return <zh>ȷtrue,򷵻false.</zh>
   * <en>true if this object is the same as the obj argument;
   * false otherwise.</en>
   */
  public boolean equals(Object obj) {
    if (this == obj) {
      return false ;
    }
    if (!(obj instanceof Category3DPlot)) {
      return false ;
    }
    if (!super.equals(obj)) {
      return false ;
    }

    Category3DPlot that = (Category3DPlot) obj ;
    if (this.xOffset != that.xOffset) {
      return false ;
    }
    if (this.yOffset != that.yOffset) {
      return false ;
    }

    return true ;
  }

  /**
   * <zh>hashcodeֵ.</zh>
   * <en>Returns a hash code value for the object.</en>
   *
   * @return int <zh>hashcodeֵ.</zh>
   * <en>A hash code value for this object.</en>
   */
  public int hashCode() {
    int hashCode = super.hashCode() ;
    hashCode = 37 * hashCode + (int)this.xOffset ;
    hashCode = 37 * hashCode + (int)this.yOffset ;

    return hashCode ;
  }

  /**
   * <zh>ؿ¡Ķ.</zh>
   * <en>Creates and returns a copy of this object.</en>
   * @return object   a clone of this instance.
   * @exception  CloneNotSupportedException  if the object's class does not
   *               support the Cloneable interface. Subclasses
   *               that override the clone method can also
   *               throw this exception to indicate that an instance cannot
   *               be cloned.
   */
  public Object clone() throws CloneNotSupportedException {
    return super.clone() ;
  }

  /**
   * <zh>toString.</zh>
   * <en>Returns a string representation of the object. </en>
   *
   * @return  a string representation of the object.
   */
  public String toString() {
    return "[xOffset=" + this.xOffset + ",yOffset" + this.yOffset + "]" ;
  }
}
