package com.ijchart.xychart.range ;

import java.io.Serializable ;
import java.text.DateFormat ;
import java.util.Date ;

import com.ijchart.xychart.util.EqualsUtil ;
import com.ijchart.xychart.util.HashcodeUtil ;

/**
 * <p>Title: IJChart</p>
 *
 * <p>Description: a chart library for the Java(tm) platform.</p>
 * <zh>͵ֵݷΧ.ǲɱ.</zh>
 * <en>A range specified in terms of two java.util.Date objects.
 * Instances of this class are immutable.</en>
 *
 * <p>Copyright: Copyright (c) 2013</p>
 *
 * <p>Company: </p>
 *
 * @author zhang feng min,shanghai china.
 * <a href="mailto:ijchartsh@gmail.com">Contract me by email.</a>
 * @version 1.0
 */
public class DateRange extends NumberRange implements Serializable, Cloneable {
  /**
   * <zh>л汾ID.</zh>
   * <en>For serialization.</en>
   */
  private static final long serialVersionUID = 3022002740379937636L ;

  /**
   * <zh>Сֵ.</zh>
   * <en>The lower bound for the range.</en>
   */
  private Date lowerDate ;

  /**
   * <zh>ڴֵ.</zh>
   * <en>The upper bound for the range.</en>
   */
  private Date upperDate ;

  /**
   * <zh>һСһ,lower<=upper.</zh>
   * <en>Constructs a new range.</en>
   *
   * @param lower <zh>Сֵ (Ϊnull).</zh>
   * <en>The lower bound (null not permitted).</en>
   * @param upper <zh>ֵ (Ϊnull).</zh>
   * <en>The upper bound (null not permitted).</en>
   */
  public DateRange(Date lower, Date upper) {
    super(lower.getTime(), upper.getTime()) ;

    this.lowerDate = lower ;
    this.upperDate = upper ;
  }

  /**
   * <zh>longֹ,ʱΪ1970-1-1 00:00:00 GMTʼĺ.</zh>
   * <en>Constructs a new range using two values that will be interpreted as
   * "milliseconds since midnight GMT, 1-Jan-1970".</en>
   *
   * @param lower <zh>Сֵ.</zh>
   * <en>The lower (oldest) date.</en>
   * @param upper <zh>ֵ.</zh>
   * <en>The upper (most recent) date.</en>
   */
  public DateRange(long lower, long upper) {
    super(lower, upper) ;

    this.lowerDate = new Date(lower) ;
    this.upperDate = new Date(upper) ;
  }

  /**
   * <zh>һ{@link NumberRange}ݷΧ.</zh>
   * <en>Constructs a new range that is based on another {@link NumberRange}.  The
   * other range does not have to be a {@link DateRange}.  If it is not, the
   * upper and lower bounds are evaluated as milliseconds since midnight
   * GMT, 1-Jan-1970.</en>
   *
   * @param other <zh>һݷΧ.</zh>
   * <en>The other range (null not permitted).</en>
   */
  public DateRange(NumberRange other) {
    this((long) other.getLowerBound(), (long) other.getUpperBound()) ;
  }

  /**
   * <zh>Сֵ.</zh>
   * <en>Returns the lower (earlier) date for the range.</en>
   *
   * @return <zh>Сֵ.</zh>
   * <en>The lower date for the range.</en>
   */
  public Date getLowerDate() {
    return this.lowerDate ;
  }

  /**
   * <zh>شֵ.</zh>
   * <en>Returns the upper (later) date for the range.</en>
   *
   * @return <zh>ֵ.</zh>
   * <en>The upper date for the range.</en>
   */
  public Date getUpperDate() {
    return this.upperDate ;
  }

  /**
   * <zh>Ƿ.</zh>
   * <en>Tests the marker for equality with an arbitrary double.</en>
   *
   * @param obj  the double (null permitted).
   *
   * @return A boolean.
   */
  public boolean equals(Object obj) {
    if (obj == null) {
      return false ;
    }
    if (!(obj instanceof DateRange)) {
      return false ;
    }
    if (!super.equals(obj)) {
      return false ;
    }

    DateRange that = (DateRange) obj ;
    if (!EqualsUtil.equals(this.lowerDate, that.lowerDate)) {
      return false ;
    }
    if (!EqualsUtil.equals(this.upperDate, that.upperDate)) {
      return false ;
    }

    return true ;
  }

  /**
   * <zh>¡.</zh>
   * <en>Creates and returns a copy of this object.</en>
   *
   * @throws java.lang.CloneNotSupportedException
   * @return Object
   */
  protected Object clone() throws CloneNotSupportedException {
    return super.clone() ;
  }

  /**
   * <zh>hashcode.</zh>
   * <en>Returns a hash code.</en>
   *
   * @return A hash code.
   */
  public int hashCode() {
    int hashcode = super.hashCode() ;
    hashcode = 37 * hashcode + HashcodeUtil.hashCode(this.lowerDate) ;
    hashcode = 37 * hashcode + HashcodeUtil.hashCode(this.upperDate) ;

    return hashcode ;
  }

  /**
   * <zh>toString.</zh>
   * <en>Returns a string representing the date range (useful for debugging).</en>
   *
   * @return A string
   */
  public String toString() {
    DateFormat df = DateFormat.getDateTimeInstance() ;
    return "[lower=" + df.format(this.lowerDate) + ",upper=" + df.format(this.upperDate) + "]" ;
  }

}
