package com.ijchart.xychart.render.category ;

import java.awt.geom.GeneralPath ;
import java.awt.geom.Rectangle2D ;
import java.io.Serializable ;

import com.ijchart.xychart.axis.category.CategoryAxis ;
import com.ijchart.xychart.axis.value.AbstractValueAxis ;
import com.ijchart.xychart.chartinfo.RenderShapeCollection ;
import com.ijchart.xychart.constant.AreaRenderEndType ;
import com.ijchart.xychart.constant.ChartOrientation ;
import com.ijchart.xychart.constant.HorizontalAlignment ;
import com.ijchart.xychart.constant.LegendType ;
import com.ijchart.xychart.constant.TextPosition ;
import com.ijchart.xychart.constant.VerticalAlignment ;
import com.ijchart.xychart.data.IChartData ;
import com.ijchart.xychart.data.NumberChartData ;
import com.ijchart.xychart.dataset.IMatrixDataset ;
import com.ijchart.xychart.shape.GeneralPathChartShape ;
import com.ijchart.xychart.shape.IChartShape ;
import com.ijchart.xychart.ui.RectangleEdge ;
import com.ijchart.xychart.util.EqualsUtil ;
import com.ijchart.xychart.util.HashcodeUtil ;

/**
 * <p>Title: IJChart</p>
 *
 * <p>Description: a chart library for the Java(tm) platform.</p>
 * <zh>(area)͵״ͼ.</zh>
 * <en>Draw area bar shapes.</en>
 * <p>Copyright: Copyright (c) 2013</p>
 *
 * <p>Company: </p>
 *
 * @author zhang feng min,shanghai china.
 * <a href="mailto:ijchartsh@gmail.com">Contract me by email.</a>
 * @version 1.0
 */
public class AreaRender extends AbstractCategoryRender implements Cloneable, Serializable {
  /**
   * <zh>л汾ID.</zh>
   * <en>For serialization.</en>
   */
  private static final long serialVersionUID = 5967956672101527995L ;

  /**
   * <zh>ȱʡƵһһĻƷʽ.</zh>
   * <en>A default flag that controls how the firs or last of the area is drawn.</en>
   */
  public static final AreaRenderEndType DEFAULT_END_TYPE = AreaRenderEndType.TAPER ;

  /**
   * <zh>ƵһһĻƷʽ.</zh>
   * <en>A flag that controls how the firs or last of the area is drawn.</en>
   */
  private AreaRenderEndType endType ;

  /**
   * <zh>.</zh>
   * <en>constructor.</en>
   *
   * @param dataset IMatrixDataset <zh>ݼ (Ϊnull).</zh>
   * <en>The dataset (null not permitted).</en>
   */
  public AreaRender(IMatrixDataset dataset) {
    this(dataset, DEFAULT_END_TYPE) ;
  }

  /**
   * <zh>.</zh>
   * <en>constructor.</en>
   *
   * @param dataset IMatrixDataset <zh>ݼ (Ϊnull).</zh>
   * <en>The dataset (null not permitted).</en>
   * @param endType AreaRenderEndType <zh>ƵһһĻƷʽ (Ϊnull).</zh>
   * <en>A flag that controls how the firs or last of the area is drawn (null not permitted).</en>
   */
  public AreaRender(IMatrixDataset dataset, AreaRenderEndType endType) {
    super(dataset) ;
    if (endType == null) {
      throw new IllegalArgumentException("Null 'endType' argument.") ;
    }

    this.endType = endType ;
  }

  /**
   * <zh>ؿƵһһĻƷʽ.</zh>
   * <en>Returns the flag that controls how the firs or last of the area is drawn.</en>
   *
   * @return AreaRenderEndType <zh>{@link AreaRenderEndType} (Ϊnull).</zh>
   * <en>The {@link AreaRenderEndType} object (null not permitted).</en>
   */
  public AreaRenderEndType getEndType() {
    return this.endType ;
  }

  /**
   * <zh>ÿƵһһĻƷʽ,м
   * {@link com.ijchart.xychart.event.RenderChangeEvent}¼.</zh>
   * <en>Returns the flag that controls how the firs or last of the area is drawn,and sends a
   * {@link com.ijchart.xychart.event.RenderChangeEvent} to all registered listeners.</en>
   *
   * @param endType AreaRenderEndType
   */
  public void setEndType(AreaRenderEndType endType) {
    if (endType == null) {
      throw new IllegalArgumentException("Null 'endType' argument.") ;
    }

    if (!this.endType.equals(endType)) {
      this.endType = endType ;
      fireRenderChanged() ;
    }
  }

  /**
   * <zh>еͼ.</zh>
   * <en>Draws all data items.</en>
   *
   * @param categoryAxis CategoryAxis <zh>category (Ϊnull).</zh>
   * <en>The category axis (null not permitted).</en>
   * @param valueAxis AbstractValueAxis <zh>value (Ϊnull).</zh>
   * <en>The value axis (null not permitted).</en>
   * @param valueAxisEdge RectangleEdge <zh>valueλ (Ϊnull).</zh>
   * <en>The value axis edge (null not permitted).</en>
   * @param dataArea Rectangle2D <zh> (Ϊnull).</zh>
   * <en>The chart data area (null not permitted).</en>
   * @param orientation ChartOrientation <zh>ͼķ (Ϊnull).</zh>
   * <en>The chart orientation (null not permitted).</en>
   * @param renderShapes RenderShapeCollection <zh>renderͼμ (Ϊnull).</zh>
   * <en>Collects information about the render (null not permitted).</en>
   * @param alpha float <zh>alphaֵ,ڱ[0.0,1.0].</zh>
   * <en>The alpha transparency (in the range 0.0f to 1.0f, where 0.0f is fully transparent,
   * and 1.0f is fully opaque).</en>
   */
  public void drawAllItems(CategoryAxis categoryAxis, AbstractValueAxis valueAxis,
                           RectangleEdge valueAxisEdge, Rectangle2D dataArea,
                           ChartOrientation orientation, RenderShapeCollection renderShapes,
                           float alpha) {
    if (categoryAxis == null) {
      throw new IllegalArgumentException("Null 'categoryAxis' argument.") ;
    }
    if (valueAxis == null) {
      throw new IllegalArgumentException("Null 'valueAxis' argument.") ;
    }
    if (valueAxisEdge == null) {
      throw new IllegalArgumentException("Null 'valueAxisEdge' argument.") ;
    }
    if (dataArea == null) {
      throw new IllegalArgumentException("Null 'dataArea' argument.") ;
    }
    if (orientation == null) {
      throw new IllegalArgumentException("Null 'orientation' argument.") ;
    }
    if (renderShapes == null) {
      throw new IllegalArgumentException("Null 'renderShapes' argument.") ;
    }
    if (alpha < 0.0f || alpha > 1.0f) {
      throw new IllegalArgumentException("The 'alpha' value must be in the range 0.0f to 1.0f.") ;
    }

    IMatrixDataset dataset = getDataset() ;
    if (dataset.isEmpty()) {
      return ;
    }
    int columns = dataset.getColumnCount() ;
    int rows = dataset.getRowCount() ;

    double basePosition = valueAxis.valueToJava2D(dataArea, valueAxisEdge, 0, false) ;
    if (basePosition == AbstractValueAxis.INVALID_COORDINATE_VALUE) { //ֵ᷶Χ
      return ;
    }

    this.fillPaints = getFillPaints(rows) ;
    this.borderPaints = getBorderPaints(rows) ;
    this.borderStrokes = getBorderStrokes(rows) ;

    //(area)ͼ
    if (orientation.equals(ChartOrientation.VERTICAL)) { //ֱ
      double gapSize = 0 ; //С
      if (columns > 1) {
        gapSize = dataArea.getWidth() * categoryAxis.getCategoryMargin() / (columns - 1) ;
      }
      double axisMargin = categoryAxis.getLowerMargin() + categoryAxis.getUpperMargin()
                          + ((columns == 1) ? 0 : categoryAxis.getCategoryMargin()) ; //еĿհ
      double categoryWidth = dataArea.getWidth() * (1 - axisMargin) / columns ; //ÿĿ

      for (int i = 0 ; i < rows ; i++) {
        double startX = dataArea.getX() + dataArea.getWidth() * categoryAxis.getLowerMargin() ;

        double lastY = Double.NaN ;
        int currRowCols = dataset.getColumnCount(i) ;
        for (int j = 0 ; j < currRowCols ; j++) {
          IChartData data = dataset.getData(i, j) ;
          if (data != null && data instanceof NumberChartData) {
            double value = ((NumberChartData) data).getValue() ;
            double y = valueAxis.valueToJava2D(dataArea, valueAxisEdge, value, false) ;
            //ֵ᷶Χֵ
            if (y == AbstractValueAxis.INVALID_COORDINATE_VALUE) {
              lastY = Double.NaN ;
              renderShapes.addDataItemShape(null) ;
              continue ;
            }

            HorizontalAlignment hAlign = null ;
            double x1 = 0, x2 = 0, x3 = 0 ;
            if (this.endType.equals(AreaRenderEndType.TRUNCATE)) {
              if (j == 0) {
                x1 = startX + 0.5 * categoryWidth ;
                x2 = x1 ;
                x3 = startX + categoryWidth ;
                hAlign = HorizontalAlignment.LEFT ;
              } else if (j == columns - 1) {
                x1 = startX + (categoryWidth + gapSize) * j ;
                x2 = x1 + 0.5 * categoryWidth ;
                x3 = x2 ;
                hAlign = HorizontalAlignment.RIGHT ;
              } else {
                x1 = startX + (categoryWidth + gapSize) * j ;
                x2 = x1 + 0.5 * categoryWidth ;
                x3 = x1 + categoryWidth ;
                hAlign = HorizontalAlignment.CENTER ;
              }
            } else if (this.endType.equals(AreaRenderEndType.TAPER)) {
              x1 = startX + (categoryWidth + gapSize) * j ;
              x2 = x1 + 0.5 * categoryWidth ;
              x3 = x1 + categoryWidth ;
              hAlign = HorizontalAlignment.CENTER ;
            }

            double nextValue = basePosition ;
            if (j < columns - 1) {
              IChartData nextData = dataset.getData(i, j + 1) ;
              if (nextData != null && nextData instanceof NumberChartData) {
                nextValue = ((NumberChartData) nextData).getValue() ;
              }
            }

            double y1 = Double.isNaN(lastY) ? basePosition : lastY ;
            double y2 = y ;
            double y3 = 0 ;
            if (this.endType.equals(AreaRenderEndType.TAPER) && j == columns - 1) {
              y3 = basePosition ;
            } else {
              y3 = valueAxis.valueToJava2D(dataArea, valueAxisEdge, 0.5 * (value + nextValue), false) ;
            }

            TextPosition position = null ;
            if (y >= basePosition) {
              position = TextPosition.OUTSIDE_BOTTOM ;
            } else {
              position = TextPosition.OUTSIDE_TOP ;
            }

            GeneralPath path = new GeneralPath() ;
            path.moveTo((float) x1, (float) y1) ;
            path.lineTo((float) x2, (float) y2) ;
            path.lineTo((float) x3, (float) y3) ;
            path.lineTo((float) x3, (float) basePosition) ;
            path.lineTo((float) x1, (float) basePosition) ;
            path.closePath() ;

            setItemLabelHorizontalAlignment(i, j, hAlign) ;
            setItemLabelPosition(i, j, position) ;

            Rectangle2D drawArea = null ;
            if (y >= basePosition) {
              drawArea = new Rectangle2D.Double(x1, basePosition, x3 - x1, y - basePosition) ;
            } else {
              drawArea = new Rectangle2D.Double(x1, y, x3 - x1, basePosition - y) ;
            }

            Object[] params = {path, drawArea} ;
            IChartShape shape = createItemShape(orientation, valueAxis, i, j, null, fillPaints[i],
                                                borderPaints[i], borderStrokes[i], alpha, params) ;
            renderShapes.addDataItemShape(shape) ;
            lastY = y3 ;
          } else {
            lastY = Double.NaN ;
            renderShapes.addDataItemShape(null) ;
          }
        }
      }

    } else if (orientation.equals(ChartOrientation.HORIZONTAL)) { //ˮƽ
      double gapSize = 0 ; //С
      if (columns > 1) {
        gapSize = dataArea.getHeight() * categoryAxis.getCategoryMargin() / (columns - 1) ;
      }
      double axisMargin = categoryAxis.getLowerMargin() + categoryAxis.getUpperMargin()
                          + ((columns == 1) ? 0 : categoryAxis.getCategoryMargin()) ; //еĿհ
      double categoryWidth = dataArea.getHeight() * (1 - axisMargin) / columns ; //ÿĿ

      for (int i = 0 ; i < rows ; i++) {
        double startY = dataArea.getY() + dataArea.getHeight() * categoryAxis.getUpperMargin() ;

        double lastX = Double.NaN ;
        for (int j = 0 ; j < columns ; j++) {
          IChartData data = dataset.getData(i, j) ;
          if (data != null && data instanceof NumberChartData) {
            double value = ((NumberChartData) data).getValue() ;
            double x = valueAxis.valueToJava2D(dataArea, valueAxisEdge, value, false) ;
            //ֵ᷶Χֵ
            if (x == AbstractValueAxis.INVALID_COORDINATE_VALUE) {
              lastX = Double.NaN ;
              renderShapes.addDataItemShape(null) ;
              continue ;
            }

            HorizontalAlignment hAlign = null ;
            VerticalAlignment vAlign = null ;
            double y1 = 0, y2 = 0, y3 = 0 ;
            if (this.endType.equals(AreaRenderEndType.TRUNCATE)) {
              if (j == 0) {
                y1 = startY + 0.5 * categoryWidth ;
                y2 = y1 ;
                y3 = startY + categoryWidth ;
                hAlign = HorizontalAlignment.LEFT ;
                vAlign = VerticalAlignment.TOP ;
              } else if (j == columns - 1) {
                y1 = startY + (categoryWidth + gapSize) * j ;
                y2 = y1 + 0.5 * categoryWidth ;
                y3 = y2 ;
                hAlign = HorizontalAlignment.RIGHT ;
                vAlign = VerticalAlignment.BOTTOM ;
              } else {
                y1 = startY + (categoryWidth + gapSize) * j ;
                y2 = y1 + 0.5 * categoryWidth ;
                y3 = y1 + categoryWidth ;
                hAlign = HorizontalAlignment.CENTER ;
                vAlign = VerticalAlignment.CENTER ;
              }
            } else if (this.endType.equals(AreaRenderEndType.TAPER)) {
              y1 = startY + (categoryWidth + gapSize) * j ;
              y2 = y1 + 0.5 * categoryWidth ;
              y3 = y1 + categoryWidth ;
              hAlign = HorizontalAlignment.CENTER ;
              vAlign = VerticalAlignment.CENTER ;
            }

            double nextValue = basePosition ;
            if (j < columns - 1) {
              IChartData nextData = dataset.getData(i, j + 1) ;
              if (nextData != null && nextData instanceof NumberChartData) {
                nextValue = ((NumberChartData) nextData).getValue() ;
              }
            }

            double x1 = Double.isNaN(lastX) ? basePosition : lastX ;
            double x2 = x ;
            double x3 = 0 ;
            if (this.endType.equals(AreaRenderEndType.TAPER) && j == columns - 1) {
              x3 = basePosition ;
            } else {
              x3 = valueAxis.valueToJava2D(dataArea, valueAxisEdge, 0.5 * (value + nextValue), false) ;
            }

            TextPosition position = null ;
            if (x >= basePosition) {
              position = TextPosition.OUTSIDE_RIGHT ;
            } else {
              position = TextPosition.OUTSIDE_LEFT ;
            }

            GeneralPath path = new GeneralPath() ;
            path.moveTo((float) basePosition, (float) y1) ;
            path.lineTo((float) x1, (float) y1) ;
            path.lineTo((float) x2, (float) y2) ;
            path.lineTo((float) x3, (float) y3) ;
            path.lineTo((float) basePosition, (float) y3) ;
            path.closePath() ;

            setItemLabelHorizontalAlignment(i, j, hAlign) ;
            setItemLabelVerticalAlignment(i, j, vAlign) ;
            setItemLabelPosition(i, j, position) ;

            Rectangle2D drawArea = null ;
            if (x >= basePosition) {
              drawArea = new Rectangle2D.Double(basePosition, y1, x - basePosition, y3 - y1) ;
            } else {
              drawArea = new Rectangle2D.Double(x, y1, basePosition - x, y3 - y1) ;
            }

            Object[] params = {path, drawArea} ;
            IChartShape shape = createItemShape(orientation, valueAxis, i, j, null, fillPaints[i],
                                                borderPaints[i], borderStrokes[i], alpha, params) ;
            renderShapes.addDataItemShape(shape) ;
            lastX = x3 ;
          } else {
            lastX = Double.NaN ;
            renderShapes.addDataItemShape(null) ;
          }
        }
      }

    }

  }

  /**
   * <zh>ͼ.</zh>
   * <en>Return the legend type.</en>
   *
   * @return LegendType <zh>ͼ.</zh>
   * <en>The legend type.</en>
   */
  public LegendType getLegendType() {
    return LegendType.ROW ;
  }

  /**
   * <zh>ֻͼ,ͼε.</zh>
   * <en>Create a shape with no attribute specified.</en>
   *
   * @param rect Rectangle2D <zh>ͼľ (Ϊnull).</zh>
   * <en>The bar rectangle (null not permitted).</en>
   * @param text String <zh>Ҫʾ (Ϊnull).</zh>
   * <en>The text (null permitted).</en>
   * @param appendParam Object <zh>ӵĲ,ڴݶĶ (Ϊnull).</zh>
   * <en>The append parameter,used for special propose (null permitted).</en>
   *
   * @return IChartShape <zh>ͼ.</zh>
   * <en>The shape.</en>
   */
  protected IChartShape createItemShapeOnly(Rectangle2D rect, String text,
                                            Object appendParam) {
    if (appendParam == null) {
      throw new IllegalArgumentException("Null 'appendParam' argument.") ;
    }

    Object[] params = (Object[]) appendParam ;
    GeneralPath path = (GeneralPath) params[0] ;
    Rectangle2D drawArea = (Rectangle2D) params[1] ;

    GeneralPathChartShape shape = new GeneralPathChartShape(path, drawArea, null) ;
    shape.setText(text) ;

    return shape ;
  }

  /**
   * <zh>Ƿ.</zh>
   * <en>Tests this instance for equality with an arbitrary object.</en>
   *
   * @param obj  the object (null permitted).
   *
   * @return A boolean.
   */
  public boolean equals(Object obj) {
    if (obj == this) {
      return true ;
    }
    if (!(obj instanceof AreaRender)) {
      return false ;
    }
    if (!super.equals(obj)) {
      return false ;
    }

    AreaRender that = (AreaRender) obj ;
    if (!EqualsUtil.equals(this.endType, that.endType)) {
      return false ;
    }

    return true ;
  }

  /**
   * <zh>hashcodeֵ.</zh>
   * <en>Returns a hash code value for the object.</en>
   *
   * @return int <zh>hashcodeֵ.</zh>
   * <en>A hash code value for this object.</en>
   */
  public int hashCode() {
    int result = super.hashCode() + HashcodeUtil.hashCode(this.endType) ;

    return result ;
  }

  /**
   * <zh>ؿ¡Ķ.</zh>
   * <en>Creates and returns a copy of this object.</en>
   * @return object   a clone of this instance.
   * @exception  CloneNotSupportedException  if the object's class does not
   *               support the Cloneable interface. Subclasses
   *               that override the clone method can also
   *               throw this exception to indicate that an instance cannot
   *               be cloned.
   */
  public Object clone() throws CloneNotSupportedException {
    return super.clone() ;
  }

  /**
   * <zh>toString.</zh>
   * <en>Returns a string representation of the object.</en>
   *
   * @return  a string representation of the object.
   */
  public String toString() {
    return "[endType=" + this.endType + "]" ;
  }

}
