package com.ijchart.xychart.render.category ;

import java.awt.geom.Rectangle2D ;
import java.io.Serializable ;

import com.ijchart.xychart.constant.ChartOrientation ;
import com.ijchart.xychart.constant.Direction ;
import com.ijchart.xychart.dataset.IMatrixDataset ;
import com.ijchart.xychart.event.RenderChangeEvent ;
import com.ijchart.xychart.shape.CylinderTextChartShape ;
import com.ijchart.xychart.shape.IChartShape ;
import com.ijchart.xychart.ui.IEffect3D ;
import com.ijchart.xychart.util.EqualsUtil ;
import com.ijchart.xychart.util.HashcodeUtil ;

/**
 * <p>Title: IJChart</p>
 *
 * <p>Description: a chart library for the Java(tm) platform.</p>
 * <zh>Բͼ.</zh>
 * <en>Draw cylinder shapes.</en>
 *
 * <p>Copyright: Copyright (c) 2013</p>
 *
 * <p>Company: </p>
 *
 * @author zhang feng min,shanghai china.
 * <a href="mailto:ijchartsh@gmail.com">Contract me by email.</a>
 * @version 1.0
 */
public class CylinderRender extends Bar3DRender implements Cloneable, Serializable {
  /**
   * <zh>л汾ID.</zh>
   * <en>For serialization.</en>
   */
  private static final long serialVersionUID = 5959510001868929426L ;

  /**
   * <zh>ͼ.</zh>
   * <en>The chart orientation.</en>
   */
  private ChartOrientation chartOrientation ;

  /**
   * <zh>.</zh>
   * <en>constructor.</en>
   *
   * @param effect3D IEffect3D <zh>3DʾЧ (Ϊnull).</zh>
   * <en>The IEffect3D object (null not permitted).</en>
   * @param dataset IMatrixDataset <zh>ݼ (Ϊnull).</zh>
   * <en>The dataset (null not permitted).</en>
   * @param chartOrientation <zh>ͼ (Ϊnull).</zh>
   * <en>The chart orientation (null not permitted).</en>
   */
  public CylinderRender(IEffect3D effect3D, IMatrixDataset dataset,
                        ChartOrientation chartOrientation) {
    super(effect3D, dataset) ;

    if (chartOrientation == null) {
      throw new IllegalArgumentException("Null 'chartOrientation' argument.") ;
    }
    this.chartOrientation = chartOrientation ;
  }

  /**
   * <zh>ͼķ.</zh>
   * <en>Returns the orientation of the plot.</en>
   *
   * @return <zh>ͼķ (Ϊnull).</zh>
   * <en>The orientation of the plot (never null).</en>
   */
  public ChartOrientation getChartOrientation() {
    return this.chartOrientation ;
  }

  /**
   * <zh>ͼķ,еļ{@link RenderChangeEvent}¼.</zh>
   * <en>Sets the orientation for the plot and sends a {@link RenderChangeEvent} to
   * all registered listeners.</en>
   *
   * @param chartOrientation ChartOrientation <zh>ͼķ (Ϊnull).</zh>
   * <en>The orientation (null not permitted).</en>
   */
  public void setOrientation(ChartOrientation chartOrientation) {
    if (chartOrientation == null) {
      throw new IllegalArgumentException("Null 'chartOrientation' argument.") ;
    }
    this.chartOrientation = chartOrientation ;
    notifyListeners(new RenderChangeEvent(this)) ;
  }

  /**
   * <zh>ֻͼ,ͼε.</zh>
   * <en>Create a shape with no attribute specified.</en>
   *
   * @param rect Rectangle2D <zh>ͼľ (Ϊnull).</zh>
   * <en>The bar rectangle (null not permitted).</en>
   * @param text String <zh>Ҫʾ (Ϊnull).</zh>
   * <en>The text (null permitted).</en>
   * @param appendParam Object <zh>ӵĲ,ڴݶĶ (Ϊnull).</zh>
   * <en>The append parameter,used for special propose (null permitted).</en>
   *
   * @return IChartShape <zh>ͼ.</zh>
   * <en>The shape.</en>
   */
  protected IChartShape createItemShapeOnly(Rectangle2D rect, String text,
                                            Object appendParam) {
    //ƫx,y offset
    Rectangle2D cloneRect = (Rectangle2D) rect.clone() ;

    if (this.chartOrientation.equals(ChartOrientation.VERTICAL)) {
      cloneRect.setRect(cloneRect.getX(), cloneRect.getY() + getEffect3D().getYOffset() * 0.5,
                        cloneRect.getWidth(), cloneRect.getHeight()) ;
    } else if (this.chartOrientation.equals(ChartOrientation.HORIZONTAL)) {
      cloneRect.setRect(cloneRect.getX() - getEffect3D().getXOffset() * 0.5,
                        cloneRect.getY(), cloneRect.getWidth(), cloneRect.getHeight()) ;
    }

    CylinderTextChartShape cylinder = new CylinderTextChartShape(cloneRect, text) ;
    if (this.chartOrientation.equals(ChartOrientation.VERTICAL)) {
      cylinder.setDirection(Direction.VERTICAL) ;
    } else if (this.chartOrientation.equals(ChartOrientation.HORIZONTAL)) {
      cylinder.setDirection(Direction.HORIZONTAL) ;
    }
    cylinder.setXOffset(getEffect3D().getXOffset()) ;
    cylinder.setYOffset(getEffect3D().getYOffset()) ;

    return cylinder ;
  }

  /**
   * <zh>Ƿ.</zh>
   * <en>Tests this instance for equality with an arbitrary object.</en>
   *
   * @param obj  the object (null permitted).
   *
   * @return A boolean.
   */
  public boolean equals(Object obj) {
    if (obj == this) {
      return true ;
    }
    if (!(obj instanceof CylinderRender)) {
      return false ;
    }
    if (!super.equals(obj)) {
      return false ;
    }

    CylinderRender that = (CylinderRender) obj ;
    if (EqualsUtil.equals(this.chartOrientation, that.chartOrientation)) {
      return false ;
    }

    return true ;
  }

  /**
   * <zh>hashcodeֵ.</zh>
   * <en>Returns a hash code value for the object.</en>
   *
   * @return int <zh>hashcodeֵ.</zh>
   * <en>A hash code value for this object.</en>
   */
  public int hashCode() {
    int result = super.hashCode() ;
    result = 37 * result + HashcodeUtil.hashCode(this.chartOrientation) ;

    return result ;
  }

  /**
   * <zh>ؿ¡Ķ.</zh>
   * <en>Creates and returns a copy of this object.</en>
   * @return object   a clone of this instance.
   * @exception  CloneNotSupportedException  if the object's class does not
   *               support the Cloneable interface. Subclasses
   *               that override the clone method can also
   *               throw this exception to indicate that an instance cannot
   *               be cloned.
   */
  public Object clone() throws CloneNotSupportedException {
    return super.clone() ;
  }

  /**
   * <zh>toString.</zh>
   * <en>Returns a string representation of the object.</en>
   *
   * @return  a string representation of the object.
   */
  public String toString() {
    return "[chartOrientation=" + this.chartOrientation + "]" ;
  }
}
