package com.ijchart.xychart.render.noaxis ;

import java.awt.Paint ;
import java.awt.Stroke ;
import java.awt.geom.Rectangle2D ;
import java.io.Serializable ;

import com.ijchart.xychart.chartinfo.RenderShapeCollection ;
import com.ijchart.xychart.dataset.IMatrixDataset ;
import com.ijchart.xychart.label.ILabelGenerator ;
import com.ijchart.xychart.render.AbstractRender ;
import com.ijchart.xychart.shape.AbstractChartShape ;
import com.ijchart.xychart.shape.AbstractRectangleTextChartShape ;
import com.ijchart.xychart.shape.IChartShape ;
import com.ijchart.xychart.tooltip.IToolTipGenerator ;
import com.ijchart.xychart.util.StringUtil ;

/**
 * <p>Title: IJChart</p>
 *
 * <p>Description: a chart library for the Java(tm) platform.</p>
 * <zh>ṩûrenderĳʵ.</zh>
 * <en>Base class providing common services for no axis renders.</en>
 *
 * <p>Copyright: Copyright (c) 2013</p>
 *
 * <p>Company: </p>
 *
 * @author zhang feng min,shanghai china.
 * <a href="mailto:ijchartsh@gmail.com">Contract me by email.</a>
 * @version 1.0
 */
public abstract class AbstractNoAxisRender extends AbstractRender implements Cloneable,
  Serializable {
  /**
   * <zh>л汾ID.</zh>
   * <en>For serialization.</en>
   */
  private static final long serialVersionUID = -4897953022216086942L ;

  /**
   * <zh>.</zh>
   * <en>constructor.</en>
   *
   * @param dataset IMatrixDataset <zh>ݼ (Ϊnull).</zh>
   * <en>The dataset (null not permitted).</en>
   */
  protected AbstractNoAxisRender(IMatrixDataset dataset) {
    super(dataset) ;
  }

  /**
   * <zh>еͼ.</zh>
   * <en>Draws all data items.</en>
   *
   * @param dataArea Rectangle2D <zh> (Ϊnull).</zh>
   * <en>The chart data area (null not permitted).</en>
   * @param renderShapes RenderShapeCollection <zh>renderͼμ (Ϊnull).</zh>
   * <en>Collects information about the render (null not permitted).</en>
   * @param alpha float <zh>alphaֵ,ڱ[0.0,1.0].</zh>
   * <en>The alpha transparency (in the range 0.0f to 1.0f, where 0.0f is fully transparent,
   * and 1.0f is fully opaque).</en>
   */
  public abstract void drawAllItems(Rectangle2D dataArea, RenderShapeCollection renderShapes,
                                    float alpha) ;

  /**
   * <zh>һRectangleTextChartShape.</zh>
   * <en>Create a RectangleTextChartShape object.</en>
   *
   * @param row int <zh>ݼ.</zh>
   * <en>The dataset row num.</en>
   * @param column int <zh>ݼ.</zh>
   * <en>The dataset column num.</en>
   * @param autoFillPaint <zh>ԶѡPaint (Ϊnull).</zh>
   * <en>The auto selected fill paint (null permitted).</en>
   * @param autoBorderPaint <zh>ԶѡĻƱ߿Paint (Ϊnull).</zh>
   * <en>The auto selected border paint (null permitted).</en>
   * @param autoBorderStroke <zh>ԶѡĻƱ߿Stroke (Ϊnull).</zh>
   * <en>The auto selected border stroke (null permitted).</en>
   * @param alpha float <zh>alphaֵ,ڱ[0.0,1.0].</zh>
   * <en>The alpha transparency (in the range 0.0f to 1.0f, where 0.0f is fully transparent,
   * and 1.0f is fully opaque).</en>
   * @param appendParam Object <zh>ӵĲ,ڴݶĶ (Ϊnull).</zh>
   * <en>The append parameter,used for special propose (null permitted).</en>
   *
   * @return IChartShape
   */
  protected IChartShape createItemShape(int row, int column, Paint autoFillPaint,
                                        Paint autoBorderPaint, Stroke autoBorderStroke,
                                        float alpha, Object appendParam) {
    if (!isItemVisible(row, column)) {
      return null ;
    }

    String label = null ;
    ILabelGenerator labelGen = getLabelGenerator() ;
    if (labelGen != null && isItemLabelVisible(row, column)) {
      if (labelGen.isGroup()) {
        int index = 0 ;
        if (appendParam != null && (appendParam instanceof Number)) {
          index = ((Number) appendParam).intValue() ;
        }
        label = labelGen.generateGroupLabel(getDataset(), row, column, index) ;
      } else {
        label = labelGen.generateLabel(getDataset(), row, column) ;
      }
    }

    IChartShape shape = createItemShapeOnly(null, label, appendParam) ;
    Paint fillPaint = getItemFillPaintHelper(autoFillPaint, row, column) ;
    if (fillPaint != null) {
      shape.setFillPaint(fillPaint) ;
    }
    shape.setFillShape(isItemFill(row, column)) ;
    shape.setGradientPaintTransformType(getItemFillPaintTransformer(row, column)) ;
    if (shape instanceof AbstractChartShape) {
      ((AbstractChartShape) shape).setFillAlpha(alpha) ;
    }

    shape.setDrawBorder(isItemDrawBorder(row, column)) ;
    if (isAutoSelectBorderPaint() && autoBorderPaint != null) {
      shape.setBorderPaint(autoBorderPaint) ;
    } else {
      shape.setBorderPaint(getItemBorderPaint(row, column)) ;
    }
    if (isAutoSelectBorderStroke() && autoBorderStroke != null) {
      shape.setBorderStroke(autoBorderStroke) ;
    } else {
      shape.setBorderStroke(getItemBorderStroke(row, column)) ;
    }

    if (shape instanceof AbstractRectangleTextChartShape) {
      AbstractRectangleTextChartShape textShape = (AbstractRectangleTextChartShape) shape ;
      if (StringUtil.isEmpty(textShape.getText())) {
        textShape.setText(label) ;
      }
      textShape.setTextPosition(getItemLabelPosition(row, column)) ;
      textShape.setTextDirection(getItemLabelDirection(row, column)) ;
      textShape.setDrawText(isItemLabelVisible(row, column)) ;
      textShape.setTextFont(getItemLabelFont(row, column)) ;
      textShape.setTextPaint(getItemLabelPaint(row, column)) ;

      textShape.setForceInsideTextNoWrap(true) ; //ǿƲ
      textShape.setFillAlpha(alpha) ;
    }

    IToolTipGenerator tooltipGen = getToolTipGenerator() ;
    if (tooltipGen != null && isGeneratorTooltip()) {
      String tooltip = null ;
      if (tooltipGen.isGroup()) {
        int index = 0 ;
        if (appendParam != null && (appendParam instanceof Number)) {
          index = ((Number) appendParam).intValue() ;
        }
        tooltip = tooltipGen.generateGroupToolTip(getDataset(), row, column, index) ;
      } else {
        tooltip = tooltipGen.generateToolTip(getDataset(), row, column) ;
      }
      if (!StringUtil.isEmptyWithTrim(tooltip)) {
        shape.setTooltipText(tooltip) ;
      }
    }

    if (shape != null) {
      shape.setData(getDataset().getData(row, column)) ;
    }

    return shape ;
  }

  /**
   * <zh>Ƿ.</zh>
   * <en>Indicates if some other object is "equal to" this one.</en>
   *
   * @param obj <zh>ҪȽϵĶ</zh>
   * <en>the reference object with which to compare.</en>
   * @return <zh>ȷtrue,򷵻false.</zh>
   * <en>true if this object is the same as the obj argument; false otherwise.</en>
   */
  public boolean equals(Object obj) {
    if (this == obj) {
      return true ;
    }
    if (!(obj instanceof AbstractNoAxisRender)) {
      return false ;
    }
    if (!super.equals(obj)) {
      return false ;
    }

    return true ;
  }

  /**
   * <zh>hashcodeֵ.</zh>
   * <en>Returns a hash code value for the object.</en>
   *
   * @return int <zh>hashcodeֵ.</zh>
   * <en>A hash code value for this object.</en>
   */
  public int hashCode() {
    return super.hashCode() ;
  }

  /**
   * <zh>ؿ¡Ķ.</zh>
   * <en>Creates and returns a copy of this object.</en>
   * @return object   a clone of this instance.
   * @exception  CloneNotSupportedException  if the object's class does not
   *               support the Cloneable interface. Subclasses
   *               that override the clone method can also
   *               throw this exception to indicate that an instance cannot
   *               be cloned.
   */
  public Object clone() throws CloneNotSupportedException {
    return super.clone() ;
  }

  /**
   * <zh>toString.</zh>
   * <en>Returns a string representation of the object.</en>
   *
   * @return  a string representation of the object.
   */
  public String toString() {
    return "AbstractNoAxisRender" ;
  }

}
