package com.ijchart.xychart.render.noaxis ;

import java.awt.BasicStroke ;
import java.awt.Color ;
import java.awt.Font ;
import java.awt.GradientPaint ;
import java.awt.Paint ;
import java.awt.Stroke ;
import java.awt.geom.GeneralPath ;
import java.awt.geom.Point2D ;
import java.awt.geom.Rectangle2D ;
import java.io.IOException ;
import java.io.ObjectInputStream ;
import java.io.ObjectOutputStream ;
import java.io.Serializable ;

import com.ijchart.xychart.axis.value.AbstractValueAxis ;
import com.ijchart.xychart.axis.value.XYNumberValueAxis ;
import com.ijchart.xychart.chartinfo.RenderShapeCollection ;
import com.ijchart.xychart.constant.ChartOrientation ;
import com.ijchart.xychart.constant.DataBackgroundShapeDrawType ;
import com.ijchart.xychart.constant.HorizontalAlignment ;
import com.ijchart.xychart.constant.LegendType ;
import com.ijchart.xychart.constant.TextPosition ;
import com.ijchart.xychart.constant.TransformShapeType ;
import com.ijchart.xychart.data.IChartData ;
import com.ijchart.xychart.data.XYNumberChartData ;
import com.ijchart.xychart.dataset.IMatrixDataset ;
import com.ijchart.xychart.render.INoAxisRender ;
import com.ijchart.xychart.shape.EllipseTextChartShape ;
import com.ijchart.xychart.shape.GeneralPathChartShape ;
import com.ijchart.xychart.shape.IChartShape ;
import com.ijchart.xychart.shape.LineChartShape ;
import com.ijchart.xychart.shape.RectangleTextChartShape ;
import com.ijchart.xychart.ui.RectangleEdge ;
import com.ijchart.xychart.util.AngleUtil ;
import com.ijchart.xychart.util.EqualsUtil ;
import com.ijchart.xychart.util.HashcodeUtil ;
import com.ijchart.xychart.util.SerialUtil ;

/**
 * <p>Title: IJChart</p>
 *
 * <p>Description: a chart library for the Java(tm) platform.</p>
 * <zh>Ƽͼ.</zh>
 * <en>Draw polar chart shapes.</en>
 * <p>Copyright: Copyright (c) 2013</p>
 *
 * <p>Company: </p>
 *
 * @author zhang feng min,shanghai china.
 * <a href="mailto:ijchartsh@gmail.com">Contract me by email.</a>
 * @version 1.0
 */
public class PolarRender extends AbstractNoAxisRender implements INoAxisRender, Cloneable,
  Serializable {
  /**
   * <zh>л汾ID.</zh>
   * <en>For serialization.</en>
   */
  private static final long serialVersionUID = -1729867463882569495L ;

  /**
   * <zh>ȱʡĻܵĿհ.</zh>
   * <en>The default margin around data area.</en>
   */
  public static final double DEFAULT_MARGIN = 20 ;

  /**
   * <zh>ȱʡĻߵstroke.</zh>
   * <en>The default grid line stroke.</en>
   */
  public static final Stroke DEFAULT_GRIDLINE_STROKE = new BasicStroke(
    0.5f, BasicStroke.CAP_BUTT, BasicStroke.JOIN_BEVEL,
    0.0f, new float[] {2.0f, 2.0f}, 0.0f) ;

  /**
   * <zh>ȱʡĻߵpaint.</zh>
   * <en>The default grid line paint.</en>
   */
  public static final Paint DEFAULT_GRIDLINE_PAINT = Color.gray ;

  /**
   * <zh>ȱʡĽǶȱǩǷʾ.</zh>
   * <en>A default flag that controls whether or not the angle labels are visible.</en>
   */
  public static final boolean DEFAULT_ANGLE_LABELS_VISIBLE = true ;

  /**
   * <zh>ȱʡĻƽǶȱǩfont.</zh>
   * <en>The default font used to draw the angle labels.</en>
   */
  public static final Font DEFAULT_ANGLE_LABEL_FONT = new Font("SansSerif", Font.PLAIN, 12) ;

  /**
   * <zh>Ʊǩpaint.</zh>
   * <en>The paint used to display the angle labels.</en>
   */
  public static final Paint DEFAULT_LABEL_PAINT = Color.black ;

  /**
   * <zh>ȱʡǶǷɼ.</zh>
   * <en>A default flag that controls whether the angle grid-lines are visible.</en>
   */
  public static final boolean DEFAULT_ANGLE_GRIDLINES_VISIBLE = true ;

  /**
   * <zh>ȱʡ뾶Ƿɼ.</zh>
   * <en>A default flag that controls whether the radius grid-lines are visible.</en>
   */
  public static final boolean DEFAULT_RADIUS_GRIDLINES_VISIBLE = true ;

  /**
   * <zh>ܵĿհ.</zh>
   * <en>The margin around data area.</en>
   */
  private double margin ;

  /**
   * <zh>ǶȱǩǷʾ.</zh>
   * <en>A flag that controls whether or not the angle labels are visible.</en>
   */
  private boolean angleLabelsVisible ;

  /**
   * <zh>ƽǶȱǩfont.</zh>
   * <en>The font used to draw the angle labels.</en>
   */
  private Font angleLabelFont ;

  /**
   * <zh>Ʊǩpaint.</zh>
   * <en>The paint used to display the angle labels.</en>
   */
  private transient Paint angleLabelPaint ;

  /**
   * <zh>ǶǷɼ.</zh>
   * <en>A flag that controls whether the angle grid-lines are visible.</en>
   */
  private boolean angleGridlinesVisible ;

  /**
   * <zh>ƽǶߵstroke.</zh>
   * <en>The stroke used to draw the angle grid-lines.</en>
   */
  private transient Stroke angleGridlineStroke ;

  /**
   * <zh>ƽǶߵpaint.</zh>
   * <en>The paint used to draw the angle grid-lines.</en>
   */
  private transient Paint angleGridlinePaint ;

  /**
   * <zh>뾶Ƿɼ.</zh>
   * <en>A flag that controls whether the radius grid-lines are visible.</en>
   */
  private boolean radiusGridlinesVisible ;

  /**
   * <zh>ư뾶ߵstroke.</zh>
   * <en>The stroke used to draw the radius grid-lines.</en>
   */
  private transient Stroke radiusGridlineStroke ;

  /**
   * <zh>ư뾶ߵpaint.</zh>
   * <en>The paint used to draw the radius grid-lines.</en>
   */
  private transient Paint radiusGridlinePaint ;

  /**
   * <zh>ƿ̶ߵ.</zh>
   * <en>The axis for draw tick marks.</en>
   */
  private XYNumberValueAxis axis ;

  /**
   * <zh>.</zh>
   * <en>constructor.</en>
   *
   * @param dataset IMatrixDataset <zh>ݼ (Ϊnull).</zh>
   * <en>The dataset (null not permitted).</en>
   */
  public PolarRender(IMatrixDataset dataset) {
    super(dataset) ;

    this.margin = DEFAULT_MARGIN ;
    this.angleLabelsVisible = DEFAULT_ANGLE_LABELS_VISIBLE ;
    this.angleLabelFont = DEFAULT_ANGLE_LABEL_FONT ;
    this.angleLabelPaint = DEFAULT_LABEL_PAINT ;
    this.angleGridlinesVisible = DEFAULT_ANGLE_GRIDLINES_VISIBLE ;
    this.angleGridlineStroke = DEFAULT_GRIDLINE_STROKE ;
    this.angleGridlinePaint = DEFAULT_GRIDLINE_PAINT ;
    this.radiusGridlinesVisible = DEFAULT_RADIUS_GRIDLINES_VISIBLE ;
    this.radiusGridlineStroke = DEFAULT_GRIDLINE_STROKE ;
    this.radiusGridlinePaint = DEFAULT_GRIDLINE_PAINT ;
    this.axis = null ;
  }

  /**
   * <zh>ػܵĿհ.</zh>
   * <en>Returns the margin around data area.</en>
   *
   * @return double <zh>հ׵ĴС.</zh>
   * <en>The margin.</en>
   */
  public double getMargin() {
    return this.margin ;
  }

  /**
   * <zh>ûܵĿհ,м
   * {@link com.ijchart.xychart.event.RenderChangeEvent}¼.</zh>
   * <en>Sets the margin around data area and sends a
   * {@link com.ijchart.xychart.event.RenderChangeEvent} to all registered listeners.</en>
   *
   * @param margin double <zh>հ׵ĴС.</zh>
   * <en>The margin.</en>
   */
  public void setMargin(double margin) {
    if (margin < 0) {
      throw new IllegalArgumentException("Argument 'margin' must be >=0.") ;
    }

    if (this.margin != margin) {
      this.margin = margin ;
      fireRenderChanged() ;
    }
  }

  /**
   * <zh>ؽǶȱǩǷʾ.</zh>
   * <en>Returns the flag that whether or not the angle labels are visible.</en>
   *
   * @return boolean <zh>ֵ.</zh>
   * <en>A boolean.</en>
   */
  public boolean isAngleLabelsVisible() {
    return this.angleLabelsVisible ;
  }

  /**
   * <zh>ýǶȱǩǷʾ,м
   * {@link com.ijchart.xychart.event.RenderChangeEvent}¼.</zh>
   * <en>Sets the flag that controls whether or not the angle labels are visible and sends a
   * {@link com.ijchart.xychart.event.RenderChangeEvent} to all registered listeners.</en>
   *
   * @param flag boolean <zh>ֵ.</zh>
   * <en>A boolean.</en>
   */
  public void setAngleLabelsVisible(boolean flag) {
    if (this.angleLabelsVisible != flag) {
      this.angleLabelsVisible = flag ;
      fireRenderChanged() ;
    }
  }

  /**
   * <zh>ػƽǶȱǩfont.</zh>
   * <en>Returns the font used to draw the angle labels.</en>
   *
   * @return Font <zh>font (Ϊnull).</zh>
   * <en>The font (never null).</en>
   */
  public Font getAngleLabelFont() {
    return this.angleLabelFont ;
  }

  /**
   * <zh>ûƽǶȱǩfont,м
   * {@link com.ijchart.xychart.event.RenderChangeEvent}¼.</zh>
   * <en>Sets the font used to draw the angle labels and sends a
   * {@link com.ijchart.xychart.event.RenderChangeEvent} to all registered listeners.</en>
   *
   * @param font Font <zh>font (Ϊnull).</zh>
   * <en>The font (null not permitted).</en>
   */
  public void setAngleLabelFont(Font font) {
    if (font == null) {
      throw new IllegalArgumentException("Null 'font' argument") ;
    }

    this.angleLabelFont = font ;
    fireRenderChanged() ;
  }

  /**
   * <zh>ػƱǩpaint.</zh>
   * <en>Returns the paint used to display the angle labels.</en>
   *
   * @return Paint <zh>paint (Ϊnull).</zh>
   * <en>The paint (never null).</en>
   */
  public Paint getAngleLabelPaint() {
    return this.angleLabelPaint ;
  }

  /**
   * <zh>ûƱǩpaint,м
   * {@link com.ijchart.xychart.event.RenderChangeEvent}¼.</zh>
   * <en>Sets the paint used to display the angle labels and sends a
   * {@link com.ijchart.xychart.event.RenderChangeEvent} to all registered listeners.</en>
   *
   * @param paint Paint <zh>paint (Ϊnull).</zh>
   * <en>The paint (null not permitted).</en>
   */
  public void setAngleLabelPaint(Paint paint) {
    if (paint == null) {
      throw new IllegalArgumentException("Null 'paint' argument") ;
    }

    this.angleLabelPaint = paint ;
    fireRenderChanged() ;
  }

  /**
   * <zh>ؽǶǷɼ.</zh>
   * <en>Returns the flag that controls whether the angle grid-lines are visible.</en>
   *
   * @return boolean <zh>ֵ.</zh>
   * <en>A boolean.</en>
   */
  public boolean isAngleGridlinesVisible() {
    return this.angleGridlinesVisible ;
  }

  /**
   * <zh>ýǶǷɼ,м
   * {@link com.ijchart.xychart.event.RenderChangeEvent}¼.</zh>
   * <en>A flag that controls whether the angle grid-lines are visible and sends a
   * {@link com.ijchart.xychart.event.RenderChangeEvent} to all registered listeners.</en>
   *
   * @param flag boolean <zh>ֵ.</zh>
   * <en>A boolean.</en>
   */
  public void setAngleGridlinesVisible(boolean flag) {
    if (this.angleGridlinesVisible != flag) {
      this.angleGridlinesVisible = flag ;
      fireRenderChanged() ;
    }
  }

  /**
   * <zh>ػƽǶߵstroke.</zh>
   * <en>Returns the stroke used to draw the angle grid-lines.</en>
   *
   * @return Stroke <zh>stroke (Ϊnull).</zh>
   * <en>The stroke (never null).</en>
   */
  public Stroke getAngleGridlineStroke() {
    return this.angleGridlineStroke ;
  }

  /**
   * <zh>ûƽǶߵstroke,м
   * {@link com.ijchart.xychart.event.RenderChangeEvent}¼.</zh>
   * <en>Sets the stroke used to draw the angle grid-lines and sends a
   * {@link com.ijchart.xychart.event.RenderChangeEvent} to all registered listeners.</en>
   *
   * @param stroke Stroke <zh>stroke (Ϊnull).</zh>
   * <en>The stroke (null not permitted).</en>
   */
  public void setAngleGridlineStroke(Stroke stroke) {
    if (stroke == null) {
      throw new IllegalArgumentException("Null 'stroke' argument") ;
    }

    this.angleGridlineStroke = stroke ;
    fireRenderChanged() ;
  }

  /**
   * <zh>ػƽǶߵpaint.</zh>
   * <en>Returns the paint used to draw the angle grid-lines.</en>
   *
   * @return Paint <zh>paint (Ϊnull).</zh>
   * <en>The paint (never null).</en>
   */
  public Paint getAngleGridlinePaint() {
    return this.angleGridlinePaint ;
  }

  /**
   * <zh>ûƽǶߵpaint,м
   * {@link com.ijchart.xychart.event.RenderChangeEvent}¼.</zh>
   * <en>Sets the paint used to draw the angle grid-lines and sends a
   * {@link com.ijchart.xychart.event.RenderChangeEvent} to all registered listeners.</en>
   *
   * @param paint Paint <zh>paint (Ϊnull).</zh>
   * <en>The paint (null not permitted).</en>
   */
  public void setAngleGridlinePaint(Paint paint) {
    if (paint == null) {
      throw new IllegalArgumentException("Null 'paint' argument") ;
    }

    this.angleGridlinePaint = paint ;
    fireRenderChanged() ;
  }

  /**
   * <zh>ذ뾶Ƿɼ.</zh>
   * <en>Returns the flag that controls whether the radius grid-lines are visible.</en>
   *
   * @return boolean <zh>ֵ.</zh>
   * <en>A boolean.</en>
   */
  public boolean isRadiusGridlinesVisible() {
    return radiusGridlinesVisible ;
  }

  /**
   * <zh>ð뾶Ƿɼ,м
   * {@link com.ijchart.xychart.event.RenderChangeEvent}¼.</zh>
   * <en>A flag that controls whether the radius grid-lines are visible and sends a
   * {@link com.ijchart.xychart.event.RenderChangeEvent} to all registered listeners.</en>
   *
   * @param flag boolean <zh>ֵ.</zh>
   * <en>A boolean.</en>
   */
  public void setRadiusGridlinesVisible(boolean flag) {
    if (this.radiusGridlinesVisible != flag) {
      this.radiusGridlinesVisible = flag ;
      fireRenderChanged() ;
    }
  }

  /**
   * <zh>ػư뾶ߵstroke.</zh>
   * <en>Returns the stroke used to draw the radius grid-lines.</en>
   *
   * @return Stroke <zh>stroke (Ϊnull).</zh>
   * <en>The stroke (never null).</en>
   */
  public Stroke getRadiusGridlineStroke() {
    return this.radiusGridlineStroke ;
  }

  /**
   * <zh>ûư뾶ߵstroke,м
   * {@link com.ijchart.xychart.event.RenderChangeEvent}¼.</zh>
   * <en>Sets the stroke used to draw the radius grid-lines and sends a
   * {@link com.ijchart.xychart.event.RenderChangeEvent} to all registered listeners.</en>
   *
   * @param stroke Stroke <zh>stroke (Ϊnull).</zh>
   * <en>The stroke (null not permitted).</en>
   */
  public void setRadiusGridlineStroke(Stroke stroke) {
    if (stroke == null) {
      throw new IllegalArgumentException("Null 'stroke' argument") ;
    }

    this.radiusGridlineStroke = stroke ;
    fireRenderChanged() ;
  }

  /**
   * <zh>ػư뾶ߵpaint,м
   * {@link com.ijchart.xychart.event.RenderChangeEvent}¼.</zh>
   * <en>Returns the paint used to draw the radius grid-lines and sends a
   * {@link com.ijchart.xychart.event.RenderChangeEvent} to all registered listeners.</en>
   *
   * @return Paint <zh>paint (Ϊnull).</zh>
   * <en>The paint (never null).</en>
   */
  public Paint getRadiusGridlinePaint() {
    return this.radiusGridlinePaint ;
  }

  /**
   * <zh>ûư뾶ߵpaint.</zh>
   * <en>Sets the paint used to draw the radius grid-lines.</en>
   *
   * @param paint Paint <zh>paint (Ϊnull).</zh>
   * <en>The paint (null not permitted).</en>
   */
  public void setRadiusGridlinePaint(Paint paint) {
    if (paint == null) {
      throw new IllegalArgumentException("Null 'paint' argument") ;
    }

    this.radiusGridlinePaint = paint ;
    fireRenderChanged() ;
  }

  /**
   * <zh>ػƿ̶ߵ.</zh>
   * <en>Returns the axis for draw tick marks.</en>
   *
   * @return XYNumberValueAxis <zh> (Ϊnull).</zh>
   * <en>The axis (maybe null).</en>
   */
  public XYNumberValueAxis getAxis() {
    return this.axis ;
  }

  /**
   * <zh>ûƿ̶ߵ.</zh>
   * <en>Sets the axis for draw tick marks.</en>
   *
   * @param axis XYNumberValueAxis <zh> (Ϊnull).</zh>
   * <en>The axis (null permitted).</en>
   */
  public void setAxis(XYNumberValueAxis axis) {
    this.axis = axis ;
  }

  /**
   * <zh>еͼ.</zh>
   * <en>Draws all data items.</en>
   *
   * @param dataArea Rectangle2D <zh> (Ϊnull).</zh>
   * <en>The chart data area (null not permitted).</en>
   * @param renderShapes RenderShapeCollection <zh>renderͼμ (Ϊnull).</zh>
   * <en>Collects information about the render (null not permitted).</en>
   * @param alpha float <zh>alphaֵ,ڱ[0.0,1.0].</zh>
   * <en>The alpha transparency (in the range 0.0f to 1.0f, where 0.0f is fully transparent,
   * and 1.0f is fully opaque).</en>
   */
  public void drawAllItems(Rectangle2D dataArea, RenderShapeCollection renderShapes, float alpha) {
    if (dataArea == null) {
      throw new IllegalArgumentException("Null 'dataArea' argument.") ;
    }
    if (renderShapes == null) {
      throw new IllegalArgumentException("Null 'renderShapes' argument.") ;
    }
    if (alpha < 0.0f || alpha > 1.0f) {
      throw new IllegalArgumentException("The 'alpha' value must be in the range 0.0f to 1.0f.") ;
    }

    IMatrixDataset dataset = getDataset() ;
    if (dataset.isEmpty()) {
      return ;
    }

    int rows = dataset.getRowCount() ;
    this.fillPaints = getFillPaints(rows) ;
    this.borderPaints = getBorderPaints(rows) ;
    this.borderStrokes = getBorderStrokes(rows) ;

    double centerX = dataArea.getCenterX() ;
    double centerY = dataArea.getCenterY() ;
    renderShapes.setDataBackgroundShapeDrawType(DataBackgroundShapeDrawType.ONE_BY_ROW) ;

    //draw radius gridlines
    double maxRadius = Double.MIN_VALUE ; //the max radius
    double areaSize = Math.min(dataArea.getWidth(), dataArea.getHeight()) * 0.5 - this.margin ;
    RectangleEdge edge = RectangleEdge.TOP ;
    Rectangle2D axisArea = new Rectangle2D.Double(centerX, centerY, areaSize, areaSize) ;

    if (this.axis != null && this.radiusGridlinesVisible) {
      String[] ticks = this.axis.getNumberTicks() ;
      if (ticks != null && ticks.length > 0) {
        for (int i = 0 ; i < ticks.length ; i++) {
          double tickValue = Double.parseDouble(ticks[i]) ;
          double tickPos = this.axis.valueToJava2D(axisArea, edge, tickValue, false) ;
          if (tickPos == AbstractValueAxis.INVALID_COORDINATE_VALUE) {
            renderShapes.addRadiusGridlineShape(null) ;
            continue ;
          }
          double radius = tickPos - centerX ;
          maxRadius = Math.max(maxRadius, radius) ;

          double upperLeftX = centerX - radius ;
          double uppperLeftY = centerY - radius ;
          Rectangle2D drawArea = new Rectangle2D.Double(upperLeftX, uppperLeftY,
                                                        radius * 2, radius * 2) ;
          EllipseTextChartShape ellipseShape = new EllipseTextChartShape(drawArea, null) ;
          ellipseShape.setTransformShapeType(TransformShapeType.OUTLINE) ;
          ellipseShape.setBorderPaint(this.radiusGridlinePaint) ;
          ellipseShape.setBorderStroke(this.radiusGridlineStroke) ;
          ellipseShape.setFillShape(false) ;
          ellipseShape.setDrawBorder(true) ;

          renderShapes.addRadiusGridlineShape(ellipseShape) ;
        }
      }
    }

    //draw angle gridlines
    String[][] ticks = getAngleTicks() ;
    if (this.angleGridlinesVisible && ticks != null) {
      if (maxRadius <= 0) {
        maxRadius = Math.min(dataArea.getWidth(), dataArea.getHeight()) * 0.5 - this.margin ;
      }

      for (int i = 0 ; i < ticks.length ; i++) {
        double tick = Double.parseDouble(ticks[i][0]) - 90.0 ;
        double x = centerX + maxRadius * Math.cos(tick / 180.0 * Math.PI) ;
        double y = centerY + maxRadius * Math.sin(tick / 180.0 * Math.PI) ;

        LineChartShape lineShape = new LineChartShape(centerX, centerY, x, y) ;
        lineShape.setDrawBorder(true) ;
        lineShape.setBorderPaint(this.angleGridlinePaint) ;
        lineShape.setBorderStroke(this.angleGridlineStroke) ;
        renderShapes.addAngleGridlineShape(lineShape) ;

        Rectangle2D textArea = new Rectangle2D.Double(x, y, 1, 1) ;
        RectangleTextChartShape textShape = new RectangleTextChartShape(textArea, ticks[i][1]) ;
        textShape.setFillShape(false) ;
        textShape.setDrawBorder(false) ;

        double modAngle = tick % 360.0 ;
        if (modAngle == -90.0) {
          textShape.setTextPosition(TextPosition.OUTSIDE_TOP) ;
          textShape.setTextHorizontalAlign(HorizontalAlignment.CENTER) ;
        } else if (modAngle == 0.0) {
          textShape.setTextPosition(TextPosition.OUTSIDE_RIGHT) ;
          textShape.translate(5, -3) ;
        } else if (modAngle == 90.0) {
          textShape.setTextPosition(TextPosition.OUTSIDE_BOTTOM) ;
          textShape.setTextHorizontalAlign(HorizontalAlignment.CENTER) ;
        } else if (modAngle == 180.0) {
          textShape.setTextPosition(TextPosition.OUTSIDE_LEFT) ;
        } else {
          int quadrant = AngleUtil.getQuadrantByDegreeAngle(tick) ;
          if (quadrant == 1 || quadrant == 4) {
            textShape.setTextPosition(TextPosition.OUTSIDE_RIGHT) ;
          } else if (quadrant == 2 || quadrant == 3) {
            textShape.setTextPosition(TextPosition.OUTSIDE_LEFT) ;
          }
        }
        renderShapes.addPieLabelShape(textShape) ;
      }
    }

    //draw data shapes
    for (int i = 0 ; i < rows ; i++) {
      int currRowCols = dataset.getColumnCount(i) ;
      Point2D[] points = new Point2D.Double[currRowCols] ;
      for (int j = 0 ; j < currRowCols ; j++) {
        IChartData data = dataset.getData(i, j) ;
        if (data != null && data instanceof XYNumberChartData) {
          XYNumberChartData xyData = (XYNumberChartData) data ;
          double theta = xyData.getX() - 90.0 ;
          double radius = xyData.getY() ;
          if (Double.isNaN(theta) || Double.isNaN(radius)) {
            points[j] = null ;
          } else {
            double r = axis.valueToJava2D(axisArea, edge, radius, false) ;
            if (r == AbstractValueAxis.INVALID_COORDINATE_VALUE) {
              renderShapes.addDataItemShape(null) ;
              renderShapes.addDataItemBackgroundShape(null) ;
              continue ;
            }
            r -= centerX ;
            double xPos = centerX + r * Math.cos(theta / 180.0 * Math.PI) ;
            double yPos = centerY + r * Math.sin(theta / 180.0 * Math.PI) ;
            points[j] = new Point2D.Double(xPos, yPos) ;
          }
        } else {
          points[j] = null ;
        }
      }

      for (int m = 0 ; m < currRowCols ; m++) {
        if (points[m] == null) {
          renderShapes.addDataItemShape(null) ;
        } else {
          LineChartShape lineShape = null ;
          boolean findNotNull = false ;
          for (int n = m + 1 ; n < currRowCols ; n++) {
            if (points[n] != null) {
              lineShape = new LineChartShape(points[m].getX(), points[m].getY(),
                                             points[n].getX(), points[n].getY()) ;
              findNotNull = true ;
              break ;
            }
          }

          if (!findNotNull) {
            for (int n = 0 ; n < m ; n++) {
              if (points[n] != null) {
                lineShape = new LineChartShape(points[m].getX(), points[m].getY(),
                                               points[n].getX(), points[n].getY()) ;
                break ;
              }
            }
          }

          if (lineShape != null) {
            IChartShape shape = createItemShape(i, m, null, fillPaints[i], borderStrokes[i],
                                                alpha, lineShape) ;
            renderShapes.addDataItemShape(shape) ;
          } else {
            renderShapes.addDataItemShape(null) ;
          }
        }
      }

      Boolean fill = isRowItemFill(i) ;
      if ((fill != null && fill.booleanValue()) || isBaseItemFill()) {
        GeneralPath path = new GeneralPath() ;
        boolean hasMove = false ;
        for (int m = 0 ; m < currRowCols ; m++) {
          if (points[m] != null) {
            if (!hasMove) {
              path.moveTo((float) points[m].getX(), (float) points[m].getY()) ;
              hasMove = true ;
            } else {
              path.lineTo((float) points[m].getX(), (float) points[m].getY()) ;
            }
          }
        }

        path.closePath() ;
        GeneralPathChartShape pathShape = new GeneralPathChartShape(path) ;
        pathShape.setFillShape(true) ;
        pathShape.setDrawBorder(false) ;
        pathShape.setFillPaint(fillPaints[i]) ;
        pathShape.setFillAlpha(alpha) ;
        renderShapes.addDataItemBackgroundShape(pathShape) ;
      } else {
        renderShapes.addDataItemBackgroundShape(null) ;
      }
    }

  }

  /**
   * <zh>еĽǶȿ̶(λǶ).</zh>
   * <en>Returns the angle ticks (in degree).</en>
   *
   * @return String[] <zh>Ƕֵ,άĵһֵǽǶ,ڶֵҪʾĿ̶ȱǩ.</zh>
   * <en>The angles array,the first element is degree,the second is tick label.</en>
   */
  protected String[][] getAngleTicks() {
    return new String[][] { {"0", "0"}, {"45", "45"}, {"90", "90"}, {"135", "135"}, {"180", "180"},
      {"225", "225"}, {"270", "270"}, {"315", "315"}
    } ;
  }

  /**
   * <zh>ͼ.</zh>
   * <en>Return the legend type.</en>
   *
   * @return LegendType <zh>ͼ.</zh>
   * <en>The legend type.</en>
   */
  public LegendType getLegendType() {
    return LegendType.ROW ;
  }

  /**
   * <zh>ֻͼ,ͼε.</zh>
   * <en>Create a shape with no attribute specified.</en>
   *
   * @param rect Rectangle2D <zh>ͼľ (Ϊnull).</zh>
   * <en>The bar rectangle (null not permitted).</en>
   * @param text String <zh>Ҫʾ (Ϊnull).</zh>
   * <en>The text (null permitted).</en>
   * @param appendParam Object <zh>ӵĲ,ڴݶĶ (Ϊnull).</zh>
   * <en>The append parameter,used for special propose (null permitted).</en>
   *
   * @return IChartShape <zh>ͼ.</zh>
   * <en>The shape.</en>
   */
  protected IChartShape createItemShapeOnly(Rectangle2D rect, String text,
                                            Object appendParam) {
    if (appendParam == null) {
      throw new IllegalArgumentException("Null 'appendParam' argument.") ;
    }

    return (IChartShape) appendParam ;
  }

  /**
   * <zh>Ʊɫ.</zh>
   * <en>Draws a background for the data area.</en>
   *
   * @param dataArea Rectangle2D <zh> (Ϊnull).</zh>
   * <en>The chart data area (null not permitted).</en>
   * @param orientation ChartOrientation <zh>ͼƷ (Ϊnull).</zh>
   * <en>The chart draw orientation (null not permitted).</en>
   * @param renderShapes RenderShapeCollection <zh>renderͼμ (Ϊnull).</zh>
   * <en>Collects information about the render (null not permitted).</en>
   */
  public void drawBackground(Rectangle2D dataArea, ChartOrientation orientation,
                             RenderShapeCollection renderShapes) {
    if (dataArea == null) {
      throw new IllegalArgumentException("Null 'dataArea' argument.") ;
    }
    if (orientation == null) {
      throw new IllegalArgumentException("Null 'orientation' argument.") ;
    }
    if (renderShapes == null) {
      throw new IllegalArgumentException("Null 'renderShapes' argument.") ;
    }

    Paint p = getBackgroundPaint() ;
    if (p == null) {
      return ;
    }

    if (p instanceof GradientPaint) {
      GradientPaint gp = (GradientPaint) p ;
      if (orientation.equals(ChartOrientation.VERTICAL)) {
        p = new GradientPaint((float) dataArea.getCenterX(), (float) dataArea.getMaxY(),
                              gp.getColor1(), (float) dataArea.getCenterX(),
                              (float) dataArea.getMinY(), gp.getColor2()) ;
      } else if (orientation.equals(ChartOrientation.HORIZONTAL)) {
        p = new GradientPaint((float) dataArea.getMinX(), (float) dataArea.getCenterY(),
                              gp.getColor1(), (float) dataArea.getMaxX(),
                              (float) dataArea.getCenterY(), gp.getColor2()) ;
      }
    }

    //ɫͼ
    EllipseTextChartShape shape = new EllipseTextChartShape(dataArea, null) ;
    shape.setDrawBorder(false) ;
    shape.setFillPaint(p) ;
    shape.setFillShape(true) ;

    renderShapes.setBackgroundShape(shape) ;
  }

  /**
   * <zh>Ƿ.</zh>
   * <en>Tests this instance for equality with an arbitrary object.</en>
   *
   * @param obj  the object (null permitted).
   *
   * @return A boolean.
   */
  public boolean equals(Object obj) {
    if (obj == this) {
      return true ;
    }
    if (!(obj instanceof PolarRender)) {
      return false ;
    }
    if (!super.equals(obj)) {
      return false ;
    }

    PolarRender that = (PolarRender) obj ;
    if (this.margin != that.margin) {
      return false ;
    }
    if (this.angleLabelsVisible != that.angleLabelsVisible) {
      return false ;
    }
    if (!EqualsUtil.equals(this.angleLabelFont, that.angleLabelFont)) {
      return false ;
    }
    if (!EqualsUtil.equals(this.angleLabelPaint, that.angleLabelPaint)) {
      return false ;
    }
    if (this.angleGridlinesVisible != that.angleGridlinesVisible) {
      return false ;
    }
    if (!EqualsUtil.equals(this.angleGridlineStroke, that.angleGridlineStroke)) {
      return false ;
    }
    if (!EqualsUtil.equals(this.angleGridlinePaint, that.angleGridlinePaint)) {
      return false ;
    }
    if (this.radiusGridlinesVisible != that.radiusGridlinesVisible) {
      return false ;
    }
    if (!EqualsUtil.equals(this.radiusGridlineStroke, that.radiusGridlineStroke)) {
      return false ;
    }
    if (!EqualsUtil.equals(this.radiusGridlinePaint, that.radiusGridlinePaint)) {
      return false ;
    }

    return true ;
  }

  /**
   * <zh>hashcodeֵ.</zh>
   * <en>Returns a hash code value for the object.</en>
   *
   * @return int <zh>hashcodeֵ.</zh>
   * <en>A hash code value for this object.</en>
   */
  public int hashCode() {
    double result = super.hashCode() ;
    result = 37 * result + margin ;
    result = 37 * result + HashcodeUtil.hashCode(this.angleLabelsVisible) ;
    result = 37 * result + HashcodeUtil.hashCode(this.angleLabelFont) ;
    result = 37 * result + HashcodeUtil.hashCode(this.angleLabelPaint) ;
    result = 37 * result + HashcodeUtil.hashCode(this.angleGridlinesVisible) ;
    result = 37 * result + HashcodeUtil.hashCode(this.angleGridlineStroke) ;
    result = 37 * result + HashcodeUtil.hashCode(this.angleGridlinePaint) ;
    result = 37 * result + HashcodeUtil.hashCode(this.radiusGridlinesVisible) ;
    result = 37 * result + HashcodeUtil.hashCode(this.radiusGridlineStroke) ;
    result = 37 * result + HashcodeUtil.hashCode(this.radiusGridlinePaint) ;

    return (int) result ;
  }

  /**
   * <zh>ؿ¡Ķ.</zh>
   * <en>Creates and returns a copy of this object.</en>
   * @return object   a clone of this instance.
   * @exception  CloneNotSupportedException  if the object's class does not
   *               support the Cloneable interface. Subclasses
   *               that override the clone method can also
   *               throw this exception to indicate that an instance cannot
   *               be cloned.
   */
  public Object clone() throws CloneNotSupportedException {
    return super.clone() ;
  }

  /**
   * <zh>toString.</zh>
   * <en>Returns a string representation of the object.</en>
   *
   * @return  a string representation of the object.
   */
  public String toString() {
    return "PolarRender" ;
  }

  /**
   * Provides serialization support.
   *
   * @param stream  the output stream.
   *
   * @throws IOException  if there is an I/O error.
   */
  private void writeObject(ObjectOutputStream stream) throws IOException {
    stream.defaultWriteObject() ;

    SerialUtil.writePaint(stream, this.angleLabelPaint) ;
    SerialUtil.writePaint(stream, this.angleGridlinePaint) ;
    SerialUtil.writePaint(stream, this.radiusGridlinePaint) ;

    SerialUtil.writeStroke(stream, this.angleGridlineStroke) ;
    SerialUtil.writeStroke(stream, this.radiusGridlineStroke) ;
  }

  /**
   * Provides serialization support.
   *
   * @param stream  the input stream.
   *
   * @throws IOException  if there is an I/O error.
   * @throws ClassNotFoundException  if there is a classpath problem.
   */
  private void readObject(ObjectInputStream stream) throws IOException, ClassNotFoundException {
    stream.defaultReadObject() ;

    this.angleLabelPaint = SerialUtil.readPaint(stream) ;
    this.angleGridlinePaint = SerialUtil.readPaint(stream) ;
    this.radiusGridlinePaint = SerialUtil.readPaint(stream) ;

    this.angleGridlineStroke = SerialUtil.readStroke(stream) ;
    this.radiusGridlineStroke = SerialUtil.readStroke(stream) ;
  }

}
