package com.ijchart.xychart.render.paint ;

import java.awt.Color ;
import java.awt.Paint ;
import java.io.Serializable ;

/**
 * <p>Title: IJChart</p>
 *
 * <p>Description: a chart library for the Java(tm) platform.</p>
 * <zh>Ҷݶ,ⲿǲ޸ĵ.</zh>
 * <en>A paint scale that returns shades of gray,this class is immutable.</en>
 * <p>Copyright: Copyright (c) 2013</p>
 *
 * <p>Company: </p>
 *
 * @author zhang feng min,shanghai china.
 * <a href="mailto:ijchartsh@gmail.com">Contract me by email.</a>
 * @version 1.0
 */
public class GrayPaintScale implements IPaintScale, Serializable, Cloneable {
  /**
   * <zh>л汾ID.</zh>
   * <en>For serialization.</en>
   */
  private static final long serialVersionUID = 9184291040145384229L ;

  /**
   * <zh>Сֵ߽.</zh>
   * <en>The lower bound.</en>
   */
  private double lowerBound ;

  /**
   * <zh>ֵ߽.</zh>
   * <en>The upper bound.</en>
   */
  private double upperBound ;

  /**
   * <zh>ȱʡĹ.</zh>
   * <en>The default constructor.</en>
   */
  public GrayPaintScale() {
    this(DEFAULT_LOWER_BOUND, DEFAULT_UPPER_BOUND) ;
  }

  /**
   * <zh>.</zh>
   * <en>The constructor.</en>
   *
   * @param lowerBound <zh>Сֵ߽.</zh>
   * <en>The lower bound.</en>
   * @param upperBound <zh>ֵ߽.</zh>
   * <en>The upper bound.</en>
   */
  public GrayPaintScale(double lowerBound, double upperBound) {
    if (lowerBound >= upperBound) {
      throw new IllegalArgumentException("Must lowerBound < upperBound.") ;
    }

    this.lowerBound = lowerBound ;
    this.upperBound = upperBound ;
  }

  /**
   * <zh>Сֵ߽.</zh>
   * <en>The lower bound.</en>
   *
   * @return double <zh>Сֵ߽.</zh>
   * <en>The lower bound.</en>
   */
  public double getLowerBound() {
    return this.lowerBound ;
  }

  /**
   * <zh>ֵ߽.</zh>
   * <en>The upper bound.</en>
   *
   * @return double <zh>ֵ߽.</zh>
   * <en>The upper bound.</en>
   */
  public double getUpperBound() {
    return this.upperBound ;
  }

  /**
   * <zh>ĳֵӦpaint.</zh>
   * <en>Returns a Paint instance for the specified value.</en>
   *
   * @param value double <zh>ֵ.</zh>
   * <en>The value.</en>
   * @return Paint <zh>paint (Ϊnull).</zh>
   * <en>The paint object (never null).</en>
   */
  public Paint getPaint(double value) {
    double v = Math.max(value, this.lowerBound) ;
    v = Math.min(v, this.upperBound) ;
    int g = (int) ((v - this.lowerBound) / (this.upperBound - this.lowerBound) * 255.0) ;
    return new Color(g, g, g) ;

  }

  /**
   * <zh>Ƿ.</zh>
   * <en>Tests this instance for equality with an arbitrary object.</en>
   *
   * @param obj  the object (null permitted).
   *
   * @return A boolean.
   */
  public boolean equals(Object obj) {
    if (obj == this) {
      return true ;
    }
    if (!(obj instanceof GrayPaintScale)) {
      return false ;
    }
    if (!super.equals(obj)) {
      return false ;
    }

    GrayPaintScale that = (GrayPaintScale) obj ;
    if (this.lowerBound != that.lowerBound) {
      return false ;
    }
    if (this.upperBound != that.upperBound) {
      return false ;
    }

    return true ;
  }

  /**
   * <zh>hashcodeֵ.</zh>
   * <en>Returns a hash code value for the object.</en>
   *
   * @return int <zh>hashcodeֵ.</zh>
   * <en>A hash code value for this object.</en>
   */
  public int hashCode() {
    double result = super.hashCode() ;
    result = 37 * result + this.lowerBound ;
    result = 37 * result + this.upperBound ;

    return (int) result ;
  }

  /**
   * <zh>ؿ¡Ķ.</zh>
   * <en>Creates and returns a copy of this object.</en>
   * @return object   a clone of this instance.
   * @exception  CloneNotSupportedException  if the object's class does not
   *               support the Cloneable interface. Subclasses
   *               that override the clone method can also
   *               throw this exception to indicate that an instance cannot
   *               be cloned.
   */
  public Object clone() throws CloneNotSupportedException {
    return super.clone() ;
  }

  /**
   * <zh>toString.</zh>
   * <en>Returns a string representation of the object.</en>
   *
   * @return  a string representation of the object.
   */
  public String toString() {
    return "[lowerBound=" + this.lowerBound + ",upperBound=" + this.upperBound + "]" ;
  }

}
