package com.ijchart.xychart.render.xy ;

import java.awt.geom.Ellipse2D ;
import java.awt.geom.GeneralPath ;
import java.awt.geom.Rectangle2D ;
import java.io.Serializable ;
import java.util.ArrayList ;
import java.util.Collections ;
import java.util.List ;

import com.ijchart.xychart.constant.LegendType ;
import com.ijchart.xychart.constant.SortType ;
import com.ijchart.xychart.data.IChartData ;
import com.ijchart.xychart.data.IXYChartData ;
import com.ijchart.xychart.dataset.IMatrixDataset ;
import com.ijchart.xychart.exception.RowXValueDuplicateException ;
import com.ijchart.xychart.shape.GeneralPathChartShape ;
import com.ijchart.xychart.shape.IChartShape ;

/**
 * <p>Title: IJChart</p>
 *
 * <p>Description: a chart library for the Java(tm) platform.</p>
 * <zh>XYͼĳ.</zh>
 * <en>An abstract class for draw xy area shapes.</en>
 *
 * <p>Copyright: Copyright (c) 2013</p>
 *
 * <p>Company: </p>
 *
 * @author zhang feng min,shanghai china.
 * <a href="mailto:ijchartsh@gmail.com">Contract me by email.</a>
 * @version 1.0
 */
public abstract class AbstractXYAreaRender extends AbstractXYRender implements Cloneable,
  Serializable {
  /**
   * <zh>л汾ID.</zh>
   * <en>For serialization.</en>
   */
  private static final long serialVersionUID = -851274132171292876L ;

  /**
   * <zh>ݼݵxֵ.</zh>
   * <en>The unique x values of data in dataset.</en>
   */
  protected List uniqueXValues ;

  /**
   * <zh>.</zh>
   * <en>constructor.</en>
   *
   * @param dataset IMatrixDataset <zh>ݼ (Ϊnull).</zh>
   * <en>The dataset (null not permitted).</en>
   */
  public AbstractXYAreaRender(IMatrixDataset dataset) {
    super(dataset) ;
  }

  /**
   * <zh>ֻͼ,ͼε.</zh>
   * <en>Create a shape with no attribute specified.</en>
   *
   * @param rect Rectangle2D <zh>ͼľ (Ϊnull).</zh>
   * <en>The bar rectangle (null not permitted).</en>
   * @param text String <zh>Ҫʾ (Ϊnull).</zh>
   * <en>The text (null permitted).</en>
   * @param appendParam Object <zh>ӵĲ,ڴݶĶ (Ϊnull).</zh>
   * <en>The append parameter,used for special propose (null permitted).</en>
   *
   * @return IChartShape <zh>ͼ.</zh>
   * <en>The shape.</en>
   */
  protected IChartShape createItemShapeOnly(Rectangle2D rect, String text,
                                            Object appendParam) {
    if (appendParam == null) {
      throw new IllegalArgumentException("Null 'appendParam' argument.") ;
    }

    Object[] params = (Object[]) appendParam ;
    GeneralPath path = (GeneralPath) params[0] ;
    double[] tooltipPoint = (double[]) params[1] ;

    GeneralPathChartShape shape = new GeneralPathChartShape(path) ;
    shape.setText(text) ;

    final double tooltipShapeWidth = IChartShape.DEFAULT_TOOLTIP_SHAPE_WIDTH ;
    final double halfWidth = tooltipShapeWidth * 0.5 ;
    Ellipse2D circle = new Ellipse2D.Double(tooltipPoint[0] - halfWidth,
                                            tooltipPoint[1] - halfWidth,
                                            tooltipShapeWidth,
                                            tooltipShapeWidth) ;
    shape.setOriginTooltipShape(circle) ;

    return shape ;
  }

  /**
   * <zh>ݼ,µķϸʽݼ.</zh>
   * <en>Process the dataset, returns the new format of the dataset.</en>
   *
   * @return Object[]
   */
  protected Object[] processDataset() {
    calculateUniqueXValues() ;

    IMatrixDataset dataset = getDataset() ;
    int rows = dataset.getRowCount() ;

    int xValuesCount = this.uniqueXValues.size() ;
    Object[] dataRows = new Object[rows] ;
    for (int i = 0 ; i < rows ; i++) {
      double[][] rowNumberData = new double[xValuesCount][3] ; //һеֵ
      dataRows[i] = rowNumberData ;
      for (int j = 0 ; j < xValuesCount ; j++) {
        rowNumberData[j][0] = ((Number)this.uniqueXValues.get(j)).doubleValue() ; //еxֵ
        rowNumberData[j][1] = Double.NaN ;
      }

      int currRowCols = dataset.getColumnCount(i) ;
      for (int j = 0 ; j < currRowCols ; j++) {
        IXYChartData object = (IXYChartData) dataset.getData(i, j) ;
        if (object != null) {
          double x = object.getX() ;
          if (!Double.isNaN(x)) {
            int index = Collections.binarySearch(this.uniqueXValues, new Double(x)) ;
            if (index >= 0) {
              rowNumberData[index][1] = object.getY() ;
              rowNumberData[index][2] = j ; //¼ֵ
            }
          }
        }
      }
    }

    return dataRows ;
  }

  /**
   * <zh>ݼݵxֵ,еxӦΨһ.</zh>
   * <en>Calculate the unique x values.</en>
   */
  protected void calculateUniqueXValues() {
    this.uniqueXValues = new ArrayList() ; //xֵ

    IMatrixDataset dataset = getDataset() ;
    int rows = dataset.getRowCount() ;
    for (int i = 0 ; i < rows ; i++) {
      int currRowCols = dataset.getColumnCount(i) ;
      for (int j = 0 ; j < currRowCols ; j++) {
        IChartData data = dataset.getData(i, j) ;
        if (data != null && data instanceof IXYChartData) {
          IXYChartData chartData = (IXYChartData) data ;
          double x = chartData.getX() ;
          Double xData = new Double(x) ;
          if (!this.uniqueXValues.contains(xData) && !Double.isNaN(x)) {
            this.uniqueXValues.add(xData) ;
          }
        }
      }
    }

    Collections.sort(this.uniqueXValues) ;
  }

  /**
   * <zh>ݼеÿһǷظ.</zh>
   * <en>Check whether or not the x values are duplicate of a series.</en>
   */
  protected void checkRowDataDuplicate() {
    IMatrixDataset dataset = getDataset() ;
    int rowCount = dataset.getRowCount() ;

    for (int i = 0 ; i < rowCount ; i++) {
      dataset.sortRowData(i, SortType.ASCENDING) ;
      double prevValue = Double.NaN ;
      int currRowCols = dataset.getColumnCount(i) ;
      for (int j = 0 ; j < currRowCols ; j++) {
        IXYChartData data = (IXYChartData) dataset.getData(i, j) ;
        if (data != null) {
          double currValue = data.getX() ;
          if (!Double.isNaN(prevValue) && !Double.isNaN(currValue)) {
            if (prevValue == currValue) {
              String msg = "Dataset row " + i + " column " + j + " x value duplicated." ;
              throw new RowXValueDuplicateException(msg) ;
            }
          }
          prevValue = currValue ;
        }
      }
    }
  }

  /**
   * <zh>ͼ.</zh>
   * <en>Return the legend type.</en>
   *
   * @return LegendType <zh>ͼ.</zh>
   * <en>The legend type.</en>
   */
  public LegendType getLegendType() {
    return LegendType.ROW ;
  }

  /**
   * <zh>Ƿ.</zh>
   * <en>Indicates if some other object is "equal to" this one.</en>
   *
   * @param obj <zh>ҪȽϵĶ</zh>
   * <en>the reference object with which to compare.</en>
   * @return <zh>ȷtrue,򷵻false.</zh>
   * <en>true if this object is the same as the obj argument; false otherwise.</en>
   */
  public boolean equals(Object obj) {
    if (this == obj) {
      return true ;
    }
    if (!(obj instanceof AbstractXYAreaRender)) {
      return false ;
    }
    if (!super.equals(obj)) {
      return false ;
    }

    return true ;
  }

  /**
   * <zh>hashcodeֵ.</zh>
   * <en>Returns a hash code value for the object.</en>
   *
   * @return int <zh>hashcodeֵ.</zh>
   * <en>A hash code value for this object.</en>
   */
  public int hashCode() {
    return super.hashCode() ;
  }

  /**
   * <zh>ؿ¡Ķ.</zh>
   * <en>Creates and returns a copy of this object.</en>
   * @return object   a clone of this instance.
   * @exception  CloneNotSupportedException  if the object's class does not
   *               support the Cloneable interface. Subclasses
   *               that override the clone method can also
   *               throw this exception to indicate that an instance cannot
   *               be cloned.
   */
  public Object clone() throws CloneNotSupportedException {
    return super.clone() ;
  }

  /**
   * <zh>toString.</zh>
   * <en>Returns a string representation of the object.</en>
   *
   * @return  a string representation of the object.
   */
  public String toString() {
    return "AbstractXYAreaRender" ;
  }

}
