package com.ijchart.xychart.render.xy ;

import java.awt.geom.Rectangle2D ;
import java.io.Serializable ;

import com.ijchart.xychart.axis.value.AbstractValueAxis ;
import com.ijchart.xychart.chartinfo.RenderShapeCollection ;
import com.ijchart.xychart.constant.ChartOrientation ;
import com.ijchart.xychart.constant.LegendType ;
import com.ijchart.xychart.data.IChartData ;
import com.ijchart.xychart.data.IntervalXYNumberChartData ;
import com.ijchart.xychart.dataset.IMatrixDataset ;
import com.ijchart.xychart.shape.IChartShape ;
import com.ijchart.xychart.ui.RectangleEdge ;

/**
 * <p>Title: IJChart</p>
 *
 * <p>Description: a chart library for the Java(tm) platform.</p>
 * <zh>XY״ͼͼ.</zh>
 * <en>Draw 2D xy bar shapes.</en>
 *
 * <p>Copyright: Copyright (c) 2013</p>
 *
 * <p>Company: </p>
 *
 * @author zhang feng min,shanghai china.
 * <a href="mailto:ijchartsh@gmail.com">Contract me by email.</a>
 * @version 1.0
 */
public class XYIntervalBar2DRender extends AbstractXYRender implements Cloneable,
  Serializable {
  /**
   * <zh>л汾ID.</zh>
   * <en>For serialization.</en>
   */
  private static final long serialVersionUID = 3137611200332620865L ;

  /**
   * <zh>.</zh>
   * <en>constructor.</en>
   *
   * @param dataset IMatrixDataset <zh>ݼ (Ϊnull).</zh>
   * <en>The dataset (null not permitted).</en>
   */
  public XYIntervalBar2DRender(IMatrixDataset dataset) {
    super(dataset) ;
  }

  /**
   * <zh>еͼ.</zh>
   * <en>Draws all data items.</en>
   *
   * @param xAxis AbstractValueAxis <zh>xֵ (Ϊnull).</zh>
   * <en>The x value axis (null not permitted).</en>
   * @param xAxisEdge RectangleEdge <zh>xλ(Ϊnull).</zh>
   * <en>The x axis edge.</en>
   * @param yAxis AbstractValueAxis <zh>yֵ (Ϊnull).</zh>
   * <en>The y value axis (null not permitted).</en>
   * @param yAxisEdge RectangleEdge <zh>yλ (Ϊnull).</zh>
   * <en>The y axis edge (null not permitted).</en>
   * @param dataArea Rectangle2D <zh> (Ϊnull).</zh>
   * <en>The chart data area (null not permitted).</en>
   * @param orientation ChartOrientation <zh>ͼķ (Ϊnull).</zh>
   * <en>The chart orientation (null not permitted).</en>
   * @param renderShapes RenderShapeCollection <zh>renderͼμ (Ϊnull).</zh>
   * <en>Collects information about the render (null not permitted).</en>
   * @param alpha float <zh>alphaֵ,ڱ[0.0,1.0].</zh>
   * <en>The alpha transparency (in the range 0.0f to 1.0f, where 0.0f is fully transparent,
   * and 1.0f is fully opaque).</en>
   */
  public void drawAllItems(AbstractValueAxis xAxis, RectangleEdge xAxisEdge,
                           AbstractValueAxis yAxis, RectangleEdge yAxisEdge,
                           Rectangle2D dataArea, ChartOrientation orientation,
                           RenderShapeCollection renderShapes, float alpha) {
    if (xAxis == null) {
      throw new IllegalArgumentException("Null 'xAxis' argument.") ;
    }
    if (xAxisEdge == null) {
      throw new IllegalArgumentException("Null 'xAxisEdge' argument.") ;
    }
    if (yAxis == null) {
      throw new IllegalArgumentException("Null 'yAxis' argument.") ;
    }
    if (yAxisEdge == null) {
      throw new IllegalArgumentException("Null 'yAxisEdge' argument.") ;
    }
    if (dataArea == null) {
      throw new IllegalArgumentException("Null 'dataArea' argument.") ;
    }
    if (orientation == null) {
      throw new IllegalArgumentException("Null 'orientation' argument.") ;
    }
    if (renderShapes == null) {
      throw new IllegalArgumentException("Null 'renderShapes' argument.") ;
    }
    if (alpha < 0.0f || alpha > 1.0f) {
      throw new IllegalArgumentException("The 'alpha' value must be in the range 0.0f to 1.0f.") ;
    }

    IMatrixDataset dataset = getDataset() ;
    if (dataset.isEmpty()) {
      return ;
    }

    int rows = dataset.getRowCount() ;

    this.fillPaints = getFillPaints(rows) ;
    this.borderPaints = getBorderPaints(rows) ;
    this.borderStrokes = getBorderStrokes(rows) ;

    //ߺͼ
    if (orientation.equals(ChartOrientation.VERTICAL)) { //ֱ
      for (int i = 0 ; i < rows ; i++) {
        int currRowCols = dataset.getColumnCount(i) ;

        for (int j = 0 ; j < currRowCols ; j++) {
          IChartData data = dataset.getData(i, j) ;
          if (data != null && data instanceof IntervalXYNumberChartData) {
            IntervalXYNumberChartData xyData = (IntervalXYNumberChartData) data ;
            double startXValue = xyData.getStartXValue() ;
            double endXValue = xyData.getEndXValue() ;
            double startYValue = xyData.getStartYValue() ;
            double endYValue = xyData.getEndYValue() ;

            double startX = xAxis.valueToJava2D(dataArea, xAxisEdge, startXValue, false) ;
            double endX = xAxis.valueToJava2D(dataArea, xAxisEdge, endXValue, false) ;
            double startY = yAxis.valueToJava2D(dataArea, yAxisEdge, startYValue, false) ;
            double endY = yAxis.valueToJava2D(dataArea, yAxisEdge, endYValue, false) ;

            //ֵ᷶Χֵ
            if (startX == AbstractValueAxis.INVALID_COORDINATE_VALUE
                || endX == AbstractValueAxis.INVALID_COORDINATE_VALUE
                || startY == AbstractValueAxis.INVALID_COORDINATE_VALUE
                || endY == AbstractValueAxis.INVALID_COORDINATE_VALUE) {
              renderShapes.addDataItemShape(null) ;
              continue ;
            }

            Rectangle2D rect = new Rectangle2D.Double(Math.min(startX, endX),
                                                      Math.min(startY, endY),
                                                      Math.abs(endX - startX),
                                                      Math.abs(endY - startY)) ;
            IChartShape shape = createItemShape(orientation, yAxis, i, j, rect, fillPaints[i],
                                                borderPaints[i], borderStrokes[i], alpha, null) ;
            renderShapes.addDataItemShape(shape) ;
          } else {
            renderShapes.addDataItemShape(null) ;
          }
        }
      }
    } else if (orientation.equals(ChartOrientation.HORIZONTAL)) { //ˮƽ
      for (int i = 0 ; i < rows ; i++) {

        int currRowCols = dataset.getColumnCount(i) ;
        for (int j = 0 ; j < currRowCols ; j++) {
          IChartData data = dataset.getData(i, j) ;
          if (data != null && data instanceof IntervalXYNumberChartData) {
            IntervalXYNumberChartData xyData = (IntervalXYNumberChartData) data ;
            double startXValue = xyData.getStartXValue() ;
            double endXValue = xyData.getEndXValue() ;
            double startYValue = xyData.getStartYValue() ;
            double endYValue = xyData.getEndYValue() ;

            double startX = xAxis.valueToJava2D(dataArea, xAxisEdge, startXValue, false) ;
            double endX = xAxis.valueToJava2D(dataArea, xAxisEdge, endXValue, false) ;
            double startY = yAxis.valueToJava2D(dataArea, yAxisEdge, startYValue, false) ;
            double endY = yAxis.valueToJava2D(dataArea, yAxisEdge, endYValue, false) ;

            //ֵ᷶Χֵ
            if (startX == AbstractValueAxis.INVALID_COORDINATE_VALUE
                || endX == AbstractValueAxis.INVALID_COORDINATE_VALUE
                || startY == AbstractValueAxis.INVALID_COORDINATE_VALUE
                || endY == AbstractValueAxis.INVALID_COORDINATE_VALUE) {
              renderShapes.addDataItemShape(null) ;
              continue ;
            }

            Rectangle2D rect = new Rectangle2D.Double(Math.min(startY, endY),
                                                      Math.min(startX, endX),
                                                      Math.abs(endY - startY),
                                                      Math.abs(endX - startX)) ;
            IChartShape shape = createItemShape(orientation, xAxis, i, j, rect, fillPaints[i],
                                                borderPaints[i], borderStrokes[i], alpha, null) ;
            renderShapes.addDataItemShape(shape) ;
          } else {

            renderShapes.addDataItemShape(null) ;
          }
        }
      }
    }
  }

  /**
   * <zh>ͼ.</zh>
   * <en>Return the legend type.</en>
   *
   * @return LegendType <zh>ͼ.</zh>
   * <en>The legend type.</en>
   */
  public LegendType getLegendType() {
    return LegendType.ROW ;
  }

  /**
   * <zh>Ƿ.</zh>
   * <en>Indicates if some other object is "equal to" this one.</en>
   *
   * @param obj <zh>ҪȽϵĶ</zh>
   * <en>the reference object with which to compare.</en>
   * @return <zh>ȷtrue,򷵻false.</zh>
   * <en>true if this object is the same as the obj argument; false otherwise.</en>
   */
  public boolean equals(Object obj) {
    if (this == obj) {
      return true ;
    }
    if (!(obj instanceof XYIntervalBar2DRender)) {
      return false ;
    }
    if (!super.equals(obj)) {
      return false ;
    }

    return true ;
  }

  /**
   * <zh>hashcodeֵ.</zh>
   * <en>Returns a hash code value for the object.</en>
   *
   * @return int <zh>hashcodeֵ.</zh>
   * <en>A hash code value for this object.</en>
   */
  public int hashCode() {
    int result = super.hashCode() ;

    return result ;
  }

  /**
   * <zh>ؿ¡Ķ.</zh>
   * <en>Creates and returns a copy of this object.</en>
   * @return object   a clone of this instance.
   * @exception  CloneNotSupportedException  if the object's class does not
   *               support the Cloneable interface. Subclasses
   *               that override the clone method can also
   *               throw this exception to indicate that an instance cannot
   *               be cloned.
   */
  public Object clone() throws CloneNotSupportedException {
    return super.clone() ;
  }

  /**
   * <zh>toString.</zh>
   * <en>Returns a string representation of the object.</en>
   *
   * @return  a string representation of the object.
   */
  public String toString() {
    return "XYIntervalBar2DRender" ;
  }

}
