package com.ijchart.xychart.shape ;

import java.awt.Image ;
import java.io.Serializable ;

import com.ijchart.xychart.constant.HorizontalAlignment ;
import com.ijchart.xychart.constant.VerticalAlignment ;
import com.ijchart.xychart.util.EqualsUtil ;
import com.ijchart.xychart.util.HashcodeUtil ;

/**
 * <p>Title: IJChart</p>
 *
 * <p>Description: a chart library for the Java(tm) platform.</p>
 * <zh>ͼڰͼƬĳʵ,Զͼεˮƽֱ뷽ʽ,ˮƽֱƽ,Զͼε.</zh>
 * <en>Abstract class for draw a {@link com.ijchart.xychart.shape.IChartShape} with an image in it.
 * You can define the vertical alignment,horizontal alignment,x-repeat,y-repeat,auto fit features.</en>
 *
 * <p>Copyright: Copyright (c) 2013</p>
 *
 * <p>Company: </p>
 *
 * @author zhang feng min,shanghai china.
 * <a href="mailto:ijchartsh@gmail.com">Contract me by email.</a>
 * @version 1.0
 */
public abstract class AbstractImageChartShape extends AbstractChartShape implements Serializable,
  Cloneable {
  /**
   * <zh>л汾ID.</zh>
   * <en>For serialization.</en>
   */
  private static final long serialVersionUID = -5769374486037226525L ;

  /**
   * <zh>ȱʡͼƬˮƽ뷽ʽ.</zh>
   * <en>The default image horizontal alignment.</en>
   */
  public static final HorizontalAlignment DEFAULT_IMAGE_HORIZONTAL_ALIGNMENT =
    HorizontalAlignment.CENTER ;

  /**
   * <zh>ȱʡͼƬֱ뷽ʽ.</zh>
   * <en>The default image vertical alignment.</en>
   */
  public static final VerticalAlignment DEFAULT_IMAGE_VERTICAL_ALIGNMENT =
    VerticalAlignment.CENTER ;

  /**
   * <zh>ȱʡͼƬalphaֵ.</zh>
   * <en>The default alpha for draw an image.</en>
   */
  public static final float DEFAULT_IMAGE_ALPHA = 1.0F ;

  /**
   * <zh>ͼƬˮƽ뷽ʽ.</zh>
   * <en>The image horizontal align.</en>
   */
  private HorizontalAlignment imageHorizontalAlign ;

  /**
   * <zh>ͼƬֱ뷽ʽ.</zh>
   * <en>The image vertical align.</en>
   */
  private VerticalAlignment imageVerticalAlign ;

  /**
   * <zh>ͼƬˮƽǷظ,ȱʡΪfalse.</zh>
   * <en>A boolean flag controls whether or not the image is horizontal repeat,
   * default is false.</en>
   */
  private boolean horizontalRepeat ;

  /**
   * <zh>ͼƬڴֱǷظ,ȱʡΪfalse.</zh>
   * <en>A boolean flag controls whether or not thee image is vertical repeat,default is false.</en>
   */
  private boolean verticalRepeat ;

  /**
   * <zh>ǷͨŴСͼƬͼ,ȱʡΪfalse.</zh>
   * <en>A boolean flag controls whether or not the image is zoom is or zoom out to
   * fit the {@link com.ijchart.xychart.shape.IChartShape},default is false.</en>
   */
  private boolean fillShape ;

  /**
   * <zh>ҪƵͼƬ.</zh>
   * <en>The image.</en>
   */
  private transient Image image ;

  /**
   * <zh>ͼƬalphaֵ.</zh>
   * <en>The alpha value for draw an image.</en>
   */
  private float imageAlpha ;

  /**
   * <zh>һͼƬ{@link com.ijchart.xychart.shape.IChartShape}.</zh>
   * <en>Construct a {@link com.ijchart.xychart.shape.IChartShape} with an image argument.</en>
   *
   * @param image Image <zh>һͼƬ,ͼƬݱѾ (Ϊnull).</zh>
   * <en>A image,and the image data must has been loaded (null permitted).</en>
   */
  protected AbstractImageChartShape(Image image) {
    this.imageHorizontalAlign = DEFAULT_IMAGE_HORIZONTAL_ALIGNMENT ;
    this.imageVerticalAlign = DEFAULT_IMAGE_VERTICAL_ALIGNMENT ;
    this.imageAlpha = DEFAULT_IMAGE_ALPHA ;

    this.horizontalRepeat = false ;
    this.verticalRepeat = false ;
    this.fillShape = false ;
    this.image = image ;
  }

  /**
   * <zh>ͼƬˮƽ뷽ʽ.</zh>
   * <en>Sets the image horizontal alignment.</en>
   *
   * @param horizontalAlign HorizontalAlignment <zh>ͼƬˮƽ뷽ʽ (Ϊnull).</zh>
   * <en>The image horizontal alignment (null not permitted).</en>
   */
  public void setImageHorizontalAlign(HorizontalAlignment horizontalAlign) {
    if (horizontalAlign == null) {
      throw new IllegalArgumentException("Null 'horizontalAlign' argument.") ;
    }

    this.imageHorizontalAlign = horizontalAlign ;
    fireChartShapeChanged() ;
  }

  /**
   * <zh>ͼƬˮƽ뷽ʽ.</zh>
   * <en>Returns the image horizontal alignment.</en>
   *
   * @return HorizontalAlignment <zh>ͼƬˮƽ뷽ʽ.</zh>
   * <en>The image horizontal alignment.</en>
   */
  public HorizontalAlignment getImageHorizontalAlign() {
    return imageHorizontalAlign ;
  }

  /**
   * <zh>ͼƬֱ뷽ʽ.</zh>
   * <en>Sets the image vertical alignment.</en>
   *
   * @param verticalAlign VerticalAlignment <zh>ͼƬֱ뷽ʽ (Ϊnull).</zh>
   * <en>The image vertical alignment (null not permitted).</en>
   */
  public void setImageVerticalAlign(VerticalAlignment verticalAlign) {
    if (verticalAlign == null) {
      throw new IllegalArgumentException("Null 'verticalAlign' argument.") ;
    }

    this.imageVerticalAlign = verticalAlign ;
    fireChartShapeChanged() ;
  }

  /**
   * <zh>ͼƬĴֱ뷽ʽ.</zh>
   * <en>Returns the image vertical alignment.</en>
   *
   * @return VerticalAlignment <zh>ͼƬֱ뷽ʽ.</zh>
   * <en>The image vertical alignment.</en>
   */
  public VerticalAlignment getImageVerticalAlign() {
    return imageVerticalAlign ;
  }

  /**
   * <zh>þʾͼƬ.</zh>
   * <en>Sets the image in this rectangle (null permitted).</en>
   *
   * @param image Image <zh>ʾͼƬ (Ϊnull).</zh>
   * <en>The image (null permitted).</en>
   */
  public void setImage(Image image) {
    this.image = image ;
    fireChartShapeChanged() ;
  }

  /**
   * <zh>ھʾͼƬ.</zh>
   * <en>Returns the image in this rectangle.</en>
   *
   * @return Image
   */
  public Image getImage() {
    return image ;
  }

  /**
   * <zh>ͼƬˮƽǷظ.</zh>
   * <en>Sets whether or not the image is horizontal repeat.</en>
   *
   * @param horizontalRepeat boolean <zh>ͼƬˮƽǷظ.</zh>
   * <en>A boolean flag controls whether or not the image is x-repeat.</en>
   */
  public void setHorizontalRepeat(boolean horizontalRepeat) {
    if (this.horizontalRepeat != horizontalRepeat) {
      this.horizontalRepeat = horizontalRepeat ;
      fireChartShapeChanged() ;
    }
  }

  /**
   * <zh>ͼƬǷˮƽظ.</zh>
   * <en>Returns a booleaen flag controls whether or not the image is horizontal repeat.</en>
   *
   * @return boolean <zh>ˮƽظtrue,򷵻false.</zh>
   * <en>Returns true if horizontal repeat is true,otherwise return false.</en>
   */
  public boolean isHorizontalRepeat() {
    return this.horizontalRepeat ;
  }

  /**
   * <zh>ͼƬڴֱǷظ.</zh>
   * <en>Sets whether or not the image is vertical repeat.</en>
   *
   * @param verticalRepeat boolean <zh>ͼƬڴֱǷظ.</zh>
   * <en>A boolean flag controls whether or not the image is vertical repeat.</en>
   */
  public void setVerticalRepeat(boolean verticalRepeat) {
    if (this.verticalRepeat != verticalRepeat) {
      this.verticalRepeat = verticalRepeat ;
      fireChartShapeChanged() ;
    }
  }

  /**
   * <zh>ͼƬǷڴֱظ.</zh>
   * <en>Returns a booleaen flag controls whether or not the image is vertical repeat.</en>
   *
   * @return boolean <zh>ڴֱظtrue,򷵻false.</zh>
   * <en>Returns true if vertical repeat is true,otherwise return false.</en>
   */
  public boolean isVerticalRepeat() {
    return this.verticalRepeat ;
  }

  /**
   * <zh>ûͼƬalphaֵ,м{@link com.ijchart.xychart.event.ChartShapeChangeEvent}¼.</zh>
   * <en>The alpha value for draw an image,and sends a
   * {@link com.ijchart.xychart.event.ChartShapeChangeEvent} to all registered listeners.</en>
   *
   * @param alpha float <zh>alphaֵ,ڱ[0.0,1.0].</zh>
   * <en>The alpha transparency (in the range 0.0f to 1.0f, where 0.0f is fully transparent,
   * and 1.0f is fully opaque).</en>
   */
  public void setImageAlpha(float alpha) {
    if (alpha < 0.0f || alpha > 1.0f) {
      throw new IllegalArgumentException(
        "The 'alpha' value must be in the range 0.0f to 1.0f.") ;
    }

    if (this.imageAlpha != alpha) {
      this.imageAlpha = alpha ;
      fireChartShapeChanged() ;
    }
  }

  /**
   * <zh>ػƱͼƬalphaֵ.</zh>
   * <en>Returns the alpha value for draw an image.</en>
   * @return float <zh>alphaֵ.</zh>
   * <en>The alpha value.</en>
   */
  public float getImageAlpha() {
    return imageAlpha ;
  }

  /**
   * <zh>hashcodeֵ.</zh>
   * <en>Returns a hash code value for the object.</en>
   *
   * @return int <zh>hashcodeֵ.</zh>
   * <en>A hash code value for this object.</en>
   */
  public int hashCode() {
    int result = super.hashCode() ;
    result = 37 * result + HashcodeUtil.hashCode(this.imageHorizontalAlign) ;
    result = 37 * result + HashcodeUtil.hashCode(this.imageVerticalAlign) ;
    result = 37 * result + HashcodeUtil.hashCode(this.horizontalRepeat) ;
    result = 37 * result + HashcodeUtil.hashCode(this.verticalRepeat) ;
    result = 37 * result + HashcodeUtil.hashCode(this.fillShape) ;
    result = 37 * result + HashcodeUtil.hashCode(this.image) ;
    result = 37 * result + (int) (100 * this.imageAlpha) ;

    return result ;
  }

  /**
   * <zh>Ƿ.</zh>
   * <en>Indicates whether some other object is "equal to" this one.</en>
   *
   * @param obj <zh>ҪȽϵĶ</zh>
   * <en>the reference object with which to compare.</en>
   * @return <zh>ȷtrue,򷵻false.</zh>
   * <en>true if this object is the same as the obj argument;
   * false otherwise.</en>
   */
  public boolean equals(Object obj) {
    if (this == obj) {
      return true ;
    }
    if (!(obj instanceof AbstractTextChartShape)) {
      return false ;
    }
    if (!super.equals(obj)) {
      return false ;
    }

    AbstractImageChartShape that = (AbstractImageChartShape) obj ;
    if (!EqualsUtil.equals(this.imageHorizontalAlign, that.imageHorizontalAlign)) {
      return false ;
    }
    if (!EqualsUtil.equals(this.imageVerticalAlign, that.imageVerticalAlign)) {
      return false ;
    }
    if (this.horizontalRepeat != that.horizontalRepeat) {
      return false ;
    }
    if (this.verticalRepeat != that.verticalRepeat) {
      return false ;
    }
    if (this.fillShape != that.fillShape) {
      return false ;
    }
    if (this.imageAlpha != that.imageAlpha) {
      return false ;
    }

    return true ;
  }

  /**
   * <zh>ؿ¡Ķ.</zh>
   * <en>Creates and returns a copy of this object.</en>
   * @return object   a clone of this instance.
   * @exception  CloneNotSupportedException  if the object's class does not
   *               support the Cloneable interface. Subclasses
   *               that override the clone method can also
   *               throw this exception to indicate that an instance cannot
   *               be cloned.
   */
  public Object clone() throws CloneNotSupportedException {
    AbstractImageChartShape clone = (AbstractImageChartShape)super.clone() ;

    return clone ;
  }
}
