package com.ijchart.xychart.shape ;

import java.awt.Graphics2D ;
import java.awt.Paint ;
import java.awt.Shape ;
import java.awt.geom.AffineTransform ;
import java.awt.geom.GeneralPath ;
import java.awt.geom.Line2D ;
import java.awt.geom.Point2D ;
import java.awt.geom.Rectangle2D ;
import java.io.IOException ;
import java.io.ObjectInputStream ;
import java.io.ObjectOutputStream ;
import java.io.Serializable ;

import com.ijchart.xychart.constant.ShapeType ;
import com.ijchart.xychart.ui.Graphics2DState ;
import com.ijchart.xychart.util.EqualsUtil ;
import com.ijchart.xychart.util.HashcodeUtil ;
import com.ijchart.xychart.util.SerialUtil ;
import com.ijchart.xychart.util.StrokeUtil ;

/**
 * <p>Title: IJChart</p>
 *
 * <p>Description: a chart library for the Java(tm) platform.</p>
 * <zh>Ƹߵ(high-low)ͼ.</zh>
 * <en>Draw a high-low {@link com.ijchart.xychart.shape.IChartShape}.</en>
 *
 * <p>Copyright: Copyright (c) 2013</p>
 *
 * <p>Company: </p>
 *
 * @author zhang feng min,shanghai china.
 * <a href="mailto:ijchartsh@gmail.com">Contract me by email.</a>
 * @version 1.0
 */
public class HighLowChartShape extends AbstractRectangleTextChartShape implements
  Serializable, Cloneable {
  /**
   * <zh>л汾ID.</zh>
   * <en>For serialization.</en>
   */
  private static final long serialVersionUID = 1669527055392250812L ;

  /**
   * <zh>high-lowֵӦֱ.</zh>
   * <en>The high-low line.</en>
   */
  private transient Line2D highLowLine ;

  /**
   * <zh>high-lowֱߵpaint.</zh>
   * <en>The paint used to fill high-low rectangle border.</en>
   */
  private transient Paint highLowLinePaint ;

  /**
   * <zh>openֵӦֱ.</zh>
   * <en>The open line.</en>
   */
  private transient Line2D openLine ;

  /**
   * <zh>openֱߵpaint.</zh>
   * <en>The paint used to draw open line.</en>
   */
  private transient Paint openLinePaint ;

  /**
   * <zh>closeֵӦֱ.</zh>
   * <en>The close line.</en>
   */
  private transient Line2D closeLine ;

  /**
   * <zh>closeֱߵpaint.</zh>
   * <en>The paint used to draw close line.</en>
   */
  private transient Paint closeLinePaint ;

  /**
   * <zh>drawAreaʾtext.</zh>
   * <en>Draw text in area drawArea.</en>
   *
   * @param drawArea Rectangle2D <zh> (Ϊnull).</zh>
   * <en>The draw area (null not permitted).</en>
   * @param text String <zh>Ҫʾ (Ϊnull).</zh>
   * <en>The text (null permitted).</en>
   * @param highLowLine Line2D <zh>high-lowֵӦֱ (Ϊnull).</zh>
   * <en>The high-low line (null permitted).</en>
   * @param highLowLinePaint Paint <zh>high-lowֱߵpaint (Ϊnull).</zh>
   * <en>The paint used to fill high-low rectangle border (null permitted).</en>
   * @param openLine Line2D <zh>openֵӦֱ (Ϊnull).</zh>
   * <en>The open line (null permitted).</en>
   * @param openLinePaint Paint <zh>openֱߵpaint (Ϊnull).</zh>
   * <en>The paint used to draw open line (null permitted).</en>
   * @param closeLine Line2D <zh>closeֵӦֱ (Ϊnull).</zh>
   * <en>The close line (null permitted).</en>
   * @param closeLinePaint Paint <zh>closeֱߵpaint (Ϊnull).</zh>
   * <en>The paint used to draw close line (null permitted).</en>
   */
  public HighLowChartShape(Rectangle2D drawArea, String text, Line2D highLowLine,
                           Paint highLowLinePaint, Line2D openLine, Paint openLinePaint,
                           Line2D closeLine, Paint closeLinePaint) {
    super(drawArea, text) ;

    this.highLowLine = highLowLine ;
    this.highLowLinePaint = highLowLinePaint ;
    this.openLine = openLine ;
    this.openLinePaint = openLinePaint ;
    this.closeLine = closeLine ;
    this.closeLinePaint = closeLinePaint ;
  }

  /**
   * <zh>high-lowֵӦֱ.</zh>
   * <en>Returns the high-low line.</en>
   *
   * @return Line2D <zh>ֱ߶ (Ϊnull).</zh>
   * <en>The line (maybe null).</en>
   */
  public Line2D getHighLowLine() {
    return this.highLowLine ;
  }

  /**
   * <zh>high-lowֵӦֱ,м
   * {@link com.ijchart.xychart.event.ChartShapeChangeEvent}¼.</zh>
   * <en>Sets the high-low line,and sends a
   * {@link com.ijchart.xychart.event.ChartShapeChangeEvent} to all registered listeners.</en>
   *
   * @param line Line2D <zh>ֱ߶ (Ϊnull).</zh>
   * <en>The line (null permitted).</en>
   */
  public void setHighLowLine(Line2D line) {
    if (!(this.highLowLine == null || line == null)) {
      this.highLowLine = line ;
      fireChartShapeChanged() ;
    }
  }

  /**
   * <zh>ػhigh-lowֱߵpaint.</zh>
   * <en>Returns the paint used to draw high-low line.</en>
   *
   * @return Paint <zh>Paint (Ϊnull).</zh>
   * <en>The paint (maybe null).</en>
   */
  public Paint getHighLowLinePaint() {
    return this.highLowLinePaint ;
  }

  /**
   * <zh>ûhigh-lowֱߵpaint,м
   * {@link com.ijchart.xychart.event.ChartShapeChangeEvent}¼.</zh>
   * <en>Sets the paint used to draw high-low line,and sends a
   * {@link com.ijchart.xychart.event.ChartShapeChangeEvent} to all registered listeners.</en>
   *
   * @param paint Paint <zh>Paint (Ϊnull).</zh>
   * <en>The paint (null permitted).</en>
   */
  public void setHighLowLinePaint(Paint paint) {
    if (!(this.highLowLinePaint == null || paint == null)) {
      this.highLowLinePaint = paint ;
      fireChartShapeChanged() ;
    }
  }

  /**
   * <zh>openֵӦֱ.</zh>
   * <en>Returns the open line.</en>
   *
   * @return Line2D <zh>ֱ߶ (Ϊnull).</zh>
   * <en>The line (maybe null).</en>
   */
  public Line2D getOpenLine() {
    return this.openLine ;
  }

  /**
   * <zh>openֵӦֱ,м
   * {@link com.ijchart.xychart.event.ChartShapeChangeEvent}¼.</zh>
   * <en>Sets the open line,and sends a
   * {@link com.ijchart.xychart.event.ChartShapeChangeEvent} to all registered listeners.</en>
   *
   * @param line Line2D <zh>ֱ߶ (Ϊnull).</zh>
   * <en>The line (null permitted).</en>
   */
  public void setOpenLine(Line2D line) {
    if (!(this.openLine == null || line == null)) {
      this.openLine = line ;
      fireChartShapeChanged() ;
    }
  }

  /**
   * <zh>ػopenֱߵpaint.</zh>
   * <en>Returns the paint used to draw open line.</en>
   *
   * @return Paint <zh>Paint (Ϊnull).</zh>
   * <en>The paint (maybe null).</en>
   */
  public Paint getOpenLinePaint() {
    return this.openLinePaint ;
  }

  /**
   * <zh>ûopenֱߵpaint,м
   * {@link com.ijchart.xychart.event.ChartShapeChangeEvent}¼.</zh>
   * <en>Sets the paint used to draw open line,and sends a
   * {@link com.ijchart.xychart.event.ChartShapeChangeEvent} to all registered listeners.</en>
   *
   * @param paint Paint <zh>Paint (Ϊnull).</zh>
   * <en>The paint (null permitted).</en>
   */
  public void setOpenLinePaint(Paint paint) {
    if (!(this.openLinePaint == null || paint == null)) {
      this.openLinePaint = paint ;
      fireChartShapeChanged() ;
    }
  }

  /**
   * <zh>closeֵӦֱ.</zh>
   * <en>Returns the close line.</en>
   *
   * @return Line2D <zh>ֱ߶ (Ϊnull).</zh>
   * <en>The line (maybe null).</en>
   */
  public Line2D getCloseLine() {
    return this.closeLine ;
  }

  /**
   * <zh>closeֵӦֱ,м
   * {@link com.ijchart.xychart.event.ChartShapeChangeEvent}¼.</zh>
   * <en>Sets the close line,and sends a
   * {@link com.ijchart.xychart.event.ChartShapeChangeEvent} to all registered listeners.</en>
   *
   * @param line Line2D <zh>ֱ߶ (Ϊnull).</zh>
   * <en>The line (null permitted).</en>
   */
  public void setCloseLine(Line2D line) {
    if (!(this.closeLine == null || line == null)) {
      this.closeLine = line ;
      fireChartShapeChanged() ;
    }
  }

  /**
   * <zh>ػcloseֱߵpaint.</zh>
   * <en>Returns the paint used to draw close line.</en>
   *
   * @return Paint <zh>Paint (Ϊnull).</zh>
   * <en>The paint (maybe null).</en>
   */
  public Paint getCloseLinePaint() {
    return this.closeLinePaint ;
  }

  /**
   * <zh>ûcloseֱߵpaint,м
   * {@link com.ijchart.xychart.event.ChartShapeChangeEvent}¼.</zh>
   * <en>Sets the paint used to draw close line,and sends a
   * {@link com.ijchart.xychart.event.ChartShapeChangeEvent} to all registered listeners.</en>
   *
   * @param paint Paint <zh>Paint (Ϊnull).</zh>
   * <en>The paint (null permitted).</en>
   */
  public void setCloseLinePaint(Paint paint) {
    if (!(this.closeLinePaint == null || paint == null)) {
      this.closeLinePaint = paint ;
      fireChartShapeChanged() ;
    }
  }

  /**
   * <zh>ͼ豸(Ļӡ)ϻƴͼ</zh>
   * <en>Draw the ChartShape on a Java 2d graphics device(such as the screen or the printer).</en>
   *
   * @param g2 Graphics2D <zh>ͼ豸(Ļӡ) (Ϊnull)</zh>
   * <en>The graphics device (such as the screen or the printer) (null not permitted).</en>
   */
  public void draw(Graphics2D g2) {
    if (g2 == null) {
      throw new IllegalArgumentException("Null 'g2' argument.") ;
    }
    if (!isVisible()) {
      return ;
    }

    Line2D notNullLine = this.highLowLine ;
    if (notNullLine == null) {
      notNullLine = this.openLine ;
    }
    if (notNullLine == null) {
      notNullLine = this.closeLine ;
    }
    if (notNullLine == null) {
      return ;
    }

    Graphics2DState state = new Graphics2DState(g2) ;
    state.saveOldState() ;

    double beginX = notNullLine.getX1() ;
    double beginY = notNullLine.getY1() ;

    g2.translate(beginX, beginY) ;
    g2.rotate(getRotate()) ;
    beforeDraw(g2) ;
    g2.setStroke(getBorderStroke()) ;

    //draw high-low line
    if (this.highLowLine != null && this.highLowLinePaint != null) {
      g2.setPaint(this.highLowLinePaint) ;
      Line2D newHighLowLine = new Line2D.Double(this.highLowLine.getX1() - beginX,
                                                this.highLowLine.getY1() - beginY,
                                                this.highLowLine.getX2() - beginX,
                                                this.highLowLine.getY2() - beginY) ;
      g2.draw(newHighLowLine) ;
    }

    //draw open line
    if (this.openLine != null && this.openLinePaint != null) {
      g2.setPaint(this.openLinePaint) ;
      Line2D newOpenLine = new Line2D.Double(this.openLine.getX1() - beginX,
                                             this.openLine.getY1() - beginY,
                                             this.openLine.getX2() - beginX,
                                             this.openLine.getY2() - beginY) ;
      g2.draw(newOpenLine) ;
    }

    //draw close line
    if (this.closeLine != null && this.closeLinePaint != null) {
      g2.setPaint(this.closeLinePaint) ;
      Line2D newCloseLine = new Line2D.Double(this.closeLine.getX1() - beginX,
                                              this.closeLine.getY1() - beginY,
                                              this.closeLine.getX2() - beginX,
                                              this.closeLine.getY2() - beginY) ;
      g2.draw(newCloseLine) ;
    }

    Rectangle2D drawArea = getDrawArea() ;
    Rectangle2D newDrawArea = new Rectangle2D.Double(drawArea.getX() - beginX,
                                                     drawArea.getY() - beginY,
                                                     drawArea.getWidth(), drawArea.getHeight()) ;
    drawTextWithTrimMarginRect(g2, newDrawArea) ;

    afterDraw(g2) ;
    g2.rotate( -getRotate()) ;
    state.restoreOldState() ;
    drawSelect(g2) ;
  }

  /**
   * <zh>ͼ豸(Ļӡ)ϻͼεѡв,Ϊͼοܻص,ԻͼʱҪȻеͼκ,
   * ٵô˷ѡв.</zh>
   * <en>Draw the selected ChartShape on a Java 2d graphics device(such
   * as the screen or the printer).</en>
   *
   * @param g2 Graphics2D <zh>ͼ豸(Ļӡ) (Ϊnull)</zh>
   * <en>the graphics device (such as the screen or the printer) (null not permitted).</en>
   */
  protected void drawSelect(Graphics2D g2) {
    if (isSelected() && this.highLowLine != null) {
      double x1 = this.highLowLine.getX1() ;
      double y1 = this.highLowLine.getY1() ;
      Point2D.Double[] points = new Point2D.Double[] {
                                new Point2D.Double(0, 0),
                                new Point2D.Double(this.highLowLine.getX2() - x1,
                                                   this.highLowLine.getY2() - y1)} ;

      drawAnchorByPoints(g2, x1, y1, points) ;
    }
  }

  /**
   * <zh>ƶͼ,X᷽ƫtx,Y᷽ƫty,м
   * {@link com.ijchart.xychart.event.ChartShapeChangeEvent}¼.</zh>
   * <en>Concatenates the current Graphics2D Transform with a translation transform.and sends
   * a {@link com.ijchart.xychart.event.ChartShapeChangeEvent} to all registered listeners.
   * Subsequent rendering is translated by the specified
   * distance relative to the previous position.
   * This is equivalent to calling transform(T), where T is an
   * AffineTransform represented by the following matrix:
   * <pre>
   *		[   1    0    tx  ]
   *		[   0    1    ty  ]
   *		[   0    0    1   ]
   * </pre>.</en>
   *
   * @param tx double <zh>X᷽ľ.</zh>
   * <en>The distance to translate along the x-axis.</en>
   * @param ty double <zh>Y᷽ľ.</zh>
   * <en>The distance to translate along the y-axis.</en>
   */
  public void translate(double tx, double ty) {
    if (!(tx == 0 && ty == 0)) {
      if (this.highLowLine != null) {
        this.highLowLine.setLine(this.highLowLine.getX1() + tx, this.highLowLine.getY1() + ty,
                                 this.highLowLine.getX2() + tx, this.highLowLine.getY2() + ty) ;
      }
      if (this.openLine != null) {
        this.openLine.setLine(this.openLine.getX1() + tx, this.openLine.getY1() + ty,
                              this.openLine.getX2() + tx, this.openLine.getY2() + ty) ;
      }
      if (this.closeLine != null) {
        this.closeLine.setLine(this.closeLine.getX1() + tx, this.closeLine.getY1() + ty,
                               this.closeLine.getX2() + tx, this.closeLine.getY2() + ty) ;
      }

      fireChartShapeChanged() ;
    }
  }

  /**
   * <zh>ͼϽǵ(ͼεת).</zh>
   * <en>Returns the top left corner coordinates of this shape (not consider rotation).</en>
   *
   * @return Point2D <zh>ֵ.</zh>
   * <en>The coordinate.</en>
   */
  public Point2D getTopLeftPoint() {
    if (this.highLowLine == null && this.openLine == null && this.closeLine == null) {
      return super.getTopLeftPoint() ;
    } else {
      double x = Double.NaN, y = Double.NaN ;
      if (this.highLowLine != null) {
        x = Math.min(this.highLowLine.getX1(), this.highLowLine.getX2()) ;
        y = Math.min(this.highLowLine.getY1(), this.highLowLine.getY2()) ;
      }

      if (this.openLine != null) {
        if (Double.isNaN(x) || Double.isNaN(y)) {
          x = Math.min(this.openLine.getX1(), this.openLine.getX2()) ;
          y = Math.min(this.openLine.getY1(), this.openLine.getY2()) ;
        } else {
          x = Math.min(x, Math.min(this.openLine.getX1(), this.openLine.getX2())) ;
          y = Math.min(y, Math.min(this.openLine.getY1(), this.openLine.getY2())) ;
        }
      }

      if (this.closeLine != null) {
        if (Double.isNaN(x) || Double.isNaN(y)) {
          x = Math.min(this.closeLine.getX1(), this.closeLine.getX2()) ;
          y = Math.min(this.closeLine.getY1(), this.closeLine.getY2()) ;
        } else {
          x = Math.min(x, Math.min(this.closeLine.getX1(), this.closeLine.getX2())) ;
          y = Math.min(y, Math.min(this.closeLine.getY1(), this.closeLine.getY2())) ;
        }
      }

      return new Point2D.Double(x, y) ;
    }
  }

  /**
   * <zh>ʱͼεĵ.</zh>
   * <en>Returns the lock coordinates of this shape.</en>
   *
   * @return Point2D <zh>ֵ.</zh>
   * <en>The coordinate.</en>
   */
  public Point2D getLockPoint() {
    Line2D notNullLine = this.highLowLine ;
    if (notNullLine == null) {
      notNullLine = this.openLine ;
    }
    if (notNullLine == null) {
      notNullLine = this.closeLine ;
    }
    if (notNullLine == null) {
      return null ;
    }

    double beginX = notNullLine.getX1() ;
    double beginY = notNullLine.getY1() ;
    Line2D newLine = new Line2D.Double(notNullLine.getX1() - beginX, notNullLine.getY1() - beginY,
                                       notNullLine.getX2() - beginX, notNullLine.getY2() - beginY) ;

    AffineTransform transform = AffineTransform.getTranslateInstance(beginX, beginY) ;
    transform.rotate(getRotate()) ;
    Shape rotateShape = transform.createTransformedShape(newLine) ;
    Rectangle2D rotateBound = rotateShape.getBounds2D() ;

    return new Point2D.Double(rotateBound.getCenterX(), rotateBound.getCenterY()) ;
  }

  /**
   * <zh>任ͼ.</zh>
   * <en>Calculate the transformed shape.</en>
   */
  protected void calculateTransformedShape() {
    if (transformedShape == null) {
      if (this.highLowLine == null && this.openLine == null && this.closeLine == null) {
        transformedShape = null ;
      } else {
        Line2D notNullLine = this.highLowLine ;
        if (notNullLine == null) {
          notNullLine = this.openLine ;
        }
        if (notNullLine == null) {
          notNullLine = this.closeLine ;
        }
        if (notNullLine == null) {
          return ;
        }

        GeneralPath path = new GeneralPath() ;

        double beginX = notNullLine.getX1() ;
        double beginY = notNullLine.getY1() ;

        if (this.highLowLine != null && this.highLowLinePaint != null) {
          Line2D newHighLowLine = new Line2D.Double(this.highLowLine.getX1() - beginX,
                                                    this.highLowLine.getY1() - beginY,
                                                    this.highLowLine.getX2() - beginX,
                                                    this.highLowLine.getY2() - beginY) ;

          Shape rect = StrokeUtil.lineToRectangle(newHighLowLine, getBorderStroke(), 0) ;
          path.append(rect.getPathIterator(null), false) ;
        }

        if (this.openLine != null && this.openLinePaint != null) {
          Line2D newOpenLine = new Line2D.Double(this.openLine.getX1() - beginX,
                                                 this.openLine.getY1() - beginY,
                                                 this.openLine.getX2() - beginX,
                                                 this.openLine.getY2() - beginY) ;

          Shape rect = StrokeUtil.lineToRectangle(newOpenLine, getBorderStroke(), 0) ;
          path.append(rect.getPathIterator(null), false) ;
        }

        if (this.closeLine != null && this.closeLinePaint != null) {
          Line2D newCloseLine = new Line2D.Double(this.closeLine.getX1() - beginX,
                                                  this.closeLine.getY1() - beginY,
                                                  this.closeLine.getX2() - beginX,
                                                  this.closeLine.getY2() - beginY) ;

          Shape rect = StrokeUtil.lineToRectangle(newCloseLine, getBorderStroke(), 0) ;
          path.append(rect.getPathIterator(null), false) ;
        }

        AffineTransform transform = new AffineTransform() ;
        transform.translate(beginX, beginY) ;
        transform.rotate(getRotate()) ;
        transformedShape = transform.createTransformedShape(path) ;
      }
    }
  }

  /**
   * <zh>ȵͼ.</zh>
   * <en>Returns the hotspot shape.</en>
   *
   * @return Shape <zh>ȵͼ (Ϊnull).</zh>
   * <en>The hotspot shape (maybe null).</en>
   */
  protected Shape getHotspotShape() {
    Rectangle2D drawArea = getDrawArea() ;
    AffineTransform transform = AffineTransform.getTranslateInstance(drawArea.getX(), drawArea.getY()) ;
    transform.rotate(getRotate()) ;

    Rectangle2D rect = new Rectangle2D.Double(0, 0, drawArea.getWidth(), drawArea.getHeight()) ;
    return transform.createTransformedShape(rect) ;
  }

  /**
   * <zh>ͼε.</zh>
   * <en>The type for describe this shape.</en>
   *
   * @return ShapeType <zh>{@link ShapeType}ඨĳ (Ϊnull).</zh>
   * <en>The constant value in class {@link ShapeType}.</en>
   */
  public ShapeType getShapeType() {
    return ShapeType.SHAPE_HIGH_LOW ;
  }

  /**
   * <zh>Ƿ.</zh>
   * <en>Indicates whether some other object is "equal to" this one.</en>
   *
   * @param obj <zh>ҪȽϵĶ</zh>
   * <en>the reference object with which to compare.</en>
   * @return <zh>ȷtrue,򷵻false.</zh>
   * <en>true if this object is the same as the obj argument; false
   *  otherwise.</en>
   */
  public boolean equals(Object obj) {
    if (this == obj) {
      return false ;
    }
    if (!super.equals(obj)) {
      return false ;
    }
    if (!super.equals(obj)) {
      return false ;
    }

    if (!(obj instanceof HighLowChartShape)) {
      return false ;
    }

    HighLowChartShape that = (HighLowChartShape) obj ;
    if (!EqualsUtil.equals(this.highLowLine, that.highLowLine)) {
      return false ;
    }
    if (!EqualsUtil.equals(this.highLowLinePaint, that.highLowLinePaint)) {
      return false ;
    }
    if (!EqualsUtil.equals(this.openLine, that.openLine)) {
      return false ;
    }
    if (!EqualsUtil.equals(this.openLinePaint, that.openLinePaint)) {
      return false ;
    }
    if (!EqualsUtil.equals(this.closeLine, that.closeLine)) {
      return false ;
    }
    if (!EqualsUtil.equals(this.closeLinePaint, that.closeLinePaint)) {
      return false ;
    }

    return true ;
  }

  /**
   * <zh>hashcodeֵ.</zh>
   * <en>Returns a hash code value for the object.</en>
   *
   * @return int <zh>hashcodeֵ.</zh>
   * <en>A hash code value for this object.</en>
   */
  public int hashCode() {
    double hashCode = super.hashCode() ;
    hashCode = 37 * hashCode + HashcodeUtil.hashCode(this.highLowLine) ;
    hashCode = 37 * hashCode + HashcodeUtil.hashCode(this.highLowLinePaint) ;
    hashCode = 37 * hashCode + HashcodeUtil.hashCode(this.openLine) ;
    hashCode = 37 * hashCode + HashcodeUtil.hashCode(this.openLinePaint) ;
    hashCode = 37 * hashCode + HashcodeUtil.hashCode(this.closeLine) ;
    hashCode = 37 * hashCode + HashcodeUtil.hashCode(this.closeLinePaint) ;

    return (int) hashCode ;
  }

  /**
   * <zh>ؿ¡Ķ.</zh>
   * <en>Creates and returns a copy of this object.</en>
   * @return object   a clone of this instance.
   * @exception  CloneNotSupportedException  if the object's class does not
   *               support the Cloneable interface. Subclasses
   *               that override the clone method can also
   *               throw this exception to indicate that an instance cannot
   *               be cloned.
   */
  public Object clone() throws CloneNotSupportedException {
    return super.clone() ;
  }

  /**
   * <zh>toString.</zh>
   * <en>Returns a string representation of the object. </en>
   *
   * @return  a string representation of the object.
   */
  public String toString() {
    return "HighLowChartShape" ;
  }

  /**
   * Provides serialization support.
   *
   * @param stream  the output stream.
   *
   * @throws IOException  if there is an I/O error.
   */
  private void writeObject(ObjectOutputStream stream) throws IOException {
    stream.defaultWriteObject() ;

    SerialUtil.writeShape(stream, this.highLowLine) ;
    SerialUtil.writePaint(stream, this.highLowLinePaint) ;

    SerialUtil.writeShape(stream, this.openLine) ;
    SerialUtil.writePaint(stream, this.openLinePaint) ;

    SerialUtil.writeShape(stream, this.closeLine) ;
    SerialUtil.writePaint(stream, this.closeLinePaint) ;
  }

  /**
   * Provides serialization support.
   *
   * @param stream  the input stream.
   *
   * @throws IOException  if there is an I/O error.
   * @throws ClassNotFoundException  if there is a classpath problem.
   */
  private void readObject(ObjectInputStream stream) throws IOException, ClassNotFoundException {
    stream.defaultReadObject() ;

    this.highLowLine = (Line2D) SerialUtil.readShape(stream) ;
    this.highLowLinePaint = SerialUtil.readPaint(stream) ;

    this.openLine = (Line2D) SerialUtil.readShape(stream) ;
    this.openLinePaint = SerialUtil.readPaint(stream) ;

    this.closeLine = (Line2D) SerialUtil.readShape(stream) ;
    this.closeLinePaint = SerialUtil.readPaint(stream) ;
  }
}
