package com.ijchart.xychart.title ;

import java.awt.BasicStroke ;
import java.awt.Color ;
import java.awt.Paint ;
import java.awt.Stroke ;
import java.io.IOException ;
import java.io.ObjectInputStream ;
import java.io.ObjectOutputStream ;
import java.io.Serializable ;
import java.util.Arrays ;
import java.util.EventListener ;

import javax.swing.event.EventListenerList ;

import com.ijchart.xychart.constant.HorizontalAlignment ;
import com.ijchart.xychart.constant.VerticalAlignment ;
import com.ijchart.xychart.event.TitleChangeEvent ;
import com.ijchart.xychart.listener.IAxisChangeListener ;
import com.ijchart.xychart.listener.ITitleChangeListener ;
import com.ijchart.xychart.ui.RectangleEdge ;
import com.ijchart.xychart.ui.RectangleInsets ;
import com.ijchart.xychart.util.EqualsUtil ;
import com.ijchart.xychart.util.HashcodeUtil ;
import com.ijchart.xychart.util.SerialUtil ;

/**
 * <p>Title: IJChart</p>
 *
 * <p>Description: a chart library for the Java(tm) platform.</p>
 * <zh>ṩĳʵ.</zh>
 * <en>The abstract implementation of title.</en>
 *
 * <p>Copyright: Copyright (c) 2013</p>
 *
 * <p>Company: </p>
 *
 * @author zhang feng min,shanghai china.
 * <a href="mailto:ijchartsh@gmail.com">Contract me by email.</a>
 * @version 1.0
 */
public abstract class AbstractTitle implements ITitle, Serializable, Cloneable {
  /**
   * <zh>л汾ID.</zh>
   * <en>For serialization.</en>
   */
  private static final long serialVersionUID = -4202028181378578190L ;

  /**
   * <zh>ȱʡıλ.</zh>
   * <en>The default title position.</en>
   */
  public static final RectangleEdge DEFAULT_TITLE_POSITION = RectangleEdge.TOP ;

  /**
   * <zh>ȱʡˮƽ뷽ʽ.</zh>
   * <en>The default horizontal alignment.</en>
   */
  public static final HorizontalAlignment DEFAULT_HORIZONTAL_ALIGNMENT = HorizontalAlignment.CENTER ;

  /**
   * <zh>ȱʡĴֱ뷽ʽ.</zh>
   * <en>The default vertical alignment.</en>
   */
  public static final VerticalAlignment DEFAULT_VERTICAL_ALIGNMENT = VerticalAlignment.CENTER ;

  /**
   * <zh>ȱʡĿհ.</zh>
   * <en>Default title padding.</en>
   */
  public static final RectangleInsets DEFAULT_PADDING = RectangleInsets.INSETS_TWO ;

  /**
   * <zh>ȱʡ䱳Paint.</zh>
   * <en>The default background fill paint.</en>
   */
  public static final Paint DEFAULT_BACKGROUND_PAINT = Color.WHITE ;

  /**
   * <zh>ȱʡ߿Ƿɼ.</zh>
   * <en>A default flag that controls whether or not the chart border is drawn. </en>
   */
  public static final boolean DEFAULT_BORDER_VISIBLE = false ;

  /**
   * <zh>ȱʡǷʹñPaintͼ.</zh>
   * <en>A default flag controls whether or not fill shape with background paint.</en>
   */
  public static final boolean DEFAULT_FILL_BACKGROUND = false ;

  /**
   * <zh>Ƿʱֹ¼֪ͨƵȱʡֵ.</zh>
   * <en>A default flag that can be used to temporarily disable the listener mechanism.</en>
   */
  public static final boolean DEFAULT_NOTIFY = true ;

  /**
   * <zh>ȱʡǰalpha͸ֵ.</zh>
   * <en>The default foreground alpha transparency.</en>
   */
  public static final float DEFAULT_BACKGROUND_ALPHA = 1.0f ;

  /**
   * <zh>λ.</zh>
   * <en>The title position. </en>
   */
  private RectangleEdge position ;

  /**
   * <zh>ˮƽ뷽ʽ.</zh>
   * <en> The horizontal alignment of the title content.</en>
   */
  private HorizontalAlignment horizontalAlignment ;

  /**
   * <zh>Ĵֱ뷽ʽ.</zh>
   * <en>The vertical alignment of the title content.</en>
   */
  private VerticalAlignment verticalAlignment ;

  /**
   * <zh>Ŀհ.</zh>
   * <en>The title padding.</en>
   */
  private RectangleInsets padding ;

  /**
   * <zh>Ƿʱֹ¼֪ͨ.</zh>
   * <en>A flag that can be used to temporarily disable the listener mechanism.</en>
   */
  private boolean notify ;

  /**
   * <zh>tooltipַ.</zh>
   * <en>The tool tip text.</en>
   */
  private String toolTipText ;

  /**
   * <zh>URLַ.</zh>
   * <en>The URL text (can be null).</en>
   */
  private String urlText ;

  /**
   * <zh>߿Ƿɼ.</zh>
   * <en>A flag that controls whether or not the chart border is drawn. </en>
   */
  private boolean borderVisible ;

  /**
   * <zh>Ʊ߿Stroke.</zh>
   * <en>The stroke used to draw the chart border (if visible).</en>
   */
  private transient Stroke borderStroke ;

  /**
   * <zh>Ʊ߿Paint.</zh>
   * <en>The paint used to draw the chart border (if visible).</en>
   */
  private transient Paint borderPaint ;

  /**
   * <zh>ǷʹñPaintͼ.</zh>
   * <en>A flag controls whether or not fill shape with background paint.</en>
   */
  private boolean fillBackground ;

  /**
   * <zh>ɫ.</zh>
   * <en>The background paint.</en>
   */
  private transient Paint backgroundPaint ;

  /**
   * <zh>͸alphaֵ.</zh>
   * <en>The alpha transparency for the background paint.</en>
   */
  private float backgroundAlpha ;

  /**
   * <zh>¼.</zh>
   * <en>Storage for registered change listeners.</en>
   */
  private transient EventListenerList listenerList ;

  /**
   * <zh>.</zh>
   * <en>Creates a new title, using default attributes where necessary.</en>
   */
  protected AbstractTitle() {
    this(DEFAULT_TITLE_POSITION, DEFAULT_HORIZONTAL_ALIGNMENT, DEFAULT_VERTICAL_ALIGNMENT,
         DEFAULT_PADDING) ;
  }

  /**
   * <zh>.</zh>
   * <en>Creates a new title, using default attributes where necessary.</en>
   *
   * @param position RectangleEdge <zh>λ (Ϊnull).</zh>
   * <en>The position of the title (null not permitted).</en>
   * @param horizontalAlignment  HorizontalAlignment <zh>ˮƽ뷽ʽ (Ϊnull).</zh>
   * <en>The horizontal alignment of the title (null not permitted).</en>
   * @param verticalAlignment VerticalAlignment <zh>Ĵֱ뷽ʽ (Ϊnull).</zh>
   * <en>The vertical alignment of the title (null not permitted).</en>
   */
  protected AbstractTitle(RectangleEdge position, HorizontalAlignment horizontalAlignment,
                          VerticalAlignment verticalAlignment) {
    this(position, horizontalAlignment, verticalAlignment, DEFAULT_PADDING) ;
  }

  /**
   * <zh>.</zh>
   * <en>Creates a new title.</en>
   *
   * @param position RectangleEdge <zh>λ (Ϊnull).</zh>
   * <en>The position of the title (null not permitted).</en>
   * @param horizontalAlignment  HorizontalAlignment <zh>ˮƽ뷽ʽ (Ϊnull).</zh>
   * <en>The horizontal alignment of the title (null not permitted).</en>
   * @param verticalAlignment VerticalAlignment <zh>Ĵֱ뷽ʽ (Ϊnull).</zh>
   * <en>The vertical alignment of the title (null not permitted).</en>
   * @param padding RectangleInsets <zh>Ŀհ.</zh>
   * <en>The title padding.</en>
   */
  protected AbstractTitle(RectangleEdge position, HorizontalAlignment horizontalAlignment,
                          VerticalAlignment verticalAlignment, RectangleInsets padding) {
    if (position == null) {
      throw new IllegalArgumentException("Null 'position' argument.") ;
    }
    if (horizontalAlignment == null) {
      throw new IllegalArgumentException("Null 'horizontalAlignment' argument.") ;
    }
    if (verticalAlignment == null) {
      throw new IllegalArgumentException("Null 'verticalAlignment' argument.") ;
    }
    if (padding == null) {
      throw new IllegalArgumentException("Null 'padding' argument.") ;
    }

    this.position = DEFAULT_TITLE_POSITION ;
    this.horizontalAlignment = horizontalAlignment ;
    this.verticalAlignment = verticalAlignment ;
    this.padding = padding ;
    this.notify = DEFAULT_NOTIFY ;

    this.toolTipText = null ;
    this.urlText = null ;

    this.fillBackground = DEFAULT_FILL_BACKGROUND ;
    this.backgroundPaint = DEFAULT_BACKGROUND_PAINT ;
    this.backgroundAlpha = DEFAULT_BACKGROUND_ALPHA ;

    this.borderVisible = DEFAULT_BORDER_VISIBLE ;
    this.borderStroke = new BasicStroke(1.0f) ;
    this.borderPaint = Color.BLACK ;

    this.listenerList = new EventListenerList() ;
  }

  /**
   * <zh>رλ.</zh>
   * <en>Returns the position of the title.</en>
   *
   * @return RectangleEdge <zh>λ.</zh>
   * <en>The title position (never null).</en>
   */
  public RectangleEdge getPosition() {
    return this.position ;
  }

  /**
   * <zh>ñλ,м{@link TitleChangeEvent}¼.</zh>
   * <en>Sets the position for the title and sends a {@link TitleChangeEvent} to
   * all registered listeners.</en>
   *
   * @param position RectangleEdge <zh>λ (Ϊnull).</zh>
   * <en>The position (null not permitted).</en>
   */
  public void setPosition(RectangleEdge position) {
    if (position == null) {
      throw new IllegalArgumentException("Null 'position' argument.") ;
    }

    if (!this.position.equals(position)) {
      this.position = position ;
      fireTitleChanged() ;
    }
  }

  /**
   * <zh>رˮƽ뷽ʽ.</zh>
   * <en>Returns the horizontal alignment of the title.</en>
   *
   * @return HorizontalAlignment <zh>ˮƽ뷽ʽ (Ϊnull).</zh>
   * <en>The horizontal alignment (never null).</en>
   */
  public HorizontalAlignment getHorizontalAlignment() {
    return this.horizontalAlignment ;
  }

  /**
   * <zh>ñˮƽ뷽ʽ,м{@link TitleChangeEvent}¼.</zh>
   * <en>Sets the horizontal alignment for the title and sends a
   * {@link TitleChangeEvent} to all registered listeners.</en>
   *
   * @param alignment HorizontalAlignment <zh>ˮƽ뷽ʽ (Ϊnull).</zh>
   * <en>The horizontal alignment (null not permitted).</en>
   */
  public void setHorizontalAlignment(HorizontalAlignment alignment) {
    if (alignment == null) {
      throw new IllegalArgumentException("Null 'alignment' argument.") ;
    }

    if (!this.horizontalAlignment.equals(alignment)) {
      this.horizontalAlignment = alignment ;
      fireTitleChanged() ;
    }
  }

  /**
   * <zh>رĴֱ뷽ʽ.</zh>
   * <en>Returns the vertical alignment of the title.</en>
   *
   * @return VerticalAlignment <zh>رĴֱ뷽ʽ (Ϊnull).</zh>
   * <en>The vertical alignment (never null).</en>
   */
  public VerticalAlignment getVerticalAlignment() {
    return this.verticalAlignment ;
  }

  /**
   * <zh>ñĴֱ뷽ʽ,м{@link TitleChangeEvent}¼.</zh>
   * <en>Sets the vertical alignment for the title, and notifies any registered
   * listeners of the change.</en>
   *
   * @param alignment VerticalAlignment <zh>Ĵֱ뷽ʽ.</zh>
   * <en>The new vertical alignment (null not permitted).</en>
   */
  public void setVerticalAlignment(VerticalAlignment alignment) {
    if (alignment == null) {
      throw new IllegalArgumentException("Null 'alignment' argument.") ;
    }

    if (!this.verticalAlignment.equals(alignment)) {
      this.verticalAlignment = alignment ;
      fireTitleChanged() ;
    }
  }

  /**
   * <zh>صⷢ˱仯Ƿ֪ͨ.</zh>
   * <en>Returns the flag that indicates whether or not the notification
   * mechanism is enabled.</en>
   *
   * @return boolean <zh>ֵ.</zh>
   * <en>The flag.</en>
   */
  public boolean getNotify() {
    return this.notify ;
  }

  /**
   * <zh>õⷢ˱仯Ƿ֪ͨ,м{@link TitleChangeEvent}¼.</zh>
   * <en>Sets the flag that indicates whether or not the notification mechanism
   * is enabled.  There are certain situations (such as cloning) where you
   * want to turn notification off temporarily.</en>
   *
   * @param flag boolean <zh>ֵ.</zh>
   * <en>The new value of the flag.</en>
   */
  public void setNotify(boolean flag) {
    this.notify = flag ;

    if (notify) {
      fireTitleChanged() ;
    }
  }

  /**
   * <zh>رĿհ.</zh>
   * <en>Returns the title padding.</en>
   *
   * @return RectangleInsets <zh>Ŀհ (Ϊnull).</zh>
   * <en>Returns the title padding (never null).</en>
   */
  public RectangleInsets getPadding() {
    return this.padding ;
  }

  /**
   * <zh>ñĿհ,м{@link TitleChangeEvent}¼.</zh>
   * <en>Sets the title padding, and notifies any registered
   * listeners of the change.</en>
   *
   * @param padding RectangleInsets <zh>Ŀհ (Ϊnull).</zh>
   * <en>The title padding (null not permitted).</en>
   */
  public void setPadding(RectangleInsets padding) {
    if (!this.padding.equals(padding)) {
      this.padding = padding ;
      fireTitleChanged() ;
    }
  }

  /**
   * <zh>tooltipַ.</zh>
   * <en>Returns the tool tip text.</en>
   *
   * @return String <zh>tooltipַ</zh>
   * <en>The tool tip text (possibly null).</en>
   */
  public String getToolTipText() {
    return this.toolTipText ;
  }

  /**
   * <zh>tooltipַ,м{@link TitleChangeEvent}¼.</zh>
   * <en>Sets the tool tip text to the specified text and sends a
   * {@link TitleChangeEvent} to all registered listeners.</en>
   *
   * @param tooltip String <zh>tooltipַ.</zh>
   * <en>The text (null permitted).</en>
   */
  public void setToolTipText(String tooltip) {
    if (!EqualsUtil.equals(this.toolTipText, tooltip)) {
      this.toolTipText = tooltip ;
      fireTitleChanged() ;
    }
  }

  /**
   * <zh>URLַ.</zh>
   * <en>Returns the URL text.</en>
   *
   * @return String <zh>URLַ (Ϊnull).</zh>
   * <en>The URL text (possibly null).</en>
   */
  public String getURLText() {
    return this.urlText ;
  }

  /**
   * <zh>URLַ,м{@link TitleChangeEvent}¼.</zh>
   * <en>Sets the URL text to the specified text and sends a
   * {@link TitleChangeEvent} to all registered listeners.</en>
   *
   * @param url String <zh>URLַ.</zh>
   * <en>The URL string (null permitted).</en>
   */
  public void setURLText(String url) {
    if (!EqualsUtil.equals(this.urlText, url)) {
      this.urlText = url ;
      fireTitleChanged() ;
    }
  }

  /**
   * <zh>ǷʹñPaintͼ.</zh>
   * <en>Returns whether or not fill shape with background paint.</en>
   *
   * @return boolean <zh>ֵ.</zh>
   * <en>A boolean.</en>
   */
  public boolean isFillBackground() {
    return this.fillBackground ;
  }

  /**
   * <zh>ǷʹñPaintͼ,м{@link TitleChangeEvent}¼.</zh>
   * <en>Sets whether or not fill shape with background paint and sends a {@link TitleChangeEvent}
   * to all registered listeners.</en>
   *
   * @param flag boolean <zh>ֵ.</zh>
   * <en>A boolean.</en>
   */
  public void setFillBackground(boolean flag) {
    if (this.fillBackground != flag) {
      this.fillBackground = flag ;
      fireTitleChanged() ;
    }
  }

  /**
   * <zh>رɫ.</zh>
   * <en>Returns the background paint.</en>
   *
   * @return Paint <zh>ɫ (Ϊnull).</zh>
   * <en>The paint (possibly null).</en>
   */
  public Paint getBackgroundPaint() {
    return this.backgroundPaint ;
  }

  /**
   * <zh>ñɫ,м{@link TitleChangeEvent}¼.</zh>
   * <en>Sets the background paint and sends a {@link TitleChangeEvent} to all
   * registered listeners.  If you set this attribute to null,
   * no background is painted (which makes the title background transparent).</en>
   *
   * @param paint Paint <zh>ɫ (Ϊnull).</zh>
   * <en>The background paint (null permitted).</en>
   */
  public void setBackgroundPaint(Paint paint) {
    if (!(this.backgroundPaint == null && paint == null)) {
      this.backgroundPaint = paint ;
      fireTitleChanged() ;
    }
  }

  /**
   * <zh>رalphaֵ.</zh>
   * <en>Returns the alpha transparency of the plot area background.</en>
   *
   * @return <zh>͸alphaֵ.</zh>
   * <en>The alpha transparency.</en>
   */
  public float getBackgroundAlpha() {
    return this.backgroundAlpha ;
  }

  /**
   * <zh>ñ͸alphaֵ,м
   * {@link com.ijchart.xychart.event.PlotChangeEvent}¼.</zh>
   * <en>Sets the alpha transparency of the plot area background, and notifies
   * registered listeners that the plot has been modified.</en>
   *
   * @param alpha <zh>alphaֵ,ֵڱ[0.0,1.0].</zh>
   * <en>The new alpha value (in the range 0.0f to 1.0f).</en>
   */
  public void setBackgroundAlpha(float alpha) {
    if (alpha < 0.0f || alpha > 1.0f) {
      throw new IllegalArgumentException("The 'alpha' value must be in the range 0.0f to 1.0f.") ;
    }

    if (this.backgroundAlpha != alpha) {
      this.backgroundAlpha = alpha ;
      fireTitleChanged() ;
    }
  }

  /**
   * <zh>ر߿Ƿɼ.</zh>
   * <en>Returns a flag that controls whether or not a border is drawn around the
   * outside of the chart.</en>
   *
   * @return boolean <zh>ֵ.</zh>
   * <en>A boolean.</en>
   */
  public boolean isBorderVisible() {
    return this.borderVisible ;
  }

  /**
   * <zh>ñ߿Ƿɼ,м{@link TitleChangeEvent}¼.</zh>
   * <en>Sets a flag that controls whether or not a border is drawn around the
   * outside of the chart,and sends a {@link TitleChangeEvent} to all registered listeners.</en>
   *
   * @param visible boolean <zh>ֵ.</zh>
   * <en>A boolean.</en>
   */
  public void setBorderVisible(boolean visible) {
    if (this.borderVisible != visible) {
      this.borderVisible = visible ;
      fireTitleChanged() ;
    }
  }

  /**
   * <zh>ػƱ߿stroke.</zh>
   * <en>Returns the stroke used to draw the chart border (if visible).</en>
   *
   * @return Stroke <zh>Ʊ߿stroke (Ϊnull).</zh>
   * <en>The border stroke (never null).</en>
   */
  public Stroke getBorderStroke() {
    return this.borderStroke ;
  }

  /**
   * <zh>ûƱ߿stroke,м{@link TitleChangeEvent}¼.</zh>
   * <en>Sets the stroke used to draw the chart border (if visible),and
   * sends a {@link TitleChangeEvent} to all registered listeners.</en>
   *
   * @param stroke Stroke <zh>Ʊ߿stroke (Ϊnull).</zh>
   * <en>The stroke (null not permitted).</en>
   */
  public void setBorderStroke(Stroke stroke) {
    if (stroke == null) {
      throw new IllegalArgumentException("Null 'stroke' argument.") ;
    }

    this.borderStroke = stroke ;
    fireTitleChanged() ;
  }

  /**
   * <zh>ػƱ߿paint.</zh>
   * <en>Returns the paint used to draw the chart border (if visible).</en>
   *
   * @return Paint <zh>Ʊ߿paint (Ϊnull).</zh>
   * <en>The border paint (never null).</en>
   */
  public Paint getBorderPaint() {
    return this.borderPaint ;
  }

  /**
   * <zh>ûƱ߿paint,м{@link TitleChangeEvent}¼.</zh>
   * <en>Sets the paint used to draw the chart border (if visible),and
   * sends a {@link TitleChangeEvent} to all registered listeners.</en>
   *
   * @param paint Paint <zh>Ʊ߿paint (Ϊnull).</zh>
   * <en>The paint (null not permitted).</en>
   */
  public void setBorderPaint(Paint paint) {
    if (paint == null) {
      throw new IllegalArgumentException("Null 'paint' argument.") ;
    }

    this.borderPaint = paint ;
    fireTitleChanged() ;
  }

  /**
   * <zh>¡.</zh>
   * <en>Returns a clone of the title.</en>
   *
   * @return Object A clone of the title.
   *
   * @throws CloneNotSupportedException not thrown by this class, but it may
   *         be thrown by subclasses.
   */
  public Object clone() throws CloneNotSupportedException {
    AbstractTitle duplicate = (AbstractTitle)super.clone() ;
    duplicate.listenerList = new EventListenerList() ;

    return duplicate ;
  }

  /**
   * <zh>һ.</zh>
   * <en>Registers an object for notification of changes to the title.</en>
   *
   * @param listener ITitleChangeListener <zh>¼.</zh>
   * <en>The object that is being registered.</en>
   */
  public void addChangeListener(ITitleChangeListener listener) {
    this.listenerList.add(ITitleChangeListener.class, listener) ;
  }

  /**
   * <zh>ɾһ.</zh>
   * <en>Unregisters an object for notification of changes to the chart title.</en>
   *
   * @param listener ITitleChangeListener <zh>¼.</zh>
   * <en>The object that is being unregistered.</en>
   */
  public void removeChangeListener(ITitleChangeListener listener) {
    this.listenerList.remove(ITitleChangeListener.class, listener) ;
  }

  /**
   * <zh>ɾм.</zh>
   * <en>Deregisters all object for notification of changes to the axis.</en>
   */
  public void removeAllChangeListener() {
    Object[] listeners = this.listenerList.getListenerList() ;

    for (int i = listeners.length - 2 ; i >= 0 ; i -= 2) {
      this.listenerList.remove(IAxisChangeListener.class, (IAxisChangeListener) listeners[i + 1]) ;
    }
  }

  /**
   * <zh>жǷмlistener.</zh>
   * <en>Returns true if the specified object is registered with
   * the dataset as a listener.  Most applications won't need to call this
   * method, it exists mainly for use by unit testing code.</en>
   *
   * @param listener  the listener.
   *
   * @return A boolean.
   */
  public boolean hasListener(EventListener listener) {
    return Arrays.asList(this.listenerList.getListenerList()).contains(listener) ;
  }

  /**
   * <zh>ͨмݼ˱仯.</zh>
   * <en>Notifies all registered listeners that the dataset has changed.</en>
   */
  protected void fireTitleChanged() {
    notifyListeners(new TitleChangeEvent(this)) ;
  }

  /**
   * <zh>֪ͨб仯¼.</zh>
   * <en>Notifies all registered listeners that the chart title has changed in some way.</en>
   *
   * @param event TitleChangeEvent <zh>仯¼.</zh>
   * <en>An object that contains information about the change to the title.</en>
   */
  protected void notifyListeners(TitleChangeEvent event) {
    if (this.notify) {
      Object[] listeners = this.listenerList.getListenerList() ;
      for (int i = listeners.length - 2 ; i >= 0 ; i -= 2) {
        if (listeners[i] == ITitleChangeListener.class) {
          ((ITitleChangeListener) listeners[i + 1]).titleChanged(event) ;
        }
      }
    }
  }

  /**
   * <zh>Ƿ.</zh>
   * <en>Tests an object for equality with this title.</en>
   *
   * @param obj Object Tthe object (null not permitted).
   *
   * @return true or false.
   */
  public boolean equals(Object obj) {
    if (obj == this) {
      return true ;
    }
    if (!(obj instanceof AbstractTitle)) {
      return false ;
    }
    if (!super.equals(obj)) {
      return false ;
    }

    AbstractTitle that = (AbstractTitle) obj ;
    if (!EqualsUtil.equals(this.position, that.position)) {
      return false ;
    }
    if (!EqualsUtil.equals(this.horizontalAlignment, that.horizontalAlignment)) {
      return false ;
    }
    if (!EqualsUtil.equals(this.verticalAlignment, that.verticalAlignment)) {
      return false ;
    }
    if (!EqualsUtil.equals(this.padding, that.padding)) {
      return false ;
    }
    if (this.notify != that.notify) {
      return false ;
    }
    if (!EqualsUtil.equals(this.toolTipText, that.toolTipText)) {
      return false ;
    }
    if (!EqualsUtil.equals(this.urlText, that.urlText)) {
      return false ;
    }
    if (this.fillBackground != that.fillBackground) {
      return false ;
    }
    if (!EqualsUtil.equals(this.backgroundPaint, that.backgroundPaint)) {
      return false ;
    }
    if (this.backgroundAlpha != that.backgroundAlpha) {
      return false ;
    }
    if (this.borderVisible != that.borderVisible) {
      return false ;
    }
    if (!EqualsUtil.equals(this.borderStroke, that.borderStroke)) {
      return false ;
    }
    if (!EqualsUtil.equals(this.borderPaint, that.borderPaint)) {
      return false ;
    }

    return true ;
  }

  /**
   * Returns a hashcode for the title.
   *
   * @return The hashcode.
   */
  public int hashCode() {
    int result = HashcodeUtil.hashCode(this.position) ;
    result = 37 * result + HashcodeUtil.hashCode(this.horizontalAlignment) ;
    result = 37 * result + HashcodeUtil.hashCode(this.verticalAlignment) ;
    result = 37 * result + HashcodeUtil.hashCode(this.padding) ;
    result = 37 * result + HashcodeUtil.hashCode(this.notify) ;
    result = 37 * result + HashcodeUtil.hashCode(this.toolTipText) ;
    result = 37 * result + HashcodeUtil.hashCode(this.urlText) ;
    result = 37 * result + HashcodeUtil.hashCode(this.fillBackground) ;
    result = 29 * result + HashcodeUtil.hashCode(this.backgroundPaint) ;
    result = 29 * result + (int) (100 * backgroundAlpha) ;
    result = 29 * result + HashcodeUtil.hashCode(this.borderVisible) ;
    result = 29 * result + HashcodeUtil.hashCode(this.borderStroke) ;
    result = 29 * result + HashcodeUtil.hashCode(this.borderPaint) ;

    return result ;
  }

  /**
   * Provides serialization support.
   *
   * @param stream  the output stream.
   *
   * @throws IOException  if there is an I/O error.
   */
  private void writeObject(ObjectOutputStream stream) throws IOException {
    stream.defaultWriteObject() ;

    SerialUtil.writePaint(stream, this.backgroundPaint) ;
    SerialUtil.writeStroke(stream, this.borderStroke) ;
    SerialUtil.writePaint(stream, this.borderPaint) ;
  }

  /**
   * Provides serialization support.
   *
   * @param stream  the input stream.
   *
   * @throws IOException  if there is an I/O error.
   * @throws ClassNotFoundException  if there is a classpath problem.
   */
  private void readObject(ObjectInputStream stream) throws IOException, ClassNotFoundException {
    stream.defaultReadObject() ;

    this.backgroundPaint = SerialUtil.readPaint(stream) ;
    this.borderStroke = SerialUtil.readStroke(stream) ;
    this.borderPaint = SerialUtil.readPaint(stream) ;

    this.listenerList = new EventListenerList() ;
  }

}
