package com.ijchart.xychart.ui ;

import java.awt.GradientPaint ;
import java.awt.Shape ;
import java.awt.geom.Rectangle2D ;
import java.io.Serializable ;

import com.ijchart.xychart.constant.GradientPaintTransformType ;

/**
 * <p>Title: IJChart</p>
 *
 * <p>Description: a chart library for the Java(tm) platform.</p>
 * <zh>GradientPaintTransformTypeGradient任.</zh>
 * <en>Transforms a GradientPaint to range over the width of a target shape.</en>
 *
 * <p>Copyright: Copyright (c) 2013</p>
 *
 * <p>Company: </p>
 *
 * @author zhang feng min,shanghai china.
 * <a href="mailto:ijchartsh@gmail.com">Contract me by email.</a>
 * @version 1.0
 */
public class DefaultGradientPaintTransformer implements Serializable {

  /**
   * <zh>л汾ID.</zh>
   * <en>For serialization.</en>
   */
  private static final long serialVersionUID = -2541729665909339150L ;

  /**
   * <zh>任.</zh>
   * <en>The transform type.</en>
   */
  private GradientPaintTransformType type ;

  /**
   * <zh>һGradient任.</zh>
   * <en>Creates a new transformer.</en>
   */
  public DefaultGradientPaintTransformer() {
    this(GradientPaintTransformType.VERTICAL) ;
  }

  /**
   * <zh>һGradient任.</zh>
   * <en>Creates a new transformer.</en>
   *
   * @param type <zh>任</zh>
   * <en>the transform type.</en>
   */
  public DefaultGradientPaintTransformer(GradientPaintTransformType type) {
    this.type = type ;
  }

  /**
   * <zh>{@link GradientPaint}任.</zh>
   * <en>Transforms a GradientPaint instance.</en>
   *
   * @param paint <zh>ԭʼpaint.</zh>
   * <en>the original paint.</en>
   * @param target <zh>任ͼ</zh>
   * <en>the target shape.</en>
   *
   * @return <zh>任{@link GradientPaint}.</zh>
   * <en>The transformed paint.</en>
   */
  public GradientPaint transform(GradientPaint paint, Shape target) {
    GradientPaint result = paint ;
    final Rectangle2D bounds = target.getBounds2D() ;

    if (this.type.equals(GradientPaintTransformType.VERTICAL)) {
      result = new GradientPaint((float) bounds.getCenterX(), (float) bounds.getMinY(),
                                 paint.getColor1(), (float) bounds.getCenterX(),
                                 (float) bounds.getMaxY(), paint.getColor2()) ;
    } else if (this.type.equals(GradientPaintTransformType.HORIZONTAL)) {
      result = new GradientPaint((float) bounds.getMinX(), (float) bounds.getCenterY(),
                                 paint.getColor1(), (float) bounds.getMaxX(),
                                 (float) bounds.getCenterY(), paint.getColor2()) ;
    } else if (this.type.equals(GradientPaintTransformType.CENTER_HORIZONTAL)) {
      result = new GradientPaint((float) bounds.getCenterX(), (float) bounds.getCenterY(),
                                 paint.getColor1(), (float) bounds.getMaxX(),
                                 (float) bounds.getCenterY(), paint.getColor2(), true) ;
    } else if (this.type.equals(GradientPaintTransformType.CENTER_VERTICAL)) {
      result = new GradientPaint((float) bounds.getCenterX(), (float) bounds.getMinY(),
                                 paint.getColor1(), (float) bounds.getCenterX(),
                                 (float) bounds.getCenterY(), paint.getColor2(), true) ;
    }

    return result ;
  }

  /**
   * <zh>Ƿ.</zh>
   * <en>Tests this instance for equality with an arbitrary object.</en>
   *
   * @param obj <zh>ҪȽϵĶ.</zh>
   * <en>the object to test against (null permitted).</en>
   *
   * @return A boolean.
   */
  public boolean equals(final Object obj) {
    if (obj == this) {
      return true ;
    }

    if (!(obj instanceof DefaultGradientPaintTransformer)) {
      return false ;
    }

    DefaultGradientPaintTransformer that = (DefaultGradientPaintTransformer) obj ;
    return this.type.equals(that.type) ;
  }

  /**
   * <zh>¡˶.</zh>
   * <en>Returns a clone of the transformer.</en>
   *
   * @return A clone.
   *
   * @throws CloneNotSupportedException not thrown by this class, but
   *         subclasses (if any) might.
   */
  public Object clone() throws CloneNotSupportedException {
    return super.clone() ;
  }

  /**
   * <zh>hashcodeֵ.</zh>
   * <en>Returns a hash code for this object.</en>
   *
   * @return A hash code.
   */
  public int hashCode() {
    return (this.type != null) ? this.type.hashCode() : 0 ;
  }

}
