package com.ijchart.xychart.ui ;

import java.awt.Composite ;
import java.awt.Font ;
import java.awt.Graphics2D ;
import java.awt.Paint ;
import java.awt.Stroke ;
import java.awt.geom.AffineTransform ;

/**
 * <p>Title: IJChart</p>
 *
 * <p>Description: a chart library for the Java(tm) platform.</p>
 * <zh>ڴ洢ָǰGraphics2dĸ״̬,:composite,clip shape,paint,stroke,fontȵ.</zh>
 * <en>For storage and restoration of the status of the current Graphics2d,
 * such as: composite,clip shape,paint,stroke,font, and so on.</en>
 *
 * <p>Copyright: Copyright (c) 2013</p>
 *
 * <p>Company: </p>
 *
 * @author zhang feng min,shanghai china.
 * <a href="mailto:ijchartsh@gmail.com">Contract me by email.</a>
 * @version 1.0
 */
public final class Graphics2DState {
  /**
   * <zh>ǰͼλ (Ϊnull).</zh>
   * <en>The current Graphics2D object (null not permitted).</en>
   */
  private Graphics2D g2 ;

  /**
   * <zh>ǰͼλµComposite.</zh>
   * <en>The composite in current Graphics2d.</en>
   */
  private Composite composite ;

  /**
   * <zh>ǰͼλµpaint.</zh>
   * <en>The paint in current Graphics2d.</en>
   */
  private Paint paint ;

  /**
   * <zh>ǰͼλµstroke.</zh>
   * <en>The stroke in current Graphics2d.</en>
   */
  private Stroke stroke ;

  /**
   * <zh>ǰͼλµ.</zh>
   * <en>The font in current Graphics2d.</en>
   */
  private Font font ;

  /**
   * <zh>Ƿ񱣴˾״̬</zh>
   * <en>A flag controls whether or not the old state has saved.</en>
   */
  private boolean isSave ;

  /**
   * <zh>Graphics2DAffineTransform.</zh>
   * <en>The AffineTransform of Graphics2D.</en>
   */
  private AffineTransform transform ;

  /**
   * <zh>ǰͼλ (Ϊnull).</zh>
   * <en>The current Graphics2D object (null not permitted).</en>
   *
   * @param g2 Graphics2D
   */
  public Graphics2DState(Graphics2D g2) {
    if (g2 == null) {
      throw new IllegalArgumentException("Null 'g2' argument.") ;
    }

    this.g2 = g2 ;
    this.isSave = false ;
  }

  /**
   * <zh>ɵ״̬.</zh>
   * <en>Save old state of this Graphics2D.</en>
   */
  public void saveOldState() {
    this.composite = g2.getComposite() ;
    this.paint = g2.getPaint() ;
    this.stroke = g2.getStroke() ;
    this.font = g2.getFont() ;

    this.isSave = true ;
    this.transform = g2.getTransform() ;
  }

  /**
   * <zh>ָɵ״̬.</zh>
   * <en>Restore old state of this Graphics2D.</en>
   */
  public void restoreOldState() {
    if (this.isSave) {
      g2.setComposite(this.composite) ;
      g2.setPaint(this.paint) ;
      g2.setStroke(this.stroke) ;
      g2.setFont(this.font) ;
      g2.setTransform(this.transform) ;
    }
  }

  /**
   * <zh>ָAffineTransform.</zh>
   * <en>Restore AffineTransform only.</en>
   */
  public void restoreTransform() {
    if (this.isSave) {
      g2.setTransform(this.transform) ;
    }
  }
}
