package com.ijchart.xychart.util ;

import java.awt.geom.Arc2D ;
import java.awt.geom.GeneralPath ;
import java.awt.geom.Line2D ;
import java.awt.geom.Point2D ;

/**
 * <p>Title: IJChart</p>
 *
 * <p>Description: a chart library for the Java(tm) platform.</p>
 * <zh>ͼδĹ.</zh>
 * <en>Nice static helpers for geometry.</en>
 *
 * <p>Copyright: Copyright (c) 2013</p>
 *
 * <p>Company: </p>
 *
 * @author zhang feng min,shanghai china.
 * <a href="mailto:ijchartsh@gmail.com">Contract me by email.</a>
 * @version 1.0
 */
public abstract class GeometryUtil {
  /**
   * <zh>֮ľ.</zh>
   * <en>Calculate the distance between two points.</en>
   *
   * @param p1 Point2D <zh>1 (Ϊnull).</zh>
   * <en>The point1 (null not permitted).</en>
   * @param p2 Point2D <zh>1 (Ϊnull).</zh>
   * <en>The point1 (null not permitted).</en>
   *
   * @return double <zh>.</zh>
   * <en>The distince.</en>
   */
  public static double distanceOfPoints(Point2D p1, Point2D p2) {
    if (p1 == null) {
      throw new IllegalArgumentException("Null 'p1' argument") ;
    }
    if (p2 == null) {
      throw new IllegalArgumentException("Null 'p2' argument") ;
    }

    double disX = p2.getX() - p1.getX() ;
    double disY = p2.getY() - p1.getY() ;
    double dis = Math.sqrt(disX * disX + disY * disY) ;

    return dis ;
  }

  /**
   * <zh>е.</zh>
   * <en>Calculate the midpoint of two points.</en>
   *
   * @param p1 Point2D <zh>1 (Ϊnull).</zh>
   * <en>The point1 (null not permitted).</en>
   * @param p2 Point2D <zh>1 (Ϊnull).</zh>
   * <en>The point1 (null not permitted).</en>
   *
   * @return Point2D <zh>е (Ϊnull).</zh>
   * <en>The midpoint (never null).</en>
   */
  public static Point2D middlePoint(Point2D p1, Point2D p2) {
    if (p1 == null) {
      throw new IllegalArgumentException("Null 'p1' argument") ;
    }
    if (p2 == null) {
      throw new IllegalArgumentException("Null 'p2' argument") ;
    }

    double x = (p1.getX() + p2.getX()) / 2 ;
    double y = (p1.getY() + p2.getY()) / 2 ;

    return new Point2D.Double(x, y) ;
  }

  /**
   * <zh>ֱ,ԴstartPointendPointΪ,startPointľdisToStartPointĵ.</zh>
   * <en>Two points where the straight line,with direction from startPoint to endPoint as from
   *  the point startPoint distance disToStartPoint.</en>
   *
   * @param startPoint Point2D <zh>ʼ.</zh>
   * <en>The start point.</en>
   * @param endPoint Point2D <zh>յ.</zh>
   * <en>The end point.</en>
   * @param disToStartPoint double <zh>.</zh>
   * <en>The distince.</en>
   *
   * @return Point2D <zh>.</zh>
   * <en>The point.</en>
   */
  public static Point2D extentPoint(Point2D startPoint, Point2D endPoint, double disToStartPoint) {
    if (startPoint == null) {
      throw new IllegalArgumentException("Null 'startPoint' argument") ;
    }
    if (endPoint == null) {
      throw new IllegalArgumentException("Null 'endPoint' argument") ;
    }

    double disX = endPoint.getX() - startPoint.getX() ;
    double disY = endPoint.getY() - startPoint.getY() ;
    double dis = Math.sqrt(disX * disX + disY * disY) ;
    double sin = (endPoint.getY() - startPoint.getY()) / dis ;
    double cos = (endPoint.getX() - startPoint.getX()) / dis ;
    double deltaX = disToStartPoint * cos ;
    double deltaY = disToStartPoint * sin ;

    return new Point2D.Double(startPoint.getX() + deltaX, startPoint.getY() + deltaY) ;
  }

  /**
   * <zh>ͼβPIE͵,յڵĽǶȺָĻεĽǶȲһ,˴ʵĽǶ.</zh>
   * <en>Calculate the actual angle of arc end point.</en>
   *
   * @param arc Arc2D <zh> (Ϊnull).</zh>
   * <en>The arc (null not permitted).</en>
   *
   * @return double <zh>Ƕ.</zh>
   * <en>The angle.</en>
   */
  public static double calculateArcEndPointAngle(Arc2D arc) {
    double halfArcWidth = 0.5 * arc.getWidth() ;
    double halfArcHeight = 0.5 * arc.getHeight() ;

    double actualMaxAngle = 0.0 ;
    Point2D endPoint = arc.getEndPoint() ;
    double endPointX = endPoint.getX() ;
    double endPointY = endPoint.getY() ;
    double endPointAngle = Math.abs(Math.atan((endPointY - halfArcHeight)
                                              / (endPointX - halfArcWidth)) / Math.PI * 180) ; //յԲߵĶ
    if (endPointX >= halfArcWidth && endPointY <= halfArcHeight) {
      actualMaxAngle = endPointAngle ;
    } else if (endPointX <= halfArcWidth && endPointY <= halfArcHeight) {
      actualMaxAngle = 180 - endPointAngle ;
    } else if (endPointX <= halfArcWidth && endPointY >= halfArcHeight) {
      actualMaxAngle = 180 + endPointAngle ;
    } else if (endPointX >= halfArcWidth && endPointY >= halfArcHeight) {
      actualMaxAngle = 360 - endPointAngle ;
    }

    if (arc.getAngleStart() + arc.getAngleExtent() >= 360) {
      actualMaxAngle += 360 ;
    }
    if (arc.getAngleExtent() < 0) {
      actualMaxAngle -= 360 ;
    }

    return actualMaxAngle ;
  }

  /**
   * <zh>ֱߵĽ,ֱཻ߲򷵻null.</zh>
   * <en>Calculate the cross point of two line2,return null if not intersection.</en>
   *
   * @param line1 Line2D <zh>ֱ1 (Ϊnull).</zh>
   * <en>The first line (null not permitted).</en>
   * @param line2 Line2D <zh>ֱ2 (Ϊnull).</zh>
   * <en>The second line (null not permitted).</en>
   *
   * @return Point2D <zh> (Ϊnull).</zh>
   * <en>The cross point (maybe null).</en>
   */
  public static Point2D findLineCrossPoint(Line2D line1, Line2D line2) {
    if (line1 == null) {
      throw new IllegalArgumentException("Null 'line1' argument") ;
    }
    if (line2 == null) {
      throw new IllegalArgumentException("Null 'line2' argument") ;
    }

    Point2D p1 = line1.getP1(), p2 = line1.getP2(), p3 = line2.getP1(), p4 = line2.getP2() ;
    // calculate differences
    double xD1 = p2.getX() - p1.getX() ;
    double xD2 = p4.getX() - p3.getX() ;
    double yD1 = p2.getY() - p1.getY() ;
    double yD2 = p4.getY() - p3.getY() ;
    double xD3 = p1.getX() - p3.getX() ;
    double yD3 = p1.getY() - p3.getY() ;

    // calculate the lengths of the two lines
    double len1 = Math.sqrt(xD1 * xD1 + yD1 * yD1) ;
    double len2 = Math.sqrt(xD2 * xD2 + yD2 * yD2) ;

    // calculate angle between the two lines.
    double dot = (xD1 * xD2 + yD1 * yD2) ; // dot product
    double deg = dot / (len1 * len2) ;

    // if abs(angle)==1 then the lines are parallell,
    // so no intersection is possible
    if (Math.abs(deg) == 1) {
      return null ;
    }

    // find intersection Pt between two lines
    double div = yD2 * xD1 - xD2 * yD1 ;
    double ua = (xD2 * yD3 - yD2 * xD3) / div ;
    Point2D pt = new Point2D.Double(p1.getX() + ua * xD1, p1.getY() + ua * yD1) ;

    // calculate the combined length of the two segments
    // between Pt-p1 and Pt-p2
    xD1 = pt.getX() - p1.getX() ;
    xD2 = pt.getX() - p2.getX() ;
    yD1 = pt.getY() - p1.getY() ;
    yD2 = pt.getY() - p2.getY() ;
    double segmentLen1 = Math.sqrt(xD1 * xD1 + yD1 * yD1) + Math.sqrt(xD2 * xD2 + yD2 * yD2) ;

    // calculate the combined length of the two segments
    // between Pt-p3 and Pt-p4
    xD1 = pt.getX() - p3.getX() ;
    xD2 = pt.getX() - p4.getX() ;
    yD1 = pt.getY() - p3.getY() ;
    yD2 = pt.getY() - p4.getY() ;
    double segmentLen2 = Math.sqrt(xD1 * xD1 + yD1 * yD1) + Math.sqrt(xD2 * xD2 + yD2 * yD2) ;

    // if the lengths of both sets of segments are the same as
    // the lenghts of the two lines the point is actually
    // on the line segment.

    // if the point isn't on the line, return null
    if (Math.abs(len1 - segmentLen1) > 0.01 || Math.abs(len2 - segmentLen2) > 0.01) {
      return null ;
    }

    // return the valid intersection
    return pt ;
  }

  /**
   * <zh>arc·.</zh>
   * <en>Calculate the path of arc.</en>
   *
   * @param arc Arc2D <zh> (Ϊnull).</zh>
   * <en>The arc (null not permitted).</en>
   *
   * @return GeneralPath
   */
  public static GeneralPath getArcPath(Arc2D arc) {
    if (arc == null) {
      throw new IllegalArgumentException("Null 'arc' argument.") ;
    }

    double angleStart = arc.getAngleStart() ; //ʼǶ
    double angleExtent = Math.min(arc.getAngleExtent(), 360) ;
    double arcWidth = arc.getWidth() ;
    double arcHeight = arc.getHeight() ;
    Arc2D newArc = new Arc2D.Double(0.0D, 0.0D, arcWidth, arcHeight,
                                    angleStart, angleExtent, arc.getArcType()) ;

    //ê
    double halfArcWidth = 0.5 * arc.getWidth() ;
    double halfArcHeight = 0.5 * arc.getHeight() ;

    double drawAngle = angleStart ;
    int sign = MathUtil.sign(angleExtent) ;
    final double tickSizeAngle = 5.0F * sign ; //in degree
    double actualMaxAngle = GeometryUtil.calculateArcEndPointAngle(newArc) ;

    int anchorCount = 0 ; //êĸ
    Point2D startPoint = newArc.getStartPoint() ;
    Point2D endPoint = newArc.getEndPoint() ;
    double endX = 0, endY = 0 ; //

    GeneralPath path = new GeneralPath() ;
    path.moveTo((float) startPoint.getX(), (float) startPoint.getY()) ;

    while (true) { //ڿ̶֮
      double radians = drawAngle / 180 * Math.PI ;
      double tanAngle = Math.tan(radians) ;
      int quadrant = AngleUtil.getQuadrantByDegreeAngle(drawAngle) ;

      if (anchorCount == 0) {
        endX = startPoint.getX() ;
        endY = startPoint.getY() ;
      } else {
        double x = Math.sqrt(1 / (1 / Math.pow(halfArcWidth, 2)
                                  + Math.pow(tanAngle, 2) / Math.pow(halfArcHeight, 2))) ;
        double y = Math.abs(x * tanAngle) ;

        if (quadrant == 1 || quadrant == 4) {
          endX = halfArcWidth + x ;
        } else {
          endX = halfArcWidth - x ;
        }
        if (quadrant == 1 || quadrant == 2) {
          endY = halfArcHeight - y ;
        } else {
          endY = halfArcHeight + y ;
        }
      }

      path.lineTo((float) endX, (float) endY) ;
      drawAngle += tickSizeAngle ;
      anchorCount++ ;

      if ((sign == 1 && drawAngle > actualMaxAngle) || (sign == -1 && drawAngle < actualMaxAngle)) {
        if (anchorCount >= 2) {
          break ;
        } else {
          drawAngle = actualMaxAngle ;
        }
      }
    }
    path.lineTo((float) endPoint.getX(), (float) endPoint.getY()) ;

    return path ;
  }
}
