package com.ijchart.xychart.util ;

import com.ijchart.xychart.data.XYNumberChartData ;
import com.ijchart.xychart.dataset.IMatrixDataset ;

/**
 * <p>Title: IJChart</p>
 *
 * <p>Description: a chart library for the Java(tm) platform.</p>
 * <zh>ϻعߵĹ.</zh>
 * <en>A utility class for fitting regression curves to data.</en>
 * <p>Copyright: Copyright (c) 2013</p>
 *
 * <p>Company: </p>
 *
 * @author zhang feng min,shanghai china.
 * <a href="mailto:ijchartsh@gmail.com">Contract me by email.</a>
 * @version 1.0
 */
public abstract class RegressionUtil {

  /**
   * <zh>y=a+bx̵Ĳ'a''b',ͨС˻ع.ص,result[0]'a'
   * result[1]'b'.</zh>
   * <en>Returns the parameters 'a' and 'b' for an equation y = a + bx, fitted to
   * the data using ordinary least squares regression.The result is
   * returned as a double[], where result[0] is a, and result[1] is b.</en>
   *
   * @param data double[][] <zh>.</zh>
   * <en>The datas.</en>
   *
   * @return double[] <zh>result[0]a,result[1]b.</zh>
   * <en>result[0] is 'a',and result[1] is 'b'.</en>
   */
  public static double[] getOLSRegression(double[][] data) {
    int n = data.length ;
    if (n < 2) {
      throw new IllegalArgumentException("Not enough data.") ;
    }

    double sumX = 0 ;
    double sumY = 0 ;
    double sumXX = 0 ;
    double sumXY = 0 ;
    for (int i = 0 ; i < n ; i++) {
      double x = data[i][0] ;
      double y = data[i][1] ;
      sumX += x ;
      sumY += y ;
      double xx = x * x ;
      sumXX += xx ;
      double xy = x * y ;
      sumXY += xy ;
    }

    double sxx = sumXX - (sumX * sumX) / n ;
    double sxy = sumXY - (sumX * sumY) / n ;
    double xbar = sumX / n ;
    double ybar = sumY / n ;

    double[] result = new double[2] ;
    result[1] = sxy / sxx ;
    result[0] = ybar - result[1] * xbar ;

    return result ;
  }

  /**
   * <zh>y=a+bx̵Ĳ'a''b',ͨС˻ع.ص,result[0]'a'
   * result[1]'b'.</zh>
   * <en>Returns the parameters 'a' and 'b' for an equation y = a + bx, fitted to
   * the data using ordinary least squares regression.The result is
   * returned as a double[], where result[0] is a, and result[1] is b.</en>
   *
   * @param dataset IMatrixDataset <zh>ݼ,ݼÿҪ
   * {@link com.ijchart.xychart.data.XYNumberChartData}. (Ϊnull).</zh>
   * <en>The dataset,and each data is
   * {@link com.ijchart.xychart.data.XYNumberChartData} object in dataset. (null not permitted).</en>
   * @param rowIdx int <zh>,ڵ0.</zh>
   * <en>The row index,must be greater than or equal to 0.</en>
   *
   * @return double[] <zh>result[0]a,result[1]b.</zh>
   * <en>result[0] is 'a',and result[1] is 'b'.</en>
   */
  public static double[] getOLSRegression(IMatrixDataset dataset, int rowIdx) {
    if (dataset == null) {
      throw new IllegalArgumentException("Null 'dataset' argument") ;
    }
    if (rowIdx < 0) {
      throw new IllegalArgumentException("Argument 'rowIdx' must be >=0.") ;
    }

    int cols = dataset.getColumnCount(rowIdx) ;
    if (cols < 2) {
      throw new IllegalArgumentException("Not enough data.") ;
    }

    double sumX = 0 ;
    double sumY = 0 ;
    double sumXX = 0 ;
    double sumXY = 0 ;
    for (int i = 0 ; i < cols ; i++) {
      Object obj = dataset.getData(rowIdx, i) ;
      if (obj == null) {
        continue ;
      }
      if (!(obj instanceof XYNumberChartData)) {
        throw new IllegalArgumentException("Data must be 'XYNumberChartData'.") ;
      }

      XYNumberChartData xyData = (XYNumberChartData) obj ;
      double x = xyData.getX() ;
      double y = xyData.getY() ;
      if (Double.isNaN(x) || Double.isNaN(y)) {
        continue ;
      }

      sumX += x ;
      sumY += y ;
      double xx = x * x ;
      sumXX += xx ;
      double xy = x * y ;
      sumXY += xy ;
    }

    double sxx = sumXX - (sumX * sumX) / cols ;
    double sxy = sumXY - (sumX * sumY) / cols ;
    double xbar = sumX / cols ;
    double ybar = sumY / cols ;

    double[] result = new double[2] ;
    result[1] = sxy / sxx ;
    result[0] = ybar - result[1] * xbar ;

    return result ;
  }

  /**
   * <zh>y=ax^b̵Ĳ'a''b',ݻع.ص,result[0]'a'
   * result[1]'b'.</zh>
   * <en>Returns the parameters 'a' and 'b' for an equation y=ax^b, fitted to
   * the data using a power regression equation.The result is returned as
   * an array, where double[0] is a, and double[1] is b.</en>
   *
   * @param data double[][] <zh>.</zh>
   * <en>The datas.</en>
   *
   * @return double[] <zh>result[0]a,result[1]b.</zh>
   * <en>result[0] is 'a',and result[1] is 'b'.</en>
   */
  public static double[] getPowerRegression(double[][] data) {
    int n = data.length ;
    if (n < 2) {
      throw new IllegalArgumentException("No enough data.") ;
    }

    double sumX = 0 ;
    double sumY = 0 ;
    double sumXX = 0 ;
    double sumXY = 0 ;
    for (int i = 0 ; i < n ; i++) {
      double x = Math.log(data[i][0]) ;
      double y = Math.log(data[i][1]) ;
      sumX += x ;
      sumY += y ;
      double xx = x * x ;
      sumXX += xx ;
      double xy = x * y ;
      sumXY += xy ;
    }

    double sxx = sumXX - (sumX * sumX) / n ;
    double sxy = sumXY - (sumX * sumY) / n ;
    double xbar = sumX / n ;
    double ybar = sumY / n ;

    double[] result = new double[2] ;
    result[1] = sxy / sxx ;
    result[0] = Math.pow(Math.exp(1.0), ybar - result[1] * xbar) ;

    return result ;
  }

  /**
   * <zh>y=ax^b̵Ĳ'a''b',ݻع.ص,result[0]'a'
   * result[1]'b'.</zh>
   * <en>Returns the parameters 'a' and 'b' for an equation y=ax^b, fitted to
   * the data using a power regression equation.The result is returned as
   * an array, where double[0] is a, and double[1] is b.</en>
   *
   * @param dataset IMatrixDataset <zh>ݼ,ݼÿҪ
   * {@link com.ijchart.xychart.data.XYNumberChartData}. (Ϊnull).</zh>
   * <en>The dataset,and each data is
   * {@link com.ijchart.xychart.data.XYNumberChartData} object in dataset. (null not permitted).</en>
   * @param rowIdx int <zh>,ڵ0.</zh>
   * <en>The row index,must be greater than or equal to 0.</en>
   *
   * @return double[] <zh>result[0]a,result[1]b.</zh>
   * <en>result[0] is 'a',and result[1] is 'b'.</en>
   */
  public static double[] getPowerRegression(IMatrixDataset dataset, int rowIdx) {
    int n = dataset.getColumnCount(rowIdx) ;
    if (n < 2) {
      throw new IllegalArgumentException("No enough data.") ;
    }

    double sumX = 0 ;
    double sumY = 0 ;
    double sumXX = 0 ;
    double sumXY = 0 ;
    for (int i = 0 ; i < n ; i++) {
      Object obj = dataset.getData(rowIdx, i) ;
      if (obj == null) {
        continue ;
      }
      if (!(obj instanceof XYNumberChartData)) {
        throw new IllegalArgumentException("Data must be 'XYNumberChartData'.") ;
      }

      XYNumberChartData xyData = (XYNumberChartData) obj ;
      double x = Math.log(xyData.getX()) ;
      double y = Math.log(xyData.getY()) ;
      if (Double.isNaN(x) || Double.isNaN(y)) {
        continue ;
      }

      sumX += x ;
      sumY += y ;
      double xx = x * x ;
      sumXX += xx ;
      double xy = x * y ;
      sumXY += xy ;
    }

    double sxx = sumXX - (sumX * sumX) / n ;
    double sxy = sumXY - (sumX * sumY) / n ;
    double xbar = sumX / n ;
    double ybar = sumY / n ;

    double[] result = new double[2] ;
    result[1] = sxy / sxx ;
    result[0] = Math.pow(Math.exp(1.0), ybar - result[1] * xbar) ;

    return result ;
  }

}
